package gov.va.med.hac.edi.era.jpa;

import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;

import java.io.*;
import java.util.logging.*;
import javax.naming.NamingException;
import javax.persistence.EntityManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class holds the logic that enables the staging tables to be loaded from
 * the .DAT file that contains stage data.
 * 
 * The process is run from the command line and needs to be passed the location
 * of the data file.
 * 
 * It delegates loading the data off to the processLoad method of
 * ProcessHacStg835Load (the workhorse of this process.) The main method in this
 * class contains a last resort error handler that logs any errors that occur
 * during processing.
 * 
 * @author vhahacpennet
 * 
 */
public final class HacStg835Load {
	
	static final Logger summaryLOGGER = LoggerFactory.getLogger(HacStg835Load.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + HacStg835Load.class);
	
	/**
	 * This is the main entry point of HacStg835Load. The process can be called
	 * from the command line. It takes one parameter - the full path (including
	 * the filename) of the file that contains data to put in the staging
	 * repository.
	 * 
	 * @param args
	 *            - 1 arg - the full path and filename of the data file.
	 */
	public static void main(final String[] args) {

		// This flag keeps track the the state of the process. It allows
		// a status message to be printed after the process runs that indicates
		// if the process ran successfully or not
		boolean processSuccess = false;
		EntityManager dbEM = null;

		// Parse the command line
		CommandLine line = null;
		Options opts = getCliOptions();
		try {
			CommandLineParser parser = new GnuParser();
			line = parser.parse(opts, args);
			// If the "help" option is specified, just show usage options and
			// exit
			if (line.hasOption("h")) {
				printUsage(opts, args, false);
				System.exit(1);
			}
			// If any required command line parameter is missing, just show
			// usage options and exit
		} catch (MissingOptionException moe) {
			printUsage(opts, args, true);
			System.exit(1);
		} catch (ParseException pe) {
			System.out.println("Problem parsing the command line");
			summaryLOGGER.error("Problem parsing the command line");
			traceLOGGER.error("Problem parsing the command line");
			printUsage(opts, args, false);
			System.exit(1);
		}

		// Last resort error handler
		// Catch any exception (outside of setting up for logging)
		try {

			LogUtils.stampStartingTime("HacStg835Load Start", summaryLOGGER);
			summaryLOGGER.info("HacStg835Load Start");
			summaryLOGGER.info("Code Version: HIPAA II Project");
			traceLOGGER.info("HacStg835Load Start");
			traceLOGGER.info("Code Version: Refactoring Project");

			File configFile = null;
			String configFilePath = line.getOptionValue("f");
			if (configFilePath != null && configFilePath.length() > 0) {
				configFile = JPAUtil.getJpaConfigFile(configFilePath.trim());
			} else {
				configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
			}

			String inputFileName = line.getOptionValue("i");
			if(inputFileName != null)
			{
				inputFileName = inputFileName.trim();
			}
			if (inputFileName == null || inputFileName.length() < 1) {
				summaryLOGGER
						.info("HacStg835Load: This program requires an input file name"
								+ " as a command line argument");
				traceLOGGER
				.info("HacStg835Load: This program requires an input file name"
						+ " as a command line argument");
			} else {
				// Set up an entity manager using the hibernate properties in
				// dbhibernate.properties
				// You may change the properties set in this file to point at a
				// different schema, log sql statements, etc.
				// This code also uses the persistence unit named stg835Load
				// (defined in persistance.xml),
				// this is where JPA looks to see if it should persist a class
				// or
				// not. JPA also uses this
				// info to see how it should map a class to the database
				// see stg835LoadMappings.xml for more details
				// For JPA to connect to the database "dbhibernate.properties"
				// should contain the following properties
				// hibernate.connection.username, hibernate.connection.password,
				// &
				// hibernate.connection.url

				dbEM = JPAUtil.getEntityManager(configFile, "stg835Load");
			
				final ProcessHacStg835Load pStg835Load = new ProcessHacStg835Load();
				
				// input file not found when name is passed
				// location is resources/persistence.xml 
				pStg835Load.processLoad(inputFileName, dbEM);

				summaryLOGGER.info("Process completed successfully");
				traceLOGGER.info("Process completed successfully");
				LogUtils.stampEndingTime("HacStg835Load", summaryLOGGER);
				processSuccess = true;
			}
		} catch (OutOfMemoryError e) {
			summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
			traceLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
		} catch (Throwable e) {
			// Catch all to log any unexpected exceptions
			summaryLOGGER.error("Exception in processing", e);
			traceLOGGER.error("Exception in processing", e);
		} finally {
			if (!processSuccess) {
				summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
				traceLOGGER.error("!!!!! Process seems to have failed !!!!!");
			}
		}
	}

	private static void printUsage(Options opts, String[] inArgs,
			boolean missingArg) {
		final int helpwidth = 80;
		HelpFormatter fmtr = new HelpFormatter();
		System.out.println();
		if (missingArg) {
			System.out
					.println("Some required commandline argument was missing");
		}
		fmtr.printHelp(helpwidth, "HacStg835Load", "", opts, "", true);
		if (missingArg) {
			System.out.println(inArgs.length
					+ " command line arguments were received: ");
			for (int i = 0; i < inArgs.length; i++) {
				System.out.println(inArgs[i]);
			}
		}
	}

	/**
	 * Helper method to populate command line options objects.
	 * 
	 * @return the <code>Options</code> object containing the set of command
	 *         line option.
	 */
	@SuppressWarnings("static-access")
	private static Options getCliOptions() {
		Options options = new Options();

		Option configfile = OptionBuilder
				.withArgName("jpaFile")
				.hasArg()
				.withLongOpt("jpaFile")
				.withDescription(
						"The JPA Configuration file, such as hibernate.properties.")
				.create("f");

		Option fileName = OptionBuilder
				.withArgName("fileName")
				.hasArgs(1)
				.isRequired()
				.withLongOpt("fileName")
				.withDescription(
						"The absolute path, including the filename, of the input file that contains data to be loaded into the staging tables.")
				.create("i");

		Option help = OptionBuilder.withLongOpt("help").withDescription(
				"Print this message.").create("h");

		options.addOption(help);
		options.addOption(configfile);
		options.addOption(fileName);

		return options;
	}
}