package gov.va.med.hac.edi.extract.b2.jpa;

import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.logutils.jpa.*;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.io.*;
import java.util.*;

import gov.va.med.domain.erepos.*;

public class ProcessRxExtractB2 {
  static final Logger LOGGER = LoggerFactory.getLogger(ProcessRxExtractB2.class);
  static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + ProcessRxExtractB2.class);

  protected void processReversalsExtract(String targetFilePath, String vSenderId, EntityManager dbEM)
      throws DbTransactionEnderException {
    List listOfClaimsToPersist = new ArrayList();

    try {
      // Get a list of the batch keys for new B2 (reversal) claims for this sender
      Query batchKeysQuery = dbEM.createNativeQuery(
          "SELECT DISTINCT BATCH_KEY "
              + "FROM NCPDP_CLAIM "
              + "WHERE STATUS = 'NEW' "
              + "AND CLAIM_TYPE = 'B2' "
              + "AND SENDER_ID = :senderId");
      batchKeysQuery.setParameter("senderId", vSenderId);
      List batchKeyList = batchKeysQuery.getResultList();
      int batchesCount = batchKeyList.size();
      LOGGER.info("Batches with NEW B2 claims for this sender: " + batchesCount);

      // Loop through the batches, processing them
      Iterator batchKeyIterator = batchKeyList.iterator();
      String nextBatchKey = "";
      if (batchesCount > 0) {
        while (batchKeyIterator.hasNext()) {
          // Get and log the next batch key
          nextBatchKey = (String) batchKeyIterator.next();
          LOGGER.info("Batch Key:" + nextBatchKey);

          // Get the batch corresponding to this key
          Query getBatchQuery = dbEM.createQuery("SELECT b "
                                               + "FROM gov.va.med.domain.erepos.NcpdpBatch b "
                                               + "WHERE b.batchKey=?1 ");
          getBatchQuery.setParameter(1, nextBatchKey);
          NcpdpBatch rowNcpdpBatch = (NcpdpBatch) getBatchQuery.getSingleResult();
          traceLOGGER.trace("Batch as queried : {}", new NcpdpBatchLogger(rowNcpdpBatch));
          
          if (rowNcpdpBatch != null) {

            // Put together an output file name
            String vFileNumber = rowNcpdpBatch.getFileNumber();
            File ExtFile = new File(targetFilePath + vSenderId + "_" + vFileNumber);
            PrintWriter vExtFile = new PrintWriter(new FileOutputStream(ExtFile));

            // Process all the reversal records for this batch
            listOfClaimsToPersist.addAll(createReversalRec(rowNcpdpBatch, vExtFile, dbEM));
            vExtFile.flush();
            vExtFile.close();
          }
        }

        // Save the reversals because their status has changed
        traceLOGGER.trace("Committing the database transaction");
        dbEM.getTransaction().begin();
        Iterator claimIterator = listOfClaimsToPersist.iterator();
        while (claimIterator.hasNext()) {
          dbEM.persist(claimIterator.next());
        }
        dbEM.getTransaction().commit();
        traceLOGGER.trace("Commit successful");
        LOGGER.info("Reversal Extract File for " + vSenderId + " was Successfully Created.");
      } else {
        LOGGER.info("There are no new NCPDP B2 Claims for " + vSenderId + " to be extracted.");
      }
    } catch (Exception ex) {
      // Validation for rollbacking the database
      if (dbEM != null && dbEM.getTransaction().isActive()) {
        LOGGER.debug("Rolling back the database transaction");
        dbEM.getTransaction().rollback();
        LOGGER.debug("Rollback successful");
      }
      throw new DbTransactionEnderException("Rolled Back the Transaction", ex);
    }
  }

  private List createReversalRec(NcpdpBatch rowNcpdpBatch,
      PrintWriter vExtFile, EntityManager dbEM) {
    List listOfClaimsToPersist = new ArrayList();
    StringBuffer reversalOutput = new StringBuffer();

    // Go through all the headers for this batch
    Iterator headerIterator = rowNcpdpBatch.getNcpdpHeaders().iterator();
    while (headerIterator.hasNext()) {
      NcpdpHeader rowNcpdpHeader = (NcpdpHeader) headerIterator.next();
      traceLOGGER.trace("Header as queried: {}", new NcpdpHeaderLogger(rowNcpdpHeader));

      NcpdpInsurance rowInsurance
        = (NcpdpInsurance) rowNcpdpHeader.getNcpdpInsurances().iterator().next();
      traceLOGGER.trace("Insurance as queried: {}", new NcpdpInsuranceLogger(rowInsurance));

      //Patient Segment is not used in NCPDP D.0/3.0 formats for Reversal B2 transaction
      //NcpdpPatient rowPatient = (NcpdpPatient) rowNcpdpHeader.getNcpdpPatients().iterator().next();
      //traceLOGGER.trace("Patient as queried: {}", new NcpdpPatientLogger(rowPatient));

      // Get the reversal info for this header
      Query reversalsQuery = dbEM.createQuery(
          "SELECT c "
        + "FROM gov.va.med.domain.erepos.NcpdpClaim c "
        + "WHERE c.ncpdpHeader=?  "
        + "AND c.status =? "
        + "AND c.claimType =? "
        + "ORDER BY c.dateCreated");
      reversalsQuery.setParameter(1, rowNcpdpHeader);
      reversalsQuery.setParameter(2, "NEW");
      reversalsQuery.setParameter(3, "B2");
      List claims = reversalsQuery.getResultList();
      
      // Loop through the claims for this header
      // TODO:  Isn't it true that there can only be one?
      Iterator claimsIterator = claims.iterator();
      while (claimsIterator.hasNext()) {
        NcpdpClaim rowReversal = (NcpdpClaim) claimsIterator.next();
        traceLOGGER.trace("Claim as queried: {}", new NcpdpClaimLogger(rowReversal));
        
        // Put together the output row
        reversalOutput.append(StringUtils.rPadSpaces(rowNcpdpBatch.getCreationDate(), 8));
        reversalOutput.append(StringUtils.rPadSpaces(rowNcpdpHeader.getDateOfService(), 8));
        //reversalOutput.append(StringUtils.rPadSpaces(rowReversal.getRxSvcRefNum(), 7));
        //Rx # is updated to 12 characters for D.0 format
        reversalOutput.append(StringUtils.rPadSpaces(rowReversal.getRxSvcRefNum(), 12));
        reversalOutput.append(StringUtils.rPadSpaces(rowReversal.getProductSvcId(), 14));
        reversalOutput.append(StringUtils.rPadSpaces(rowReversal.getQtyPrescribed(), 5));
        reversalOutput.append(StringUtils.rPadSpaces(rowReversal.getDaysSupply(), 4));
        reversalOutput.append(StringUtils.rPadSpaces("", 16));
        reversalOutput.append(StringUtils.rPadSpaces("", 7));
        reversalOutput.append(StringUtils.rPadSpaces(rowInsurance.getCrdhldrId(), 9));
        /*
         * In NCPDP D.0 standard Patient segment is not used. Instead we are using
         * Last and First Names from Insurance Segment
        String vName = rowPatient.getPatLastName();
        if (vName.length() > 20) {
          vName = vName.substring(0, 20);
        }
        reversalOutput.append(StringUtils.rPadSpaces(vName, 20));
        vName = rowPatient.getPatFirstName();
        if (vName.length() > 15) {
          vName = vName.substring(0, 15);
        }*/
        
        String vName = rowInsurance.getCrdhldrLastName();
        if (vName.length() > 15) {
          vName = vName.substring(0, 15);
        }
        reversalOutput.append(StringUtils.rPadSpaces(vName, 20));
        vName = rowInsurance.getCrdhldrFirstName();
        if (vName.length() > 12) {
          vName = vName.substring(0, 12);
        }

        reversalOutput.append(StringUtils.rPadSpaces(vName, 15));
        //reversalOutput.append(StringUtils.rPadSpaces(rowPatient.getDateOfBirth(), 8));
        // There is NO Date of Birth field in Insurance Segment
        reversalOutput.append(StringUtils.rPadSpaces("", 8));
        reversalOutput.append(StringUtils.rPadSpaces("C/" + rowReversal.getOrgPrsPrdSvcCd(), 20));
        reversalOutput.append(StringUtils.rPadSpaces("", 8));
        reversalOutput.append(StringUtils.rPadSpaces(rowNcpdpHeader.getTransactionRefNum(), 10));
        traceLOGGER.trace("ReversalRec as written: {}", reversalOutput);
        
        // Output this reversal to the file
        vExtFile.println(reversalOutput);
        vExtFile.flush();
        
        reversalOutput.delete(0, reversalOutput.length());
        rowReversal.setStatus("PROCESSED");
        listOfClaimsToPersist.add(rowReversal);
      }
    }
    reversalOutput.append(StringUtils.rPadSpaces("XXXXXXXX", 159));
    vExtFile.println(reversalOutput);
    reversalOutput.delete(0, reversalOutput.length());
    return listOfClaimsToPersist;
  }

}