package gov.va.med.hac.edi.extract.b2.jpa;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;

import java.io.*;
import javax.persistence.EntityManager;

import org.apache.commons.cli.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author HIPAA-READY
 * 
 */
public class RxExtractB2
{
  // Use two loggers, one for a summary file and one for a trace file.
  static final Logger summaryLOGGER = LoggerFactory.getLogger(RxExtractB2.class);
  static final Logger traceLOGGER   = LoggerFactory.getLogger("trace." + RxExtractB2.class);

  // private constructor to prevent unneeded instance creation.
  private RxExtractB2() {
  }

  /**
   * Entry point for the RxExtractB2 process. The JPA connection configuration file name can be
   * passed into this method which will override the normal persistence.xml file found in the
   * META-INF directory of the jar file. <br>
   * <br>
   * <code>usage: RxExtractB2 [-f &lt;file&gt;] [-h]</code>
   * 
   * @param args
   *          set of options where &lt;file&gt; is the full path name of the JPA configuration file.
   */
  public static void main(String[] args)
  {
    // Parse the command line
    CommandLine line = null;
    Options opts = getCliOptions();
    try {
      CommandLineParser parser = new GnuParser();
      line = parser.parse(opts, args);
      // If the "help" option is specified, just show usage options and exit
      if (line.hasOption("h")) {
        printUsage(opts, args, false);
        System.exit(1);
      }
      // If any required command line parameter is missing, just show usage options and exit
    } catch (MissingOptionException moe) {
      printUsage(opts, args, true);
      System.exit(1);
    } catch (ParseException pe) {
      System.out.println("Problem parsing the command line");
      printUsage(opts, args, false);
      System.exit(1);
    }

    // Local variables
    boolean processSuccess = false;
    String destinationPath = "";
    String senderId = "";
    String configFilePath = "";
    EntityManager em = null;
        
    // Catch exceptions, log them, and swallow them to allow other things to run afterwards
    try {
      // Put an origination message into the log file
      LogUtils.stampStartingTime("Rx Extract B2 Start", summaryLOGGER);
      summaryLOGGER.info("Code Version: Crossover Infrastructure Project");


      // Get an EntityManager, based on the -f option if passed in, otherwise based on a default
      File configFile = null;
      configFilePath = line.getOptionValue("f");
      if (configFilePath != null && configFilePath.length() > 0) {
        configFile = JPAUtil.getJpaConfigFile(configFilePath);
      } else {
        configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
      }
      em = JPAUtil.getEntityManager(configFile, "b1b2Extract");

      // If parameters are OK, run this process
      destinationPath = line.getOptionValue("p");
      senderId = line.getOptionValue("s");
      if (destinationPath != null && destinationPath.length() > 0 && senderId != null
          && senderId.length() > 0) {
        summaryLOGGER.info("Destination Path from parameters: " + destinationPath);
        summaryLOGGER.info("Sender Id from parameters: " + senderId);
        ProcessRxExtractB2 pExtractB2 = new ProcessRxExtractB2();
        pExtractB2.processReversalsExtract(destinationPath, senderId, em);

      } else {
        summaryLOGGER.error("RxExtractB2: Please Pass both Parameters");
      }

      // Put a completion message into the log file
      summaryLOGGER.info("Process completed successfully");
      LogUtils.stampEndingTime("RxExtractB2 process completed",
          summaryLOGGER);

      processSuccess = true;

    } catch (OutOfMemoryError e) {
      summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);

    } catch (Exception e) {
      summaryLOGGER.error("Exception in processing", e);

    } finally {
      if (!processSuccess) {
        summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
      }
    }

  }

  /**
   * Helper method to populate command line options objects.
   * 
   * @return the <code>Options</code> object containing the set of command line option.
   */
  @SuppressWarnings("static-access")
  private static Options getCliOptions() {
    Options options = new Options();

    Option configfile = OptionBuilder.withArgName("jpaFile").hasArg().withLongOpt("jpaFile")
        .withDescription("The JPA Configuration file, such as hibernate.properties.").create("f");

    Option destinationPath = OptionBuilder.withArgName("destinationPath").hasArg().isRequired().
        withLongOpt("destinationPath").withDescription("The path to output files.").create("p");

    Option senderId = OptionBuilder.withArgName("senderId").hasArg().isRequired().
        withLongOpt("senderId").withDescription(
            "The identifier of the Pharmacy Benefits Manager "
                + "whose claims should be extracted.").create("s");

    Option help = OptionBuilder.withLongOpt("help").withDescription("Print this message.").create(
        "h");

    options.addOption(help);
    options.addOption(configfile);
    options.addOption(destinationPath);
    options.addOption(senderId);

    return options;
  }

  private static void printUsage(Options opts, String[] inArgs, boolean missingArg) {
    final int helpwidth = 80;
    HelpFormatter fmtr = new HelpFormatter();
    System.out.println();
    if (missingArg) {
      System.out.println("Some required commandline argument was missing");
    }
    fmtr.printHelp(helpwidth, "RxExtractB2", "", opts, "", true);
    if (missingArg) {
      System.out.println(inArgs.length + " command line arguments were received: ");
      for (int i = 0; i < inArgs.length; i++) {
        System.out.println(inArgs[i]);
      }
    }
  }
}