package gov.va.med.hac.edi.extract.x12.jpa;
 
import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.DateUtil;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;

import java.util.*;
import java.io.*;
import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The overall goal of this extract is to take x12 data that is in the eRepository database and turn
 * it into a flat file representation that can be loaded by the Cache system.
 * 
 * The data in the database represents health care claims, submitted on behalf of someone who had
 * medical services performed on them (a subscriber), by a billing provider (i.e. someone who bills
 * - they may or may not be the medical service provider). This information is hierarchical in
 * nature. This process creates a flat file representation of that hierarchy so the data can be
 * ingested into the Cache system.
 * 
 * To accomplish this the data to is written into an extract file as segments of data. This data
 * structure makes it possible to identify and relate the participants involved in a health care
 * transaction hierarchically in the flat file structure.
 * 
 * For example: Business partners must interchange data electronically to adjudicate a claim. To
 * adjudicate an 837 Professional transaction the business data might contain information that
 * identifies and relates a billing/pay-to provider, a subscriber, and a patient. Therefore, a file
 * that could support adjudication of this claim might contain the following types of information:
 * 
 * The first segment of data would identify the partners that are interchanging data. This
 * "technical" segment would -wrap- business data so that the data could be exchanged. The next
 * levels in the hierarchy, underneath the envelope of data "might" represent and relate the
 * following:
 * 
 * The Billing provider (HL03=20) <- who is submitting the claim The Subscriber (HL03=22) <-on
 * behalf of whom Claim level information <- what services did this subscriber need Line level
 * information <- more information related to the claim
 * 
 * A logical view of information in an extract file might look like:
 * 
 * Start Envelope Start Billing Provider Start Claim Data End Claim Data End Billing Provider End
 * Envelope
 * 
 * The code in this process basically mines for claim related information and writes it into the
 * flat file one line at a time.
 * 
 * Each line written to the flat file starts with an identifier that denotes the type of line
 * written to the file. Each level in the hierarchy has certain line types it may contain. For
 * example, the billing providers name, address, and other information appear on a line starting
 * with the number 025. See "eRepos to HAC Flat File.xls" for more details.
 * 
 * Since each claim type -- Professional, Institutional, and Dental -- writes different lines to the
 * extract file, the process breaks writing the extract file into three different logic flows that
 * produce three separate extract files. To elaborate, although a dental claim may require
 * information about the tooth the dentist worked on a professional claim might not contain this
 * information. The three files produced each have their own suffix _den_ - for dental claims, _ins_
 * for institutional, _opt_-professional (out patient)
 * 
 * The "driving" logic flows that produce the files are contained in three different java classes
 * ExtractDntlClaims, ExtractInstClaims and ExtractPrfsnClaims. These files contain specific logic
 * that enables dental, institutional, and professional claims to be written to the extract file
 * respectively. Each one of these java classes is a subclass of ProcessCacheRecords.java.
 * 
 * ProcessCacheRecords contains logic used to write the top level envelope, discussed earlier, as
 * well as common logic -- For example, logic to to query the system for an address and write it to
 * the flat file, or a patient's name, etc.
 * 
 * In each of these classes, methods that start with the prefix "process" are used to collect data
 * from the database and drive the writing of sections of data, and methods that start with the
 * prefix "write" are used to write information to the flat file (based on the eRepos to HAC Flat
 * File.xls specification).
 * 
 * Note: write methods in the subclasses sometimes act like process methods.
 * 
 * You will also notice that several methods are overloaded. The overloaded methods allow a caller
 * to get information at a specific level within the claim information hierarchy. For example, dates
 * are needed to record when a request for payment was made, and dates are also required to track
 * when services were performed. Since each of these dates is at a different level in the
 * information hierarchy - the request date is near the top of the hierarchy, the service date near
 * the bottom - different apis exists to access each date.
 * 
 * ExtractDntlClaims.java, ExtractInstClaims.java and ExtractPrfsnClaims.java all contain a similar
 * high level process flow where the following types of records are processed and written: A Rec B
 * Rec C Rec E Rec F Rec A-999Rec
 * 
 * A records contain envelope information such as who sent the electronic data, who is receiving
 * their name, address, etc.
 * 
 * B records contain information such as who is billing, what their contact information
 * 
 * C records contain information about the subscriber (insured) and the destination payer
 * 
 * E records contain information about the claim
 * 
 * F records contain additional information about line items on the claims and hold data that may be
 * unique to a particular claim type, and other miscellaneous items.
 * 
 * The processes all implement how they write the overall claim data containing (A,B,C,E,F records)
 * a little differently - see each specific subclass for more information.
 * 
 * More information related to the X12 data interchange can be found in the industry guides at
 * \\Vhahacfs2\hipaa team\EDI Team\HIPAA Ready PBM Impl\Phase I\HIPAA Docs\WPC Version 4010 Combo
 * guides
 * 
 * You may also see the VA Specification that explains the format of the flat file, titled
 * "eRepos to HAC Flat File.xls"
 * 
 * Running the process: The process should be invoked on the command line and passed 2 arguments:
 * the senderid and the path where the extract files will be created.
 * 
 * e.g. java CacheX12Extract <SENDERID> <PATH WHERE FILES ARE CREATED>
 * 
 * -f=path for output files = C:\aaERA_Research\outputFiles
 * -s=senderId = 133052274 (taken from Fee_Final app - no idea what it signifies....
 * 
 * @author vhahacpennet
 * 
 */
public final class CacheX12Extract {
  static final Logger summaryLOGGER = LoggerFactory.getLogger(CacheX12Extract.class);
  static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + CacheX12Extract.class);
  static Map receiverIdPrefixes;
  
  static {
	    receiverIdPrefixes = new HashMap();
	    receiverIdPrefixes.put("133052274p", "84146");
	    receiverIdPrefixes.put("133052274i", "84146");
	    receiverIdPrefixes.put("133052274d", "84147");
	    receiverIdPrefixes.put("752578509p", "55915");
	    receiverIdPrefixes.put("752578509i", "55915");
	    receiverIdPrefixes.put("752578509d", "55915");	    
	    receiverIdPrefixes.put("14330i", "741612229");
	    receiverIdPrefixes.put("14330p", "741612229");
	  }

  // private constructor to prevent unneeded instance creation.
  private CacheX12Extract() {
  }

  /**
   * @author Hipaa-Ready
   * @version 1.0
   * @param args
   */
  public static void main(String[] args) {
    // Parse the command line
    CommandLine line = null;
    Options opts = getCliOptions();
    try {
      CommandLineParser parser = new GnuParser();
      
      
      // bash CACHE_X12_EXTRACT_JPA.sh -s 133052274 -p E:\ecedigs\PF_PostOffice\CACHE_LOAD_X12\ -c 
      // empty! - WHY
      line = parser.parse(opts, args);
      // If the "help" option is specified, just show usage options and exit
      if (line.hasOption("h")) {
        printUsage(opts, args, false);
        System.exit(1);
      }
      // If any required command line parameter is missing, just show usage options and exit
    } catch (MissingOptionException moe) {
      printUsage(opts, args, true);
      System.exit(1);
    } catch (ParseException pe) {
      System.out.println("Problem parsing the command line");
      printUsage(opts, args, false);
      System.exit(1);
    }

    // Local variables
    boolean processSuccess = false;
    boolean useClones = false;
    String destinationPath = "";
    String senderId = "";
    String configFilePath = "";
    EntityManager em = null;

    try {
      LogUtils.stampStartingTime("CacheX12Extract Start", summaryLOGGER);
      summaryLOGGER.info("Code Version: 5010 Project");

      // Put field names into the trace file
      // NCPDPLogUtils.logNCPDPExtractFieldNames(null, traceLOGGER);

      // Get an EntityManager, based on the -f option if passed in, otherwise based on a default
      File configFile = null;
      configFilePath = line.getOptionValue("f");
      if (configFilePath != null && configFilePath.length() > 0) {
        configFile = JPAUtil.getJpaConfigFile(configFilePath);
      } else {
        configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
      }
      
      try {
	      em = JPAUtil.getEntityManager(configFile, "eResposExtract");
	      em.setFlushMode(FlushModeType.COMMIT);
      }catch(Exception x) {
    	  System.out.println("CacheX12Extract.main() - error with mapping file: " + x.getMessage());
    	  x.printStackTrace();
      }
     
      
      if (line.hasOption("useClones"))
      {
    	  useClones = true;
      }

      // If parameters are OK, run this process
      destinationPath = line.getOptionValue("p");
      
      senderId = line.getOptionValue("s");

      if (destinationPath != null) {
        destinationPath = destinationPath.trim();
      }

      if (senderId != null) {
        senderId = senderId.trim();
      }
      

      if (destinationPath != null && destinationPath.length() > 0 && senderId != null
          && senderId.length() > 0) {
        System.out.println("Destination Path from parameters: " + destinationPath);
        System.out.println("Sender Id from parameters: " + senderId);
        System.out.println("Use Clones from parameters: " + useClones);


        String receiverIdStart;
        
        System.out.println("Starting Professional Extract at:"
                + DateUtil.getCurrentTimeFormatted("HH:mm:ss:SSS"));
        // Create the extract file for professional claims
        receiverIdStart = (String) receiverIdPrefixes.get(senderId + "p");
        if(receiverIdStart != null)
        {       
        	final ExtractPrfsnClaims prfsnClaim = new ExtractPrfsnClaims();
        	prfsnClaim.processPrfsnClaims(senderId, destinationPath, receiverIdStart, useClones, em);
        }
        System.out.println("Finished Professional Extract at:"
        		+ DateUtil.getCurrentTimeFormatted("HH:mm:ss:SSS"));

        // Create the extract file for institutional claims
        System.out.println("Starting Institutional Extract at:"
                + DateUtil.getCurrentTimeFormatted("HH:mm:ss:SSS"));
        receiverIdStart = (String) receiverIdPrefixes.get(senderId + "i");
        if(receiverIdStart != null)
        {
        	final ExtractInstClaims instClaim = new ExtractInstClaims();
        	instClaim.processInstClaims(senderId, destinationPath, receiverIdStart, useClones, em);
        }
        System.out.println("Finished Institutional Extract at:"
        		+ DateUtil.getCurrentTimeFormatted("HH:mm:ss:SSS"));

        
        // Create the extract file for dental claims
        System.out.println("Starting Dental Extract at:"
                + DateUtil.getCurrentTimeFormatted("HH:mm:ss:SSS"));
        receiverIdStart = (String) receiverIdPrefixes.get(senderId + "d");
        if(receiverIdStart != null)
        {
        	final ExtractDntlClaims dntlClaim = new ExtractDntlClaims();
        	dntlClaim.processDntlClaims(senderId, destinationPath, receiverIdStart, useClones, em);
        }

        System.out.println("Finished Dental Extract at:"
            + DateUtil.getCurrentTimeFormatted("HH:mm:ss:SSS"));
        
      } else {
        summaryLOGGER.error("CacheX12Extract: Please Pass both Parameters");
      }

      // Put a completion message into the log file
      System.out.println("Process completed successfully");
      LogUtils.stampEndingTime("CacheX12Extract completed", summaryLOGGER);
      processSuccess = true;

    } catch (OutOfMemoryError e) {
      summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
    } catch (Throwable t) {
      summaryLOGGER.error("Exception or Error Thrown During Processing", t);
    } finally {
      if (!processSuccess) {
        summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
      }
    }
  }

  /**
   * Helper method to populate command line options objects.
   * 
   * @return the <code>Options</code> object containing the set of command line option.
   */
  @SuppressWarnings("static-access")
  private static Options getCliOptions() {
    Options options = new Options();

    Option configfile = OptionBuilder.withArgName("jpaFile").hasArg().withLongOpt("jpaFile")
        .withDescription("The JPA Configuration file, such as hibernate.properties.").create("f");

    Option destinationPath = OptionBuilder.withArgName("destinationPath").hasArg().isRequired().
        withLongOpt("destinationPath").withDescription("The path to output files.").create("p");

    Option senderId = OptionBuilder.withArgName("senderId").hasArg().isRequired().
        withLongOpt("senderId").withDescription(
            "The identifier of the Clearinghouse "
                + "whose claims should be extracted.").create("s");

    
    
    Option useClones = OptionBuilder.withArgName("useClones").withLongOpt("useClones").
    withDescription("Use cloned objects to drive loading rather than Hibernate cache.").create("c");

    Option help = OptionBuilder.withLongOpt("help").withDescription("Print this message.").create(
        "h");
    
    options.addOption(help);
    options.addOption(configfile);
    options.addOption(destinationPath);
    options.addOption(senderId);
    options.addOption(useClones);

    return options;
  }

  private static void printUsage(Options opts, String[] inArgs, boolean missingArg) {
    final int helpwidth = 80;
    HelpFormatter fmtr = new HelpFormatter();
    System.out.println();
    if (missingArg) {
      System.out.println("Some required commandline argument was missing");
    }
    fmtr.printHelp(helpwidth, "CacheX12Extract", "", opts, "", true);
    if (missingArg) {
      System.out.println(inArgs.length + " command line arguments were received: ");
      for (int i = 0; i < inArgs.length; i++) {
        System.out.println(inArgs[i]);
      }
    }
  }
}