package gov.va.med.hac.edi.extract.x12.jpa;
 
import gov.va.med.hac.edi.utils.StringUtils;
import gov.va.med.hac.edi.utils.jpa.DateUtil;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.io.*;
import java.util.*;

import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;
import javax.persistence.Query;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import gov.va.med.domain.erepos.*;

public class ExtractPrfsnClaims extends ProcessCacheRecords {
	
	static final Logger summaryLOGGER = LoggerFactory.getLogger(ExtractPrfsnClaims.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + ExtractPrfsnClaims.class);
	final private static BigDecimal zeroBD = new BigDecimal("0");	

	/**
	 * This method drives the creation of the professional claim extract file.
	 * 
	 * It creates an extract file named <clearing house id> _opt_
	 * 
	 * @param vChId
	 *            - The id of the clearing house
	 * @param vFilePath
	 *            - the path where the extract file will be created
	 * @param receiverIdStart
	 *            -
	 * @param vLogFile
	 * @throws DbTransactionEnderException
	 */
	protected void processPrfsnClaims(String vChId, String vFilePath,
			String receiverIdStart, boolean useClones, EntityManager dbEM) 
			throws DbTransactionEnderException {
		try { // -- Process Prfsn claims
			
			/** Start 1 TWP Added to accommodate new file name w receiverId and writing to tmp file first ***/
			final String fileName = createFileName(vChId, "_opt_");

			File PrfsnFile = createTemporaryFileAndDir(vFilePath, fileName);
			if(PrfsnFile == null)
			{
				throw new IOException("Could not create a temporary file or directory to write the extract file to. " + vFilePath + "tmp : "+ fileName);
			}
			/** End 1 TWP Added to accommodate new file name w receiverId and writing to tmp file first ***/
			
			List claimKeysToUpdate = new ArrayList();
			int claimCount = 0;
			PrintWriter vPrfsnFile = null;

			// -- Process Prof Claims
			Query getClaimQuery = dbEM
			.createQuery("SELECT c FROM gov.va.med.domain.erepos.Claim c WHERE (c.claimType=:claimType) AND c.statusCode=:statusCode AND c.senderId=:senderId AND (c.receiverId LIKE :receiverId) ORDER BY c.dateCreated, c.fileNumber, c.st02txnCtlNbr");
			getClaimQuery.setParameter("claimType", "005010X222");
			getClaimQuery.setParameter("statusCode", "NEW");
			getClaimQuery.setParameter("senderId", vChId);
			getClaimQuery.setParameter("receiverId", receiverIdStart + "%");
			

			List<Claim> claims = getClaimQuery.getResultList();
			
			Iterator<Claim> claimIterator;
			int numClaims = claims.size();
			summaryLOGGER.info("Number of Professional claims: " + claims.size());

			if(useClones)
			{
				List<Claim> clonedClaims = cloneAndClearClaims(claims);			
				claims = null;
				dbEM.clear();				
				System.gc();
				claimIterator = clonedClaims.iterator();
			}
			else
			{
				claimIterator = claims.iterator();
			}						

			if (numClaims > 0) {
				vPrfsnFile = new PrintWriter(new FileOutputStream(PrfsnFile));

				while (claimIterator.hasNext()) // -- loop for each claim
				{
					final Claim rowClaim = (Claim) claimIterator.next();
					final String vFileNumber = rowClaim.getFileNumber();
					final String vSt02txnCtlNbr = rowClaim.getSt02txnCtlNbr();
					writeARec(vFileNumber, vSt02txnCtlNbr, vPrfsnFile, dbEM);
					writeBRec(rowClaim, vPrfsnFile, dbEM);
					writeCRec(rowClaim, vPrfsnFile, dbEM);
					writeERec(rowClaim, vPrfsnFile, dbEM);
					writeFRec(rowClaim, vPrfsnFile, dbEM);
					writeA999Rec(vPrfsnFile);
					claimKeysToUpdate.add(rowClaim.getClaimKey());
					
					if(((50+claimCount) % 50) == 0)
					{							
						if(useClones) dbEM.clear();
						vPrfsnFile.flush();
						traceLOGGER.debug("cc= " + claimCount + " " + DateUtil.getCurrentTimeFormatted("hh:mm:ss"));
					}		
					claimCount++;
				}

				vPrfsnFile.flush();
	      vPrfsnFile.close();
			}

			dbEM.clear();
			traceLOGGER.debug("Committing the database transaction");
			dbEM.getTransaction().begin();
			Iterator claimToPersistIterator = claimKeysToUpdate.iterator();
			Query deleteStatement;
			while (claimToPersistIterator.hasNext()) {
				deleteStatement = dbEM.createNativeQuery("update claim set status_code=:statusCode where claim_key=:claimKey");
				deleteStatement.setParameter("statusCode", "BUILT");
				deleteStatement.setParameter("claimKey", (String) claimToPersistIterator.next());				
				deleteStatement.executeUpdate();
			}
			dbEM.getTransaction().commit();			
			traceLOGGER.debug("Commit successful");
			
			/** Start 2 TWP Added to accommodate new file name w receiverId and writing to tmp file first ***/
			File renamedFile = new File(vFilePath + fileName);
			PrfsnFile.renameTo(renamedFile);
			traceLOGGER.debug("Was able to rename the file to " + renamedFile.getAbsolutePath());
			/** End 2 TWP Added to accommodate new file name w receiverId and writing to tmp file first ***/
			
			traceLOGGER.debug("Prfsn  Claims Processed : " + claimCount);

		} // -- end write Prfsn clmains
		catch (Exception ex) {
			traceLOGGER.error("Rolling back the database transaction");
			if (dbEM != null) {
				if (dbEM.getTransaction().isActive()) {
					dbEM.getTransaction().rollback();
				}
			}
			traceLOGGER.debug("Rollback successful");
			throw new DbTransactionEnderException(
					"Rolled Back the Transaction", ex);
		}
	}

	/**
	 * Gets subscriber (insured) information and the destination payer data and
	 * writes it to the extract file.
	 * 
	 * @param rowClaim
	 *            - the claim related to the subscriber
	 * @param vPrfsnFile
	 *            - the extract file to write the data to
	 * @param dbEM
	 *            - the entity manager used to query the database
	 */
	private void writeCRec(Claim rowClaim, PrintWriter vPrfsnFile,
			EntityManager dbEM) {
		SubscriberInfo rowSubscriberInfo = null;
		PatientInfo rowPatientInfo = null;
		String vSbrSecId = "";
		String vDtOfDeath = "";
		final String vFileNumber = rowClaim.getFileNumber();
		final String vSt02txnCtlNbr = rowClaim.getSt02txnCtlNbr();
		final String vSbsrId = rowClaim.getSubscriberId();	
		final String vSubscriberHrchlLevelKey = rowClaim.getSubscriberHrchlLevelKey();
		

		Query getSubscriberInfoQuery = dbEM
				.createQuery("SELECT si FROM gov.va.med.domain.erepos.SubscriberInfo si WHERE si.txnSet.id.fileNumber=:fileNumber AND si.txnSet.id.st02txnCtlNbr=:st02txnCtlNbr AND si.nm109sbsrId=:nm109sbsrId AND si.loopId=:loopId AND si.hrchlLevelKey=:hrchlLevelKey");
		getSubscriberInfoQuery.setParameter("fileNumber", vFileNumber);
		getSubscriberInfoQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		getSubscriberInfoQuery.setParameter("nm109sbsrId", vSbsrId);
		getSubscriberInfoQuery.setParameter("loopId", "2000B");
		getSubscriberInfoQuery.setParameter("hrchlLevelKey", vSubscriberHrchlLevelKey);
		rowSubscriberInfo = (SubscriberInfo) getSubscriberInfoQuery
				.getResultList().get(0);
		
		final String vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
		+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND NM109SBSR_ID = '" + vSbsrId + "' "
		+ "AND HRCHL_LEVEL_KEY ='" + vSubscriberHrchlLevelKey +"'";
		
		//5010: ADDED NM101ENTY_ID_CD
		EntityName rowEntityName = getEntityName(vWhereCls
				+ "AND LOOP_ID = '2010BA' AND NM101ENTY_ID_CD='IL'", dbEM);


		//5010: added HRCHL_LEVEL_KEY
		EntityAddress rowEntityAddress = null;
		if(rowEntityName != null) 
		{
			rowEntityAddress = getEntityAddress(vWhereCls
					+ "AND LOOP_ID = '2010BA' ", dbEM);
		}

		//5010: Changed REF01ID_QLFR='IL' to REF01ID_QLFR='SY'
		vSbrSecId = getRefValue(vWhereCls + "AND LOOP_ID = '2010BA' AND REF01ID_QLFR='SY' ", dbEM);
		
		Query getDemographicInfoQuery = dbEM
				.createQuery("SELECT di FROM gov.va.med.domain.erepos.DemographicInfo di WHERE di.txnSet.id.fileNumber=:fileNumber AND di.txnSet.id.st02txnCtlNbr=:st02txnCtlNbr AND di.nm109sbsrId=:nm109sbsrId AND di.loopId=:loopId AND di.hrchlLevelKey=:hrchlLevelKey");
		getDemographicInfoQuery.setParameter("fileNumber", vFileNumber);
		getDemographicInfoQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		getDemographicInfoQuery.setParameter("nm109sbsrId", vSbsrId);
		getDemographicInfoQuery.setParameter("loopId", "2010BA");
		getDemographicInfoQuery.setParameter("hrchlLevelKey", vSubscriberHrchlLevelKey);
		DemographicInfo rowDemographicInfo = null;
		if(getDemographicInfoQuery.getResultList().size() > 0)
		{
			rowDemographicInfo = (DemographicInfo) getDemographicInfoQuery.getResultList().get(0);
		}
		
		final PatientInfo voPatientInfo = null;
		Query getPatientInfoQuery = dbEM
				.createQuery("SELECT pi FROM gov.va.med.domain.erepos.PatientInfo pi WHERE pi.fileNumber=:fileNumber AND pi.st02txnCtlNbr=:st02txnCtlNbr AND pi.nm109sbsrId=:nm109sbsrId AND pi.loopId=:loopId  AND pi.hrchlLevelKey=:hrchlLevelKey");
		getPatientInfoQuery.setParameter("fileNumber", vFileNumber);
		getPatientInfoQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		getPatientInfoQuery.setParameter("nm109sbsrId", vSbsrId);
		getPatientInfoQuery.setParameter("loopId", "2000C");
		if(rowClaim.getPatientHrchlLevelKey() != null)
		{
			getPatientInfoQuery.setParameter("hrchlLevelKey", rowClaim.getPatientHrchlLevelKey());	
		}else
		{
			getPatientInfoQuery.setParameter("hrchlLevelKey", vSubscriberHrchlLevelKey);
		}
				
		
		List patientInfos = getPatientInfoQuery.getResultList();
		Iterator patientInfoIterator = patientInfos.iterator();

		if (patientInfoIterator.hasNext()) {
			rowPatientInfo = (PatientInfo) patientInfoIterator.next();
			vDtOfDeath = rowPatientInfo.getPat06deathDt();
		}


		writeC000Rec(vSbrSecId, vDtOfDeath, rowSubscriberInfo, rowEntityName,
				rowEntityAddress, rowDemographicInfo, vPrfsnFile);

		rowEntityName = getEntityName(vWhereCls + "AND LOOP_ID = '2010BB' ",
				dbEM);

		writeC005Rec(rowEntityName, vPrfsnFile);

		detach(rowSubscriberInfo, dbEM);
		detach(patientInfos, dbEM);
		detach(rowDemographicInfo, dbEM);
		detach(rowEntityName, dbEM);
//		vPrfsnFile.flush();
	}

	/**
	 * Gets basic claim information and writes it to the extract file.
	 * 
	 * @param rowClaim
	 *            - the claim
	 * @param vOutputFile
	 *            - the extract file
	 * @param dbEM
	 *            - entity manager attached to eRepository
	 */
	private void writeERec(Claim rowClaim, PrintWriter vOutputFile,
			EntityManager dbEM) {
		final String vClaimKey = rowClaim.getClaimKey();
		final String vFileNumber = rowClaim.getFileNumber();
		final String vSt02txnCtlNbr = rowClaim.getSt02txnCtlNbr();
		String vTPClaimRecId = getRefValue(vClaimKey, "2300", "D9", dbEM);
		// E000 Record can already be written
		writeE000Rec(rowClaim, "B", vTPClaimRecId, vOutputFile);

		// E005 Record has a little logic
		processE005Rec(rowClaim, vOutputFile, dbEM);

		// E009 Record
		processE009Rec(vClaimKey, vOutputFile, dbEM);

		// Write the E010 record if the total billed is positive
		final BigDecimal vTotBilled = rowClaim.getClm02clmChrgAmt();
		final BigDecimal vPatAmtPaid = getClaimAmt(vClaimKey, "F5", dbEM);
		final BigDecimal vPatBalDue = getClaimAmt(vClaimKey, "F3", dbEM);
		
		/**
	     *  HRL - TWP: Making a change similar to DEF009627 - 
	     *  Prev: if (vTotBilled.signum() == 1) This was allowing only
	     * 	non-zero, positive billed amounts. Zero-dollar claim amounts are valid
	     */
		if (vTotBilled.signum() >= 0) {
			writeE010Rec(vTotBilled, vPatAmtPaid, vPatBalDue, vOutputFile);
		}

		// E015 Record is complicated
		processE015Rec(vClaimKey, vOutputFile, dbEM);

		processE025Rec(vClaimKey, vOutputFile, dbEM);
		
		processE026Rec(vClaimKey, vOutputFile, dbEM);
		
		processE027Rec(vClaimKey, vOutputFile, dbEM);
				
		processHiCodesRec(rowClaim, vOutputFile, dbEM);

		ClaimPrcReprc rowClaimPrcReprc = null;
		Query getClaimPrcReprcQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM CLAIM_PRC_REPRC WHERE CLAIM_KEY=:claimKey AND INST_SERVICE_KEY IS NULL AND PRFSN_SERVICE_KEY IS NULL",
						ClaimPrcReprc.class);
		getClaimPrcReprcQuery.setParameter("claimKey", vClaimKey);
		List claimPrcReprcs = getClaimPrcReprcQuery.getResultList();
		Iterator claimPrcReprcIterator = claimPrcReprcs.iterator();

		if (claimPrcReprcIterator.hasNext()) {
			rowClaimPrcReprc = (ClaimPrcReprc) claimPrcReprcIterator.next();
			writeE125Rec(rowClaimPrcReprc, vOutputFile);
		}

		// The following is the code to create E205 Record - Service Location
		EntityName rowEntityName = null;
		EntityAddress rowEntityAddress = null;
		String vEntWhereClause = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND NM101ENTY_ID_CD IN ('77') "; //5010: removed these qualifiers from the in clause 'FA', 'LI', 'TL'

		detach(rowEntityName, dbEM);
		rowEntityName = getEntityName(vEntWhereClause
				+ "AND LOOP_ID = '2310C' ", dbEM); //5010: changed from D to C
		
		rowEntityAddress = null;
		if(rowEntityName != null) 
		{
			final String vAdrWhereClause = "CLAIM_KEY = '" + vClaimKey + "' AND HRCHL_LEVEL_KEY='"+rowEntityName.getHrchlLevel().getHrchlLevelKey()+"'"; //5010: added HRCHL_LEVEL_KEY
			rowEntityAddress = getEntityAddress(vAdrWhereClause
				+ "AND LOOP_ID = '2310C' ", dbEM); //5010: changed from D to C
		}

		if ((rowEntityName == null) || (rowEntityAddress == null)) {
			vEntWhereClause = "FILE_NUMBER = '" + vFileNumber + "' "
					+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
					+ "AND LOOP_ID = '2010AA' ";
			detach(rowEntityName, dbEM);
			rowEntityName = getEntityName(vEntWhereClause, dbEM);
			detach(rowEntityAddress, dbEM);		
			rowEntityAddress = null;
			if(rowEntityName != null)
			{
				vEntWhereClause = "FILE_NUMBER = '" + vFileNumber + "' "
				+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
				+ "AND LOOP_ID = '2010AA' " + " AND HRCHL_LEVEL_KEY='"+rowEntityName.getHrchlLevel().getHrchlLevelKey()+"'"; 
				rowEntityAddress = getEntityAddress(vEntWhereClause, dbEM);
			}
		}
		writeSvcLocRec(rowEntityName, rowEntityAddress, "CLAIM", vOutputFile);

		// The following is the code to create E208 Record - Referring Physician
		vEntWhereClause = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND NM101ENTY_ID_CD IN ('DN', 'P3') ";

		detach(rowEntityName, dbEM);
		rowEntityName = getEntityName(vEntWhereClause
				+ "AND LOOP_ID = '2310A' ", dbEM);

		String vTxwhereCls = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND PRV01PRVD_TYPE_CD = 'RF' " + "AND LOOP_ID = '2310A' ";
		String vAttnTaxonamyCd = getTaxonomyCd(vTxwhereCls, dbEM);
		if (rowEntityName != null) {
			writePhyRec(rowEntityName, vAttnTaxonamyCd, "E208", vOutputFile);
		}

		// The following is the code to create E209 Record - Rendering Physician
		vEntWhereClause = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND NM101ENTY_ID_CD = '82' ";

		detach(rowEntityName, dbEM);
		rowEntityName = getEntityName(vEntWhereClause
				+ "AND LOOP_ID = '2310B' ", dbEM);

		vTxwhereCls = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND PRV01PRVD_TYPE_CD = 'PE' " + "AND LOOP_ID = '2310B' ";
		vAttnTaxonamyCd = getTaxonomyCd(vTxwhereCls, dbEM);
		if (rowEntityName != null) {
			writePhyRec(rowEntityName, vAttnTaxonamyCd, "E209", vOutputFile);
		}

		// The following is the code to create E210 Record - Other Insurance
		// Info.
		processE210Rec(vClaimKey, vPatAmtPaid, vPatBalDue, vOutputFile, dbEM);

		// The following is the code to create E216 Record - Medicare Outpatient
		// Adjudication Info.

		MediOutptAdjdct rowMediOutptAdjdct = null;
		Query getMediOutptAdjdctQuery = dbEM.createNativeQuery(
				"SELECT * FROM MEDI_OUTPT_ADJDCT WHERE CLAIM_KEY=:claimKey",
				MediOutptAdjdct.class);
		getMediOutptAdjdctQuery.setParameter("claimKey", vClaimKey);
		List mediOutptAdjdcts = getMediOutptAdjdctQuery.getResultList();
		Iterator mediOutptAdjdctIterator = mediOutptAdjdcts.iterator();

		if (mediOutptAdjdctIterator.hasNext()) {
			rowMediOutptAdjdct = (MediOutptAdjdct) mediOutptAdjdctIterator
					.next();
			writeE216Rec(rowMediOutptAdjdct, vOutputFile);
		}

		// The following is the code to create E220 Record - OHI Claim
		// Adjustment
		processE220Rec(vClaimKey, vOutputFile, dbEM);

		// The following is the code to create E225 Record - Other Insurance
		// General Info.
		processE225Rec(vClaimKey, vOutputFile, dbEM);
		detach(rowEntityName, dbEM);
		detach(rowEntityAddress, dbEM);
		detach(mediOutptAdjdcts, dbEM);
		detach(claimPrcReprcs, dbEM);

//		vOutputFile.flush();
	} // end writeERec

	/**
	 * Gets claim dates and writes the dates to the extract file.
	 * 
	 * @param rowClaim
	 *            - the claim used to get the dates from
	 * @param vPrfsnFile
	 *            - the extract file the dates are written to
	 * @param dbEM
	 */
	private void processE005Rec(Claim rowClaim, PrintWriter vPrfsnFile,
			EntityManager dbEM) {
		final String vClaimKey = rowClaim.getClaimKey();

		final String vStmtDt = getDate(vClaimKey, "454", dbEM);
//		final String vStmtDt = getStmtDate(vClaimKey,
//		          "AND PRFSN_SERVICE_KEY IS NOT NULL ", dbEM);
		final String vAdmissionDt = getDate(vClaimKey, "435", dbEM);
		final String vDischargeDt = getDate(vClaimKey, "096", dbEM);
		final String vInjuryDt = getDate(vClaimKey, "431", dbEM);
		final String vAccidentDt = getDate(vClaimKey, "439", dbEM);
		final String vLastSeenDt = getDate(vClaimKey, "304", dbEM);
		final String vPrescriptionDt = getDate(vClaimKey, "471", dbEM);

		writeE005Rec(vStmtDt, vAdmissionDt, vDischargeDt, vInjuryDt,
				vAccidentDt, vLastSeenDt, vPrescriptionDt, vPrfsnFile);
	} // end processE005Rec

	/**
	 * Gets Reference Identification Qualifiers and writes them to the extract file. writes the code,
	 * date, amount etc to the file
	 * 
	 * @param rowClaim
	 *          - the claim that has reference identifiers associated with it
	 * @param vInstFile
	 *          - the file to write the codes to
	 * @param dbEM
	 */
	private void processHiCodesRec(Claim rowClaim, PrintWriter vInstFile,EntityManager dbEM) {
		final String vClaimKey = rowClaim.getClaimKey();
		String vHiCodeSegment = "";
		String attrName = "";
		String segmentNumber = "";
		String vHiQual = "";
		String vHiCdListQual = "";
		String vHiCd = "";
		String vHiDt = "";
		String vHiCndtnResponseCd = "";

		BigDecimal vHiQnty = new BigDecimal(0);
		String qualAndCode = "";
		BigDecimal vHiAmt = new BigDecimal(0);
		int numberOfSegments = 0;


		// Table HI_CODES has 60 columns, 5 each for segments 1-12, with names
		// like
		// HIxxCD_LIST_QUAL
		// HIxxPROC_CD
		// HIxxDTTM_QUAL - this date/time qualifier seems to be ignored, leaving
		// 48 possible columns
		// HIxxPROC_DT
		// HIxxAMOUNT
		// The code below decides how many of these 12 segments to consider,
		// based on the value
		// found in the CD_LIST_QUAL for segment 
		HiCodes rowHiCodes = null;
		Query getHiCodesQuery = dbEM
		.createNativeQuery(
				"SELECT * FROM HI_CODES WHERE CLAIM_KEY=:claimKey ORDER BY SEQ_NBR",
				HiCodes.class);
		getHiCodesQuery.setParameter("claimKey", vClaimKey);
		List hiCodes = getHiCodesQuery.getResultList();
		Iterator hiCodeIterator = hiCodes.iterator();

		StringBuffer e100Rec;
		StringBuffer e105Rec;
		StringBuffer e110Rec;

		// HRL - MSB Added new if condition to check for existance of ICD Codes.
		// If no codes exist write blank record with record type for E100 record
		// (like "E100")
		// Cache load process expects E100 record to be mandatory and as per EDI
		// guide it
		// situational, so writing this record with type avoids crashing the
		// process on Cache side.
		if (hiCodeIterator.hasNext()) {
			// Loop through the rows from HI_CODES, for which there might be
			// multiple on one claim
			while (hiCodeIterator.hasNext()) {
        
				rowHiCodes = (HiCodes) hiCodeIterator.next();

				e100Rec = new StringBuffer("E100");
				e105Rec = new StringBuffer("E105");
				e110Rec = new StringBuffer("E110");

				// The cd list qualifier determines the number of segments that
				// will be considered
				vHiQual = rowHiCodes.getHi01cdListQual().trim();

				if (vHiQual.equalsIgnoreCase("BK") || vHiQual.equalsIgnoreCase("ABK")) {
					numberOfSegments = 12;				
				} else if (vHiQual.equalsIgnoreCase("BP")) {
					numberOfSegments = 2;
				} else if (vHiQual.equalsIgnoreCase("BG")) {
					numberOfSegments = 12;
				} 

				// Loop through the number of segments that was just defined
				// Really, we are looping through the many columns in the table
				for (int i = 1; i <= numberOfSegments; i++) {
	        
				  // Empty out any fields obtained from the previous record.
	        // This fixes a series of bugs turned up in 5010 testing.
	        vHiCndtnResponseCd = "";
	        vHiAmt = new BigDecimal(0);
	        vHiQnty = new BigDecimal(0);
	        vHiDt = "";
					
	        // These two were already being emptied out
	        vHiCdListQual = "";
					vHiCd = "";

					// If the segment number is single-digit, put a zero in
					// front of it
					segmentNumber = "0" + i;
					segmentNumber = segmentNumber.substring(segmentNumber
							.length() - 2);

					// Get the code list qualifier field for this segment
					attrName = "Hi" + segmentNumber + "cdListQual";
					if (getAttribute(attrName, rowHiCodes) != null) {
						vHiCdListQual = (String) getAttribute(attrName,
								rowHiCodes);
					}

					// Get the code field for this segment
					attrName = "Hi" + segmentNumber + "procCd";
					if (getAttribute(attrName, rowHiCodes) != null) {
						vHiCd = (String) getAttribute(attrName, rowHiCodes);
					}

					// Get the poa indicator
					attrName = "Hi" + segmentNumber + "cndtnResponseCd";
					if (getAttribute(attrName, rowHiCodes) != null) {
						vHiCndtnResponseCd = (String) ""+getAttribute(attrName, rowHiCodes);
					}

					// Get the date field for this segment
					attrName = "Hi" + segmentNumber + "procDt";
					if (getAttribute(attrName, rowHiCodes) != null) {
						vHiDt = (String) getAttribute(attrName, rowHiCodes);
					}

					// Get the amount field for this segment
					attrName = "Hi" + segmentNumber + "amount";
					if (getAttribute(attrName, rowHiCodes) != null) {
						vHiAmt = (BigDecimal) getAttribute(attrName, rowHiCodes);
					}

					// Get the quantity field for this segment
					attrName = "Hi" + segmentNumber + "qnty";
					if (getAttribute(attrName, rowHiCodes) != null) {
						vHiQnty = (BigDecimal) getAttribute(attrName, rowHiCodes);
					}

					// We start out with the code qualifier and code
					qualAndCode = vHiCdListQual + vHiCd;

					// This code was used for pre-POA processing
					if (!("".equals(qualAndCode))) {
						// If we have a qualifier and code, create the Hi
						// record
						vHiCodeSegment = writeHiRec(vHiCdListQual, vHiCd, vHiCndtnResponseCd, vHiDt, vHiAmt,vHiQnty);

						// Segments 1-4 go onto the e100 record
						if (i <= 4) {
							e100Rec.append(vHiCodeSegment);
							// Segments 5-8 go onto the e105 record
						} else if ((i > 4) && (i <= 8)) {
							e105Rec.append(vHiCodeSegment);
							// Segments 9-12 go onto the e110 record
						} else if (i > 8) {
							e110Rec.append(vHiCodeSegment);
						}

						// Reset variables
						vHiCdListQual = "";
						vHiCd = "";
						vHiDt = "";
						vHiAmt = zeroBD;

					} 
				} // End of looping through the segments

				writeHiRecs(e100Rec.toString(), e105Rec.toString(), e110Rec
						.toString(), vInstFile);
				detach(hiCodes, dbEM);
			} // End of looping through the rows of HI_CODES
			// HRL - MSB Writing the E100 record with just record type value
		} else {
			writeHiRecs("E100", "", "", vInstFile);
		}
	}

	
	protected void processE025Rec(String vClaimKey, PrintWriter vPrfsnFile,
			EntityManager dbEM) {
		Query getAmblnTrnptInfoQuery = dbEM
		.createQuery("SELECT ai FROM gov.va.med.domain.erepos.AmblnTrnptInfo ai WHERE ai.claim.claimKey=:claimKey AND ai.cr101uomCd =:cr101uomCd and ai.prfsnSvcLine.serviceKey is null");
		getAmblnTrnptInfoQuery.setParameter("claimKey", vClaimKey);
		getAmblnTrnptInfoQuery.setParameter("cr101uomCd", "LB");
		List rowAmblnTrnptInfos = (List) getAmblnTrnptInfoQuery.getResultList();
		if(rowAmblnTrnptInfos != null)		
		{//this is optional in the guide
			Iterator rowAmblnTrnptInfoIterator = rowAmblnTrnptInfos.iterator();
			while(rowAmblnTrnptInfoIterator.hasNext())
			{			   
				writeE025Rec((AmblnTrnptInfo)rowAmblnTrnptInfoIterator.next(), vPrfsnFile);
			}
		}
	} 
	
	protected void processE026Rec(String vClaimKey, PrintWriter vOutputFile, EntityManager dbEM)
	{
		//TWP: I am ignoring the qulifier because there isn't one for address and
		//there is only 1 name and 1 address that comes in these loops
		String vWhereClause = " CLAIM_KEY='"+ vClaimKey + "' " + "AND LOOP_ID = '2310E'";
		EntityName rowEntityName = getEntityName(vWhereClause, dbEM);
		EntityAddress rowEntityAddress = null;	
		rowEntityAddress = getEntityAddress(vWhereClause, dbEM);
		if(rowEntityName != null || rowEntityAddress != null)
		{
			writeE026Rec(rowEntityName, rowEntityAddress, vOutputFile);
		}
	}
	
	protected void processE027Rec(String vClaimKey, PrintWriter vOutputFile, EntityManager dbEM)
	{
		String vWhereClause = " CLAIM_KEY='"+ vClaimKey + "' " + "AND LOOP_ID = '2310F'";
		EntityName rowEntityName = getEntityName(vWhereClause, dbEM);
		EntityAddress rowEntityAddress = null;	
		rowEntityAddress = getEntityAddress(vWhereClause, dbEM);
		if(rowEntityName != null || rowEntityAddress != null)
		{
			writeE027Rec(rowEntityName, rowEntityAddress, vOutputFile);
		}
	}

	protected void processF011Rec(String vprfsnServiceKey, PrintWriter vOutputFile, EntityManager dbEM)
	{
		String vWhereClause = " PRFSN_SERVICE_KEY='"+ vprfsnServiceKey + "' " + "AND LOOP_ID = '2420G'";		
		EntityAddress rowEntityAddress = null;	
		rowEntityAddress = getEntityAddress(vWhereClause, dbEM);
		if(rowEntityAddress != null)
		{
			writeF011Rec( rowEntityAddress, vOutputFile);
		}
	}
	
	protected void processF013Rec(String vprfsnServiceKey, PrintWriter vOutputFile, EntityManager dbEM)
	{
		String vWhereClause = " PRFSN_SERVICE_KEY='"+ vprfsnServiceKey + "' " + "AND LOOP_ID = '2420H'";
		EntityName rowEntityName = getEntityName(vWhereClause, dbEM);
		EntityAddress rowEntityAddress = null;	
		rowEntityAddress = getEntityAddress(vWhereClause, dbEM);				
		if(rowEntityName != null || rowEntityAddress != null)
		{
			writeF013Rec(rowEntityName, rowEntityAddress, vOutputFile);
		}
	}
	/**
	 * Writes ambulance certification record to the extract file
	 * @param vFile
	 */
	protected void writeE025Rec(AmblnTrnptInfo amblnTrnptInfo, PrintWriter vFile) {
		final StringBuffer e025Rec = new StringBuffer("E025");
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr103amblnTrnCd(),1));  //Amb transport code 
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr104amblnTrnRsn(),1)); //Transport Reason code
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr105uomDist(),2)); //Transport distance code
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr106trnspDist(),15)); //Transport distance
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr102ptWt(),10)); //Patient Weight
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr109rtrpPrps(),80)); //Round Trip Purpose
		e025Rec.append(StringUtils.rPadSpaces(amblnTrnptInfo.getCr110strchPrps(),80)); //Stretcher Purpose Description
		traceLOGGER.debug("E025Rec: " + e025Rec.toString());
		vFile.println(e025Rec);
		e025Rec.delete(0, e025Rec.length());
	}
	

	/**
	 * Writes the line level ambulance pickup information to the extract file. 
	 * @param rowEntityAddress
	 * @param vFile
	 */
	protected void writeE026Rec(EntityName rowEntityName, EntityAddress rowEntityAddress, PrintWriter vFile) {
		final StringBuffer e026Rec = new StringBuffer("E026");
		
		if(rowEntityName != null)
		{
			e026Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm101entyIdCd(),     //Pick Up Entity Identifier Code
				3));
			e026Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm102entyQual(),  //Pick Up Entity Type Qualifier
				1));
		}
		else
		{
			e026Rec.append(StringUtils.rPadSpaces("",3));
			e026Rec.append(StringUtils.rPadSpaces("",1));			
		}

		e026Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),     //Ambulance Pickup Entity Address Line 1 (Street)
				55));

		e026Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(),		//Ambulance Pickup Entity Address Line 2 (Street 2)
				55));
		e026Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(),		//Ambulance Pickup Entity City
				30));
		e026Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));	//Ambulance Pickup Entity State
		e026Rec.append(StringUtils.rPadSpaces(rowEntityAddress
				.getN403postalCd(), 15));											//Ambulance Pickup Entity Zip Code
		e026Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),		//Ambulance Pickup Entity Country
				3));
		traceLOGGER.debug("E026Rec: " + e026Rec.toString());
		vFile.println(e026Rec.toString());
		e026Rec.delete(0, e026Rec.length());
	}
	
	/**
	 * Writes the line level ambulance drop off information to the extract file. 
	 * @param rowEntityAddress
	 * @param vFile
	 */
	protected void writeE027Rec(EntityName rowEntityName, EntityAddress rowEntityAddress, PrintWriter vFile) {
		final StringBuffer e027Rec = new StringBuffer("E027");
		if(rowEntityName != null)
		{
			e027Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm101entyIdCd(),     //Drop Off Entity Identifier Code
				3));
			e027Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm102entyQual(),  //Drop Off Entity Type Qualifier
				1));
			e027Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),  //Drop Off Entity Type Qualifier
					60));			
		}
		else
		{
			e027Rec.append(StringUtils.rPadSpaces("",3));
			e027Rec.append(StringUtils.rPadSpaces("",1));
			e027Rec.append(StringUtils.rPadSpaces("",60));
		}
		
		e027Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),     //Ambulance Drop Off Entity Address Line 1 (Street)
				55));

		e027Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(),		//Ambulance Drop Off Entity Address Line 2 (Street 2)
				55));
		e027Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(),		//Ambulance Drop Off Entity City
				30));
		e027Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));	//Ambulance Drop Off Entity State
		e027Rec.append(StringUtils.rPadSpaces(rowEntityAddress
				.getN403postalCd(), 15));											//Ambulance Drop Off Entity Zip Code
		e027Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),		//Ambulance Drop Off Entity Country
				3));
		traceLOGGER.debug("E027Rec: " + e027Rec.toString());
		vFile.println(e027Rec.toString());
		e027Rec.delete(0, e027Rec.length());
	}


	/**
	 * Writes service line level information to the extract file.
	 * 
	 * @param rowClaim
	 * @param vOutputFile
	 * @param dbEM
	 */
	private void writeFRec(Claim rowClaim, PrintWriter vOutputFile,
			EntityManager dbEM) {
		final String vClaimKey = rowClaim.getClaimKey();
		processF000Rec(vClaimKey, vOutputFile, dbEM);
		
		
//		vOutputFile.flush();
	} // end writeFRec


	/**
	 * Writes the line level ambulance pickup information to the extract file. 
	 * @param rowEntityAddress
	 * @param vFile
	 */
	protected void writeF011Rec(EntityAddress rowEntityAddress, PrintWriter vFile) {
		final StringBuffer f011Rec = new StringBuffer("F011");
		f011Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),     //Ambulance Pickup Address Line 1 (Street)
				55));
		f011Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(),		//Ambulance Pickup Address Line 2 (Street 2)
				55));
		f011Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(),		//Ambulance Pickup City
				30));
		f011Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));	//Ambulance Pickup State
		f011Rec.append(StringUtils.rPadSpaces(rowEntityAddress
				.getN403postalCd(), 15));											//Ambulance Pickup Zip Code
		f011Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),		//Ambulance Pickup Country
				3));
		traceLOGGER.debug("F011Rec: " + f011Rec.toString());
		vFile.println(f011Rec.toString());
		f011Rec.delete(0, f011Rec.length());
	}
	
	/**
	 * Writes the line level ambulance drop off information to the extract file. 
	 * @param rowEntityAddress
	 * @param vFile
	 */
	protected void writeF013Rec(EntityName rowEntityName, EntityAddress rowEntityAddress, PrintWriter vFile) {
		final StringBuffer f013Rec = new StringBuffer("F013");
		if(rowEntityName != null)
		{
			
			f013Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),       //Ambulance Drop-Off Location
					60));
		}else
		{
			f013Rec.append(StringUtils.rPadSpaces("",60));
		}
		
		f013Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),     //Ambulance Drop-Off Address Line 1 (Street)
				55));
		f013Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(),		//Ambulance  Drop-Off Address Line 2 (Street 2)
				55));
		f013Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(),		//Ambulance  Drop-Off City
				30));
		f013Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));	//Ambulance  Drop-Off State
		f013Rec.append(StringUtils.rPadSpaces(rowEntityAddress
				.getN403postalCd(), 15));											//Ambulance  Drop-Off Zip Code
		f013Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),		//Ambulance  Drop-Off Country
				3));
		traceLOGGER.debug("F013Rec: " + f013Rec.toString());
		vFile.println(f013Rec.toString());
		f013Rec.delete(0, f013Rec.length());
	}
	

	
	/**
	 * Gets Professional Service Line information and writes it to the extract
	 * file.
	 * 
	 * @param vClaimKey
	 *            - the claim key
	 * @param vPrfsnFile
	 *            - the extract file to write the data to
	 * @param dbEM
	 */
	protected void processF000Rec(String vClaimKey, PrintWriter vPrfsnFile,
			EntityManager dbEM) {

		PrfsnSvcLine rowPrfsnSvcLine = null;
		Query getPrfsnSvcLineQuery = dbEM.createNativeQuery(
				"SELECT * FROM PRFSN_SVC_LINE WHERE CLAIM_KEY=:claimKey",
				PrfsnSvcLine.class);
		getPrfsnSvcLineQuery.setParameter("claimKey", vClaimKey);
		List prfsnSvcLines = getPrfsnSvcLineQuery.getResultList();
		Iterator prfsnSvcLineIterator = prfsnSvcLines.iterator();

		final ArrayList elementsF000 = new ArrayList(7);
		while (prfsnSvcLineIterator.hasNext()) {
			elementsF000.clear();
			rowPrfsnSvcLine = (PrfsnSvcLine) prfsnSvcLineIterator.next();
			final String vPrfsnKey = rowPrfsnSvcLine.getServiceKey();
			final String vSvcKey = "AND PRFSN_SERVICE_KEY = '" + vPrfsnKey
					+ "' ";
			final String vServiceDt = getDate(vClaimKey, vSvcKey, "472", dbEM);
			final String vLineCtrlNo = getRefValue(vClaimKey, "2400", vSvcKey,
					"6R", dbEM);
			final String vRefWhereCls = "CLAIM_KEY = '" + vClaimKey + "' "
					+ "AND LOOP_ID = '2400' " + vSvcKey
					+ "AND REF01ID_QLFR = 'G1' "; //5010: removed IN ('G1', '9F'); this will return the instance with the lowest sequence number 
			final String vPreAuthNo = getRefValue(vRefWhereCls, dbEM);
			final BigDecimal vSalesTaxAmt = getClaimAmt(vClaimKey, vSvcKey,
					"T", dbEM);
			final BigDecimal vPostageAmt = getClaimAmt(vClaimKey, vSvcKey,
					"F4", dbEM);
			final BigDecimal vOhiAprovAmt = getClaimAmt(vClaimKey, vSvcKey,
					"AAE", dbEM);

			final String vBeginTherapyDt = getDate(vClaimKey, vSvcKey, "463", dbEM);
			final String vLestCertificationDt = getDate(vClaimKey, vSvcKey, "461", dbEM);
			final String vInitialTreatmentDt = getDate(vClaimKey, vSvcKey, "454", dbEM);
			
			if (vServiceDt == null) {
				elementsF000.add(" ");
				elementsF000.add(" ");
			} else {
				elementsF000.add(vServiceDt.substring(0, 8));
				if (vServiceDt.length() > 16) {
					elementsF000.add(vServiceDt.substring(9, 17));
				} else {
					elementsF000.add(" ");
				}
			}

			if (vLineCtrlNo == null) {
				elementsF000.add(" ");
			} else {
				elementsF000.add(vLineCtrlNo);
			}

			if (vPreAuthNo == null) {
				elementsF000.add(" ");
			} else {
				elementsF000.add(vPreAuthNo);
			}

			elementsF000.add(vSalesTaxAmt);
			elementsF000.add(vPostageAmt);
			elementsF000.add(vOhiAprovAmt);			

			if (vBeginTherapyDt != null && vBeginTherapyDt.length() > 7) {
				elementsF000.add(vBeginTherapyDt.subSequence(0,8));	
			}else
			{
				elementsF000.add(" ");
			}
			
			if (vLestCertificationDt != null && vLestCertificationDt.length() > 7) {
				elementsF000.add(vLestCertificationDt.subSequence(0,8));	
			}else
			{
				elementsF000.add(" ");
			}
			
			
			if (vInitialTreatmentDt != null && vInitialTreatmentDt.length() > 7) {
				elementsF000.add(vInitialTreatmentDt.subSequence(0,8));	
			}else
			{
				elementsF000.add(" ");
			}
			
			
			writeF000Rec(rowPrfsnSvcLine, elementsF000, vPrfsnFile);						

			DblMedEqpSvc rowDblMedEqpSvc = null;
			Query getDblMedEqpSvcQuery = dbEM
					.createQuery("SELECT d FROM gov.va.med.domain.erepos.DblMedEqpSvc d WHERE d.prfsnServiceKey=:prfsnServiceKey and d.sv501procId=:sv501procId");//5010: added sv501procCode
			getDblMedEqpSvcQuery.setParameter("prfsnServiceKey", vPrfsnKey);
			getDblMedEqpSvcQuery.setParameter("sv501procId", "HC");			// Bug 52: Changed from ProcCode to ProcId
			List dblMedEqpSvcs = getDblMedEqpSvcQuery.getResultList();
			Iterator dblMedEqpSvcIterator = dblMedEqpSvcs.iterator();

			if (dblMedEqpSvcIterator.hasNext()) {
				rowDblMedEqpSvc = (DblMedEqpSvc) dblMedEqpSvcIterator.next();
				writeF008Rec(rowDblMedEqpSvc, vPrfsnFile);
			}


			Query getAmblnTrnptInfoQuery = dbEM
					.createQuery("SELECT d FROM gov.va.med.domain.erepos.AmblnTrnptInfo d WHERE d.prfsnSvcLine.serviceKey=:prfsnServiceKey and d.cr101uomCd=:cr101uomCd");//5010: added
			getAmblnTrnptInfoQuery.setParameter("prfsnServiceKey", vPrfsnKey);
			getAmblnTrnptInfoQuery.setParameter("cr101uomCd", "LB");			
			List amblnTrnptInfos = getAmblnTrnptInfoQuery.getResultList();
			Iterator amblnTrnptInfoIterator = amblnTrnptInfos.iterator();
			if (amblnTrnptInfoIterator.hasNext()) {			
				AmblnTrnptInfo amblnTrnptInfo = (AmblnTrnptInfo) amblnTrnptInfoIterator.next();
				
				Query getClaimQuantityQuery = dbEM
				.createQuery("SELECT c FROM gov.va.med.domain.erepos.ClaimQuantity c WHERE c.qty01qtyQual=:qty01qtyQual AND c.claim.claimKey=:claimKey");//5010: added
				getClaimQuantityQuery.setParameter("qty01qtyQual", "PT");
				getClaimQuantityQuery.setParameter("claimKey", vClaimKey);			
				List claimQuantities = getClaimQuantityQuery.getResultList();
				Iterator claimQuantityIterator = claimQuantities.iterator();
				ClaimQuantity claimQuantity = null; 
				if(claimQuantityIterator.hasNext())
				{
					claimQuantity =  (ClaimQuantity) claimQuantityIterator.next();
				}				
				writeF010Rec(amblnTrnptInfo, claimQuantity, vPrfsnFile); 
			}
				
			processF011Rec(vPrfsnKey, vPrfsnFile, dbEM);
			processF013Rec(vPrfsnKey, vPrfsnFile, dbEM);
			
			Query getDblMdclEqpCertQuery = dbEM
			.createQuery("SELECT d FROM gov.va.med.domain.erepos.DblMdclEqpCert d WHERE d.prfsnServiceKey=:prfsnServiceKey");//5010: added
			getDblMdclEqpCertQuery.setParameter("prfsnServiceKey", vPrfsnKey);			
			List dblMdclEqpCerts = getDblMdclEqpCertQuery.getResultList();
			Iterator dblMdclEqpCertsIterator = dblMdclEqpCerts.iterator();			
			if(dblMdclEqpCertsIterator.hasNext())
			{
				writeF014Rec((DblMdclEqpCert)dblMdclEqpCertsIterator.next(), vPrfsnFile);
			}
			
			processF020Rec(vPrfsnKey, vPrfsnFile, dbEM);
			processF021Rec(vPrfsnKey, vPrfsnFile, dbEM);

			// The following is the code to create F032 Record - Service
			// Location Line level
			EntityName rowEntityName = null;
			EntityAddress rowEntityAddress = null;
			String vEntWhereClause = "PRFSN_SERVICE_KEY = '" + vPrfsnKey + "' "
					+ "AND NM101ENTY_ID_CD='77' "; //5010: changed NM101ENTY_ID_CD IN ('77', 'FA', 'LI', 'TL') to NM101ENTY_ID_CD='77' 
			rowEntityName = getEntityName(vEntWhereClause
					+ "AND LOOP_ID = '2420C' ", dbEM);

			//added hierarchical level
			
			rowEntityAddress = null;
			if(rowEntityName != null)
			{
				final String vAdrWhereClause = "PRFSN_SERVICE_KEY = '" + vPrfsnKey + "' AND HRCHL_LEVEL_KEY='"+rowEntityName.getHrchlLevel().getHrchlLevelKey()+"'"; //5010: added  HRCHL_LEVEL_KEY=
				rowEntityAddress = getEntityAddress(vAdrWhereClause + "AND LOOP_ID = '2420C' ", dbEM);
			}

			if ((rowEntityName != null) || (rowEntityAddress != null)) {
				writeSvcLocRec(rowEntityName, rowEntityAddress, "LINE", vPrfsnFile);
			}

			// The following is the code to create F034 Record - Rendering
			// Physician Line level
			vEntWhereClause = "PRFSN_SERVICE_KEY = '" + vPrfsnKey + "' "
					+ "AND NM101ENTY_ID_CD = '82' ";

			detach(rowEntityName, dbEM);
			rowEntityName = getEntityName(vEntWhereClause
					+ "AND LOOP_ID = '2420A' ", dbEM);

			final String vTxwhereCls = "PRFSN_SERVICE_KEY = '" + vPrfsnKey
					+ "' " + "AND PRV01PRVD_TYPE_CD = 'PE' "
					+ "AND LOOP_ID = '2420A' ";
			final String vAttnTaxonamyCd = getTaxonomyCd(vTxwhereCls, dbEM);
			if (rowEntityName != null) {
				writePhyRec(rowEntityName, vAttnTaxonamyCd, "F034", vPrfsnFile);
			}

			// The following is the code to create F040 Record - OHI Service Line level Info
			processF040Rec(vClaimKey, vPrfsnKey, vPrfsnFile, dbEM);

			FormLqId rowFormLqId = null;
			Query getFormLqIdQuery = dbEM
					.createNativeQuery(
							"SELECT * FROM FORM_LQ_ID WHERE CLAIM_KEY=:claimKey and PRFSN_SERVICE_KEY=:prfnServiceKey",
							FormLqId.class);
			getFormLqIdQuery.setParameter("claimKey", vClaimKey);
			getFormLqIdQuery.setParameter("prfnServiceKey", vPrfsnKey);
			List formLqIds = getFormLqIdQuery.getResultList();
			Iterator formLqIdIterator = formLqIds.iterator();

			if (formLqIdIterator.hasNext()) {
				rowFormLqId = (FormLqId) formLqIdIterator.next();
//TWP:TODO:5010:CACHE WANTS US TO COMMENT THIS OUT				writeF060Rec(rowFormLqId, vPrfsnFile);
			}

			// The following is the code to create F065 Record - Supporting
			// Documentation at Service Line level
			SuptngDoc rowSuptngDoc = null;
			Query getSuptngDocQuery = dbEM
					.createNativeQuery(
							"SELECT * FROM SUPTNG_DOC WHERE CLAIM_KEY=:claimKey and PRFSN_SERVICE_KEY=:prfnServiceKey",
							SuptngDoc.class);
			getSuptngDocQuery.setParameter("claimKey", vClaimKey);
			getSuptngDocQuery.setParameter("prfnServiceKey", vPrfsnKey);
			List suptngDocs = getSuptngDocQuery.getResultList();
			Iterator suptngDocsIterator = suptngDocs.iterator();
			if (suptngDocsIterator.hasNext()) {
				rowSuptngDoc = (SuptngDoc) suptngDocsIterator.next();
      //TWP:TODO:5010:CACHE WANTS US TO COMMENT THIS OUT
			//	writeF065Rec(rowSuptngDoc, vPrfsnFile);
			}

			detach(rowEntityName, dbEM);
			detach(rowEntityAddress, dbEM);
			detach(suptngDocs, dbEM);
		}

	}

	/**
	 * gets and writes Health Care Pricing information to the extract file
	 * 
	 * @param vSvcKey
	 *            - the service line key
	 * @param vPrfsnFile
	 *            - the extract file to write the data to
	 * @param dbEM
	 */
	private void processF020Rec(String vSvcKey, PrintWriter vPrfsnFile,
			EntityManager dbEM) {
		// The following is the code to create F020 Record - Health Care
		// Pricing/Repricing

		ClaimPrcReprc rowClaimPrcReprc = null;
		Query getClaimPrcReprcQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM CLAIM_PRC_REPRC WHERE PRFSN_SERVICE_KEY=:prfnServiceKey",
						ClaimPrcReprc.class);
		getClaimPrcReprcQuery.setParameter("prfnServiceKey", vSvcKey);
		List claimPrcReprcs = getClaimPrcReprcQuery.getResultList();
		Iterator claimPrcReprcIterator = claimPrcReprcs.iterator();
		if (claimPrcReprcIterator.hasNext()) {
			rowClaimPrcReprc = (ClaimPrcReprc) claimPrcReprcIterator.next();
			writeF020Rec(rowClaimPrcReprc, vPrfsnFile);
			detach(rowClaimPrcReprc, dbEM);
		}
	}

	/**
	 * Gets and writes information about dispensed drugs to the extract file.
	 * 
	 * @param vSvcKey
	 *            - the service line key
	 * @param vOutputFile
	 *            - the extract file to write the data to
	 * @param dbEM
	 */
	private void processF021Rec(String vSvcKey, PrintWriter vOutputFile,
			EntityManager dbEM) {
		// The following is the code to create F021 Record - Dispensing Drug
		String vNdcCd = "";
		final String vRefWhereCls = "PRFSN_SERVICE_KEY = '" + vSvcKey + "' "
				+ "AND LOOP_ID = '2410' " + "AND REF01ID_QLFR = 'XZ' ";
		final String vRxNumber = getRefValue(vRefWhereCls, dbEM);

		DrugId rowDrugId = null;
		Query getDrugIdQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM DRUG_ID WHERE PRFSN_SERVICE_KEY=:prfnServiceKey",
						DrugId.class);
		getDrugIdQuery.setParameter("prfnServiceKey", vSvcKey);
		List drugIds = getDrugIdQuery.getResultList();
		Iterator drugIdIterator = drugIds.iterator();
		if (drugIdIterator.hasNext()) {
			rowDrugId = (DrugId) drugIdIterator.next();
			vNdcCd = rowDrugId.getLin03natlDrugCd();
		}

		DrugPricing rowDrugPricing = null;
		Query getDrugPricingQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM DRUG_PRICING WHERE PRFSN_SERVICE_KEY=:prfnServiceKey",
						DrugPricing.class);
		getDrugPricingQuery.setParameter("prfnServiceKey", vSvcKey);
		List drugPricings = getDrugPricingQuery.getResultList();
		Iterator drugPricingIterator = drugPricings.iterator();
		if (drugPricingIterator.hasNext()) {
			rowDrugPricing = (DrugPricing) drugPricingIterator.next();
			writeF021Rec(rowDrugPricing, vNdcCd, vRxNumber, vOutputFile);
		}
	}

	/**
	 * Drives the processing of F040(OHI Svc Line Info) records both the data
	 * aquisition and writing the data to the extract file.
	 * 
	 * @param vClaimKey
	 * @param vSvcKey
	 * @param vOutputFile
	 * @param dbEM
	 */
	protected void processF040Rec(String vClaimKey, String vPrfsnKey,
			PrintWriter vOutputFile, EntityManager dbEM) {
		// The following is the code to create F040 Record - OHI Service Line
		// level Info
		final String vSvcKey = "PRFSN_SERVICE_KEY = '" + vPrfsnKey
		+ "' ";
		SvcLineAdjdct rowSvcLineAdjdct = null;

		//Query getSvcLineAdjdctQuery = dbEM.createNativeQuery(
		//		"SELECT * FROM SVC_LINE_ADJDCT WHERE " + vSvcKey+ " order by SEQ_NBR", SvcLineAdjdct.class);
    String svcLindAdjudicationQueryString = "SELECT * FROM SVC_LINE_ADJDCT WHERE " + vSvcKey+ " order by SEQ_NBR";
    //summaryLOGGER.info("Query String: " + svcLindAdjudicationQueryString);
    Query getSvcLineAdjdctQuery = dbEM.createNativeQuery(svcLindAdjudicationQueryString, SvcLineAdjdct.class);
		
		List svcLineAdjdcts = getSvcLineAdjdctQuery.getResultList();
		Iterator svcLineAdjdctIterator = svcLineAdjdcts.iterator();
		String vDate = null;
//TWP: TODO: Richard: Please take a look at this should the service dates/pat remaining amount be specific to an OHI?
//They are not in this code		
		final String vSvcAdjDt = getDate(vClaimKey, "AND " + vSvcKey, "573",dbEM);			
		final BigDecimal vRemainingPatientAmount = getClaimAmt( vClaimKey, "AND " + vSvcKey,"EAF", dbEM);
	
		while (svcLineAdjdctIterator.hasNext()) {
			rowSvcLineAdjdct = (SvcLineAdjdct) svcLineAdjdctIterator.next();
			writeF040Rec(rowSvcLineAdjdct, vSvcAdjDt, vRemainingPatientAmount, vOutputFile);
			Cases rowCases = null;
			Query getCasesQuery = dbEM
					.createNativeQuery("SELECT * FROM CASES WHERE PRFSN_SERVICE_KEY=:prfnServiceKey and LOOP_SEQ_NBR=:loopSeqNbr order by SEQ_NBR",Cases.class);
			getCasesQuery.setParameter("prfnServiceKey", vPrfsnKey);
			getCasesQuery.setParameter("loopSeqNbr", rowSvcLineAdjdct.getSeqNbr()+"");

			List cases = getCasesQuery.getResultList();
			Iterator caseIterator = cases.iterator();
			while (caseIterator.hasNext()) {
				rowCases = (Cases) caseIterator.next();
				writeCasesRec(rowCases, "F050", vOutputFile);				
			}
			detach(cases, dbEM);
		}
		detach(svcLineAdjdcts, dbEM);
	} // end
	
	/**
	 * Utility method that allows a caller to get an attribute on an object via
	 * java reflection.
	 * 
	 * @param attrName
	 * @param target
	 * @return
	 */
	private Object getAttribute(String attrName, Object target) {
		Object returnObject = null;
		try {
			Class targetClass = target.getClass();
			String methodName = "get" + attrName;
			Method targetGetMethod = targetClass.getMethod(methodName);
			returnObject = targetGetMethod.invoke(target, (Object[]) null);
		} catch (Exception e) {
			throw new RuntimeException(
					"Could not access necessary data via reflection in ExtractInstClaims",
					e);
		}
		return returnObject;
	}

}