package gov.va.med.hac.edi.reconciliation;

import java.util.Collection;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gov.va.med.domain.fppsowner.Claim;
import gov.va.med.domain.fppsowner.ClaimProcedure;
import gov.va.med.domain.fppsowner.ClaimStatus;
import gov.va.med.domain.fppsowner.FeeClaimPaymentLines;
import gov.va.med.domain.fppsowner.FeeClaimPayments;
import gov.va.med.domain.fppsowner.FeeErrorCode;
import gov.va.med.domain.fppsowner.FeePaymentErrors;
import gov.va.med.domain.fppsowner.FeePaymentLineErrors;
import gov.va.med.domain.fppsowner.WsFacStatus;

class Helper {

   static final String FAILED_TO_MATCH_CLAIM_ID = "1";
   static final String FAILED_TO_MATCH_LINE_ID = "2";
   static final String FAILED_TO_MATCH_AMOUNT = "3";
   static final String DUPLICATE_LINE_ADJUDICATION = "4";
   static final String DATA_ERROR = "5";
   static final String NON = "NON";
   static final short  CLAIM_ADJUDICATIONS_CALC_METHOD_CODE = -1;
   static final String RESOLUTION_STATUS_CODE_RESOLVE = "RESOLVE";
   static final String RESOLUTION_STATUS_CODE_ERROR = "ERROR";
   static final String COMPLETED_LINE_STATUS_CODE = "COMPLETE";
   
   // original value - will not catch ANYTHING.
   static final double EPSILON = 0.005;

   private static final Logger LOGGER = LoggerFactory.getLogger(OtherTypeReconciliation.class);
   
   private Helper() { };

   static void recordPaymentError(String feeClaimPaymentId, String errorCode, String username, EntityManager entityManager) {
	   
      EntityTransaction tx = null;
      try {
         tx = entityManager.getTransaction();
         tx.begin();
         markPaymentError(feeClaimPaymentId, errorCode, username, entityManager);
         
         LOGGER.info("\nHelper.recordPaymentError() - recording error for FeeClaimPayment: " + feeClaimPaymentId);
     
          
         tx.commit();
         
      } catch (Exception e) {
         Throwable t =  ExceptionUtils.getRootCause(e);
         LOGGER.warn("Unable to update payment: {0} due to {1} "
               , feeClaimPaymentId, t == null ? e.getMessage() : t.getMessage());
         if (tx != null) {
            tx.rollback();
         }
         
         
         markAsFailure(feeClaimPaymentId, username, entityManager);        
      }
   }



   static void recordPaymentLineErrors(String feeClaimPaymentId, Collection<FeeClaimPaymentLines> paymentErrors
                                       , String errorCode, String username, EntityManager entityManager) {
	   
      EntityTransaction tx = null;
      
      try {
         tx = entityManager.getTransaction();
         tx.begin();
         
         markPaymentError(feeClaimPaymentId, errorCode, username, entityManager);
         markPaymentLineErrors(feeClaimPaymentId, paymentErrors, errorCode, username, entityManager);
         
       	LOGGER.info("\nHelper.recordPaymentLineErrors() - recording error for FeeClaimPayment: " + feeClaimPaymentId);
             
         tx.commit();
       } catch (Exception e) {
    	 LOGGER.info("Helper.recordPaymentLineErrors() ERROR: " + e.getMessage());
     	   
         Throwable t =  ExceptionUtils.getRootCause(e);
         LOGGER.warn("Unable to update payment: {0} due to {1} "
               , feeClaimPaymentId, t == null ? e.getMessage() : t.getMessage());
         
         if (tx != null) {
            tx.rollback();
         }
          
         markAsFailure(feeClaimPaymentId, username, entityManager);        
      }

   }



   /**
    * Update the WS_FAC_STATUS table with completion information
    * @param claimId
    * @param username
    * @param entityManager
    */
   static void closeIfClaimComplete(Long claimId, String username, EntityManager entityManager) {
	   
      java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());
      EntityTransaction tx = null;

      try {
          tx = entityManager.getTransaction();
          tx.begin();
         
         
         Query qclm = entityManager.createQuery("select clm from foClaim clm where clm.claimIndex = :index");
         qclm.setParameter("index", claimId);
         Claim claim = (Claim) qclm.getSingleResult();
           
         Query qwsfac = entityManager.createQuery("select ws from foWsFacStatus ws where ws.claimIndex = :index ");
         qwsfac.setParameter("index", claim.getClaimIndex());
         WsFacStatus wsfac = (WsFacStatus) qwsfac.getSingleResult();
         
         boolean complete = true;
         
         // get the procedures for the claim
         Collection<ClaimProcedure> procs = claim.getClaimProcedures();
          
         for (ClaimProcedure proc : procs) {
        	 
            if (proc.getClaimProcRecons().size() == 0) {
               complete = false;
               break;
            }
         }
         
         
         if (complete) {
            Query qclmstat = entityManager.createQuery("select s from foClaimStatus s where s.claimStatusCd = :code ");
            qclmstat.setParameter("code", "COMPLETE");
            
            
            ClaimStatus clmstat = (ClaimStatus) qclmstat.getSingleResult();

            claim.setClaimStatus(clmstat);
            claim.setModifiedBy(username);
            claim.setDateModified(today);
            
            wsfac.setClaimStatusCd("COMPLETE");
          }
         
         
         wsfac.setLinesClosed(new Long(claim.getClaimProcRecons().size()));
         
         LOGGER.info("\nHelper.closeIfClaimComplete() setting claim: " + claim.getClaimIndex() + " to COMPLETE");
           
         tx.commit();
         LOGGER.debug("Set Claim {0} to COMPLETE.", claimId);
         
         
      } catch (Exception e) {
          Throwable t = ExceptionUtils.getRootCause(e);
          LOGGER.warn("Unable to close claim {0} due to {1}", claimId, t != null ? t.getMessage() : e.getMessage());
         if (tx != null) {
            tx.rollback();
         }
      }
      
      
   }	// end closeIfClaimComplete()


   static void markPaymentError(String feeClaimPaymentId, String errorCode, String username, EntityManager entityManager) {
	   
	     boolean DEBUG = true;
	   
         java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());
         javax.persistence.Query qfec = entityManager.createQuery("select fec from foFeeErrorCode fec where fec.feeErrorCd = :code ");
         qfec.setParameter("code", errorCode);
         FeeErrorCode fec = (FeeErrorCode) qfec.getSingleResult();

         // mark non-claim and non-line type payments as errors from VITRIA.
         Query q1 = entityManager.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id");
         q1.setParameter("id", feeClaimPaymentId);
         FeeClaimPayments fcp = (FeeClaimPayments) q1.getSingleResult();
         FeePaymentErrors fpe = new FeePaymentErrors();
         
         
         fpe.setFeeErrorCode(fec);
         fpe.setCreatedBy(username);
         fpe.setDateCreated(today);
         fpe.setErrorDate(today);
         fpe.setFeeClaimPayments(fcp);
         fcp.getFeePaymentErrors().add(fpe);

         // update fee claim payment status
         fcp.setResolutionStatusCode(RESOLUTION_STATUS_CODE_ERROR);
         fcp.setModifiedBy(username);
         fcp.setDateModified(today);
         
         // this message is wrong 
         LOGGER.debug("Payment Error Recorded for Claim: {0} UNKNOWN TRANSACTION CODE: {1} "
               , feeClaimPaymentId, fcp.getTransactionTypeCode()); 
          
         if(DEBUG) {
         	System.out.println("markPaymentError()- Payment Error Recorded for Claim: " + feeClaimPaymentId + ", " + fcp.getTransactionTypeCode());
         }
         
       }



   static void markPaymentLineErrors(String feeClaimPaymentId,  Collection<FeeClaimPaymentLines> fcpls, String errorCode
                                     , String username,  EntityManager entityManager) {
	   
      java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());  
      Query qfcp = entityManager.createQuery("select fcp from foFeeClaimPayments fcp where fcp.feeClaimPaymentId = :id ");
      qfcp.setParameter("id", feeClaimPaymentId);
      FeeClaimPayments fcp = (FeeClaimPayments) qfcp.getSingleResult();
      
      for (FeeClaimPaymentLines fcpl : fcpls) {
         FeePaymentLineErrors fple = new FeePaymentLineErrors();
         fple.setLineItemSequence(fcpl.getLineItemSequence());
         fple.setFeeErrorCode(errorCode);
         fple.setFeeClaimPaymentId(feeClaimPaymentId);
         fple.setCreatedBy(username);
         fple.setDateCreated(today);
         fple.setFeeClaimPaymentLines(fcpl);
         fcpl.getFeePaymentLineErrors().add(fple);
      }
   }



   static void markAsFailure(String id, String username, EntityManager entityManager) {
	   
      java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());
      EntityTransaction tx = null;
      try {
         tx = entityManager.getTransaction();
         tx.begin();
         Query q = entityManager.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id ");
         q.setParameter("id", id);
         FeeClaimPayments fcp = (FeeClaimPayments) q.getSingleResult();
         fcp.setResolutionStatusCode("UPDATE_FAILURE");
         fcp.setModifiedBy(username);
         fcp.setDateModified(today);
         
            
         tx.commit();
         
       } catch (Exception e) {
      	   
         if (tx != null) {
            tx.rollback();
         }
         Throwable t =  ExceptionUtils.getRootCause(e);
         LOGGER.warn("Unable to mark payment {0} failure due to {1} "
                     , id, t == null ? e.getMessage() : t.getMessage());
      }
   }

   

}
