package gov.va.med.hac.edi.reposload.load278.jpa;

import gov.va.med.hac.edi.utils.jpa.*;
import java.io.*;
import javax.persistence.EntityManager;
import org.apache.commons.cli.*;
import org.slf4j.*;

public class X12278Load {
  // Use two loggers, one for a summary file and one for a trace file.
  static final Logger summaryLOGGER = LoggerFactory.getLogger(X12278Load.class);
  static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + X12278Load.class);

  // private constructor to prevent unneeded instance creation.
  private X12278Load() {
  }
  
  /**
   * Entry point for the X12278Load process. The JPA connection configuration file name can be passed
   * into this method which will override the normal persistence.xml file found in the META-INF
   * directory of the jar file.  (TWP: WE WILL OVERRIDE IT FOR THIS PROCESS TO USE NEW SCHEMA 
   * The process requires the -p parameter to be set.  -p should be set to the path of the 
   * the .DAT files (837 data) to load. 
   *  
   * <code>usage: X12278Load [-f &lt;file&gt;] [-h] -t type</code>
   * 
   * @param args
   *          set of options where &lt;file&gt; is the full path name of the JPA configuration file.
   */
  public static void main(String[] args) {
    // Parse the command line
    CommandLine line = null;
    Options opts = getCliOptions();
    try {
      CommandLineParser parser = new GnuParser();
      line = parser.parse(opts, args);
      // If the "help" option is specified, just show usage options and exit
      if (line.hasOption("h")) {
        printUsage(opts, args, false);
        System.exit(1);
      }
      // If any required command line parameter is missing, just show usage options and exit
    } catch (MissingOptionException moe) {
      printUsage(opts, args, true);
      System.exit(1);
    } catch (ParseException pe) {
      System.out.println("Problem parsing the command line");
      printUsage(opts, args, false);
      System.exit(1);
    }

    // Catch exceptions, log them, and swallow them to allow other things to run afterwards
    boolean processSuccess = false;
    String sourcePath = "";
    String configFilePath = "";
    EntityManager em = null;
    try {
      // Start the log file
      LogUtils.stampStartingTime("X12278Load", summaryLOGGER);
      summaryLOGGER.info("Code Version: 5010");

      // Get an EntityManager, based on the -f option if passed in, otherwise based on a default
      File configFile = null;
      configFilePath = line.getOptionValue("f");
      if (configFilePath != null && configFilePath.length() > 0) {
        configFile = JPAUtil.getJpaConfigFile(configFilePath);
      } else {
        configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
      }
      em = JPAUtil.getEntityManager(configFile, "preAuth");

      // If parameters are OK, run this process
      sourcePath = line.getOptionValue("p");
      if (sourcePath != null && sourcePath.length() > 0 ) {
        summaryLOGGER.info("Source directory from parameters: " + sourcePath);
        process278LoadNew pLoad = new process278LoadNew();
        pLoad.processLoad( sourcePath, em);

      } else {
        // If parameters are not OK, just log
        summaryLOGGER
            .error("X12278Load: Please pass source directory as parameter.");
      }

      // Put a completion message into the log file
      summaryLOGGER.info("Process completed successfully");
      LogUtils.stampEndingTime("X12278Load", summaryLOGGER);

      // Stop the process failure method from going into the log file in the finally block
      processSuccess = true;

    } catch (OutOfMemoryError e) {
      summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
    } catch (Throwable t) {
      summaryLOGGER.error("Exception or Error Thrown During Processing", t);
    } finally {
      if (!processSuccess) {
        summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
      }
    }
  }

  private static void printUsage(Options opts, String[] inArgs, boolean missingArg) {
    final int helpwidth = 80;
    HelpFormatter fmtr = new HelpFormatter();
    System.out.println();
    if (missingArg) {
      System.out.println("Some required commandline argument was missing");
    }
    fmtr.printHelp(helpwidth, "X12278Load", "", opts, "", true);
    if (missingArg) {
      System.out.println(inArgs.length + " command line arguments were received: ");
      for (int i = 0; i < inArgs.length; i++) {
        System.out.println(inArgs[i]);
      }
    }
  }

  /**
   * Helper method to populate command line options objects.
   * 
   * @return the <code>Options</code> object containing the set of command line option.
   */
  @SuppressWarnings("static-access")
  private static Options getCliOptions() {
    Options options = new Options();

    Option configfile = OptionBuilder.withArgName("jpaFile").hasArg().withLongOpt("jpaFile")
        .withDescription("The JPA Configuration file, such as hibernate.properties.").create("f");

    Option sourcePath = OptionBuilder.withArgName("sourcePath").hasArg().isRequired().
        withLongOpt("sourcePath").withDescription("The path to input files.").create("p");

    Option help = OptionBuilder.withLongOpt("help").withDescription("Print this message.").create(
        "h");

    options.addOption(help);
    options.addOption(configfile);
    options.addOption(sourcePath);

    return options;
  }
}