package gov.va.med.hac.edi.reposload.ncpdp.jpa;

import gov.va.med.domain.erepos.EdiProcessLog;
import gov.va.med.domain.erepos.NcpdpCobOtPayer;
import gov.va.med.domain.erepos.batchload.*;
import gov.va.med.hac.edi.utils.jpa.ConversionUtil;
import gov.va.med.hac.edi.utils.jpa.DateUtil;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.FileUtils;
import gov.va.med.hac.edi.utils.jpa.LoggingException;
import gov.va.med.domain.erepos.*;

import java.io.IOException;
import java.sql.SQLException;
import javax.persistence.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.math.BigDecimal;
import java.util.UUID;

public class ProcessNCPDPLoad {
  static final Logger  LOGGER      = LoggerFactory.getLogger(ProcessNCPDPLoad.class);
  static final Logger  traceLOGGER = LoggerFactory.getLogger("trace." + ProcessNCPDPLoad.class);

  private final String DELIMITER   = "\\^";

  /**
   * processLoad is the top-level method used to drive the loading of data into the NCPDP tables.
   * The data is loaded from .DAT files in the directory identified by the filePath parameter. Each
   * .DAT is a flat file representation of a single database table. If a data file is not present it
   * will not be loaded into the DB. This method sets up a database connection according to
   * dbHibernate.properties. It then calls several helper methods to load data, one for each .DAT
   * file available Hibernate is used to perform the actual inserts.
   * 
   * Note: Referential integrity is enforced by the db, not by hibernate. Missing data might not
   * produce an error until a flush or commit command is issued.
   * 
   * Note: Although this code uses JPA we are mapping foreign keys directly That is, we are not
   * associating objects with one another. We accomplish this via the DAO subclasses and the
   * mappings found in ormidsmapped.xml For Example, in the line
   * crNcpdpHeader.setBatchKey(tokens[1]); we are setting the foreign key of the batch object
   * Compare to the usual usage of Hibernate, in which we would set the batch object, as in
   * crNcpdpHeader.setBatch(ncpdpBatch); We have departed from the typical hibernate semantics in
   * keeping with the legacy code
   * 
   * @param filePath
   *          - path to the .DAT files
   * @throws LoggingException
   *           - this exception is thrown if there is a problem outside of a database transaction
   * @throws DbTransactionEnderException
   *           - this exception is throw to explicitly signal to the caller - that something went
   *           wrong and a database transaction was rolled back.
   */
  protected void processLoad(String filePath, EntityManager em) throws DbTransactionEnderException,
      LoggingException {
    boolean exists = false; // Reused many times
    String vFileNumber; // Used in each following block

    // Process the files
    try {
      // Put everything into the hibernate session and commit it later once the file has been read
      em.getTransaction().begin();

      // The first file to process is NCPDP_BATCH.DAT.
      BufferedReader batchFileReader = FileUtils.readFile(filePath + "NCPDP_BATCH.DAT");

      // Get the mandatory single line out of the batch file
      String vLoadRec = batchFileReader.readLine();
      if (vLoadRec == null || vLoadRec.length() == 0) {
        throw new IOException("Batch File was empty");
      }
      batchFileReader.close();

      // Get the FileNumber out of the batch line
      vFileNumber = ConversionUtil.getFirstToken(vLoadRec, DELIMITER);

      // Check that a batch with this File Number was not already loaded
      if (isBatchAlreadyLoaded(em, vFileNumber)) {
        throw new Exception("Batch: " + vFileNumber + " was already loaded");
      }

      // Since the batch was not already loaded -- load it
      batchFileReader = FileUtils.readFile(filePath + "NCPDP_BATCH.DAT");
      loadNcpdpBatch(batchFileReader, em);
      batchFileReader.close();

      // If a data file exists that has header information in it exists, load the header table.
      exists = (new File(filePath + "NCPDP_HEADER.DAT")).exists();
      if (exists) {
        BufferedReader headerFileReader = FileUtils.readFile(filePath + "NCPDP_HEADER.DAT");
        loadNcpdpHeader(headerFileReader, em);
        headerFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_HEADER loaded");
      }

      // If the patient file exists, load it into the database
      exists = (new File(filePath + "NCPDP_PATIENT.DAT")).exists();
      if (exists) {
        BufferedReader patientFileReader = FileUtils.readFile(filePath + "NCPDP_PATIENT.DAT");
        loadNcpdpPatient(patientFileReader, em);
        patientFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_PATIENT loaded");
      }

      // If the insurance file exists, load it into the database
      exists = (new File(filePath + "NCPDP_INSURANCE.DAT")).exists();
      if (exists) {
        BufferedReader insuranceFileReader = FileUtils.readFile(filePath + "NCPDP_INSURANCE.DAT");
        loadNcpdpInsurance(insuranceFileReader, em);
        insuranceFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_INSURANCE loaded");
      }

      // If the claim file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CLAIM.DAT")).exists();
      if (exists) {
        BufferedReader claimFileReader = FileUtils.readFile(filePath + "NCPDP_CLAIM.DAT");
        loadNcpdpClaim(claimFileReader, em);
        claimFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CLAIM loaded");
      }

      // If the claim_proc_med file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CLAIM_SUB_CLARIF.DAT")).exists();
      if (exists) {
        BufferedReader claimSubClarifFileReader = FileUtils.readFile(filePath
            + "NCPDP_CLAIM_SUB_CLARIF.DAT");
        loadNcpdpClaimSubClarif(claimSubClarifFileReader, em);
        claimSubClarifFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CLAIM_SUB_CLARIF loaded");
      }

      
      // If the claim_proc_med file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CLAIM_PROC_MOD.DAT")).exists();
      if (exists) {
        BufferedReader claimProcedureFileReader = FileUtils.readFile(filePath
            + "NCPDP_CLAIM_PROC_MOD.DAT");
        loadNcpdpClaimProcMod(claimProcedureFileReader, em);
        claimProcedureFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CLAIM_PROC_MOD loaded");
      }

      // If the provider file exists, load it into the database
      exists = (new File(filePath + "NCPDP_PROVIDER.DAT")).exists();
      if (exists) {
        BufferedReader providerFileReader = FileUtils.readFile(filePath + "NCPDP_PROVIDER.DAT");
        loadNcpdpProvider(providerFileReader, em);
        providerFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_PROVIDER loaded");
      }

      // If the prescriber file exists, load it into the database
      exists = (new File(filePath + "NCPDP_PRESCRIBER.DAT")).exists();
      if (exists) {
        BufferedReader prescriberFileReader = FileUtils.readFile(filePath + "NCPDP_PRESCRIBER.DAT");
        loadNcpdpPrescriber(prescriberFileReader, em);
        prescriberFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_PRESCRIBER loaded");
      }

      // If the cob file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COB.DAT")).exists();
      if (exists) {
        BufferedReader cobFileReader = FileUtils.readFile(filePath + "NCPDP_COB.DAT");
        loadNcpdpCob(cobFileReader, em);
        cobFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COB loaded");
      }

      // If the cob_ot_payer file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COB_OT_PAYER.DAT")).exists();
      if (exists) {
        BufferedReader cobOtPayerFileReader = FileUtils.readFile(filePath
            + "NCPDP_COB_OT_PAYER.DAT");
        loadNcpdpCobOtPayer(cobOtPayerFileReader, em);
        cobOtPayerFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COB_OT_PAYER loaded");
      }
      
      // If the cob_op_pat_resp_amount file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COB_OP_PAT_RESP_AMOUNT.DAT")).exists();
      if (exists) {
        BufferedReader cobOpPatRespAmountFileReader = FileUtils.readFile(filePath
            + "NCPDP_COB_OP_PAT_RESP_AMOUNT.DAT");
        loadNcpdpCobOpPatRespAmount(cobOpPatRespAmountFileReader, em);
        cobOpPatRespAmountFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COB_OP_PAT_RESP_AMOUNT loaded");
      }

      // If the cob_bene_stg_amount file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COB_BENE_STG_AMOUNT.DAT")).exists();
      if (exists) {
        BufferedReader cobBeneStgAmountFileReader = FileUtils.readFile(filePath
            + "NCPDP_COB_BENE_STG_AMOUNT.DAT");
        loadNcpdpBeneStgAmount(cobBeneStgAmountFileReader, em);
        cobBeneStgAmountFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COB_BENE_STG_AMOUNT loaded");
      }

      // If the cob_ot_payer_amount file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COB_OT_PAYER_AMOUNT.DAT")).exists();
      if (exists) {
        BufferedReader cobOtPayerAmountFileReader = FileUtils.readFile(filePath
            + "NCPDP_COB_OT_PAYER_AMOUNT.DAT");
        loadNcpdpCobOtPayerAmount(cobOtPayerAmountFileReader, em);
        cobOtPayerAmountFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COB_OT_PAYER_AMOUNT loaded");
      }

      // If the cob_ot_payer_reject file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COB_OT_PAYER_REJECT.DAT")).exists();
      if (exists) {
        BufferedReader cobOtPayerRejectFileReader = FileUtils.readFile(filePath
            + "NCPDP_COB_OT_PAYER_REJECT.DAT");
        loadNcpdpCobOtPayerReject(cobOtPayerRejectFileReader, em);
        cobOtPayerRejectFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COB_OT_PAYER_REJECT loaded");
      }

      // If the work comp file exists, load it into the database
      exists = (new File(filePath + "NCPDP_WORK_COMP.DAT")).exists();
      if (exists) {
        BufferedReader workCompFileReader = FileUtils.readFile(filePath + "NCPDP_WORK_COMP.DAT");
        loadNcpdpWorkComp(workCompFileReader, em);
        workCompFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_WORK_COMP loaded");
      }

      // If the dur pps file exists, load it into the database
      exists = (new File(filePath + "NCPDP_DUR_PPS.DAT")).exists();
      if (exists) {
        BufferedReader durPpsFileReader = FileUtils.readFile(filePath + "NCPDP_DUR_PPS.DAT");
        loadNcpdpDurPps(durPpsFileReader, em);
        durPpsFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_DUR_PPS loaded");
      }

      // If the pricing file exists, load it into the database
      exists = (new File(filePath + "NCPDP_PRICING.DAT")).exists();
      if (exists) {
        BufferedReader pricingFileReader = FileUtils.readFile(filePath + "NCPDP_PRICING.DAT");
        loadNcpdpPricing(pricingFileReader, em);
        pricingFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_PRICING loaded");
      }

      // If the pricing amount file exists, load it into the database
      exists = (new File(filePath + "NCPDP_PRICING_OT_AMOUNT.DAT")).exists();
      if (exists) {
        BufferedReader pricingOtAmountFileReader = FileUtils.readFile(filePath
            + "NCPDP_PRICING_OT_AMOUNT.DAT");
        loadNcpdpPricingOtAmount(pricingOtAmountFileReader, em);
        pricingOtAmountFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_PRICING_OT_AMOUNT loaded");
      }

      // If the coupon file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COUPON.DAT")).exists();
      if (exists) {
        BufferedReader couponFileReader = FileUtils.readFile(filePath + "NCPDP_COUPON.DAT");
        loadoNcpdpCoupon(couponFileReader, em);
        couponFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COUPON loaded");
      }

      // If the compound file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COMPOUND.DAT")).exists();
      if (exists) {
        BufferedReader compoundFileReader = FileUtils.readFile(filePath + "NCPDP_COMPOUND.DAT");
        loadNcpdpCompound(compoundFileReader, em);
        compoundFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COMPOUND loaded");
      }

      // If the compound_ingr file exists, load it into the database
      exists = (new File(filePath + "NCPDP_COMPOUND_INGR.DAT")).exists();
      if (exists) {
        BufferedReader compoundIngrFileReader = FileUtils.readFile(filePath
            + "NCPDP_COMPOUND_INGR.DAT");
        loadNcpdpCompoundIngr(compoundIngrFileReader, em);
        compoundIngrFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_COMPOUND_INGR loaded");
      }

      // If the prior_auth file exists, load it into the database
      exists = (new File(filePath + "NCPDP_PRIOR_AUTH.DAT")).exists();
      if (exists) {
        BufferedReader priorAuthFileReader = FileUtils.readFile(filePath + "NCPDP_PRIOR_AUTH.DAT");
        loadNcpdpPriorAuth(priorAuthFileReader, em);
        priorAuthFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_PRIOR_AUTH loaded");
      }

      // If the clinical file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CLINICAL.DAT")).exists();
      if (exists) {
        BufferedReader clinicalFileReader = FileUtils.readFile(filePath + "NCPDP_CLINICAL.DAT");
        loadNcpdpClinical(clinicalFileReader, em);
        clinicalFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CLINICAL loaded");
      }

      // If the clinical_diag file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CLINICAL_DIAG.DAT")).exists();
      if (exists) {
        BufferedReader clinicalDiagFileReader = FileUtils.readFile(filePath
            + "NCPDP_CLINICAL_DIAG.DAT");
        loadNcpdpClinicalDiag(clinicalDiagFileReader, em);
        clinicalDiagFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CLINICAL_DIAG loaded");
      }

      // If the clinical_mes file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CLINICAL_MEAS.DAT")).exists();
      if (exists) {
        BufferedReader clinicalMeasFileReader = FileUtils.readFile(filePath
            + "NCPDP_CLINICAL_MEAS.DAT");
        loadNcpdpClinicalMeas(clinicalMeasFileReader, em);
        clinicalMeasFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CLINICAL_MEAS loaded");
      }
      
      
      
   // If the adtl_doc file exists, load it into the database
      exists = (new File(filePath + "NCPDP_ADTL_DOCUMENT.DAT")).exists();
      if (exists) {
        BufferedReader adtlDocumentFileReader = FileUtils.readFile(filePath
            + "NCPDP_ADTL_DOCUMENT.DAT");
        loadNcpdpAdtlDocument(adtlDocumentFileReader, em);
        adtlDocumentFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_ADTL_DOCUMENT loaded");
      }
      
   // If the adtl_doc_qstn_nbr file exists, load it into the database
      exists = (new File(filePath + "NCPDP_ADTL_DOC_QSTN_NBR.DAT")).exists();
      if (exists) {
        BufferedReader adtlDocQstnNbrFileReader = FileUtils.readFile(filePath
            + "NCPDP_ADTL_DOC_QSTN_NBR.DAT");
        loadNcpdpAdtlDocQstnNbr(adtlDocQstnNbrFileReader, em);
        adtlDocQstnNbrFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_ADTL_DOC_QSTN_NBR loaded");
      }
      
   // If the facility file exists, load it into the database
      exists = (new File(filePath + "NCPDP_FACILITY.DAT")).exists();
      if (exists) {
        BufferedReader facilityFileReader = FileUtils.readFile(filePath
            + "NCPDP_FACILITY.DAT");
        loadNcpdpFacility(facilityFileReader, em);
        facilityFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_FACILITY loaded");
      }
      
   // If the narritive file exists, load it into the database
      exists = (new File(filePath + "NCPDP_NARRATIVE.DAT")).exists();
      if (exists) {
        BufferedReader narrativeFileReader = FileUtils.readFile(filePath
            + "NCPDP_NARRATIVE.DAT");
        loadNcpdpNarrative(narrativeFileReader, em);
        narrativeFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_NARRATIVE loaded");
      }
      
   //  If the cmp_ingdt_mod file exists, load it into the database
      exists = (new File(filePath + "NCPDP_CMP_INGDT_MOD.DAT")).exists();
      if (exists) {
        BufferedReader cmpIngdtFileReader = FileUtils.readFile(filePath
            + "NCPDP_CMP_INGDT_MOD.DAT");
        loadNcpdpCmpIngdtMod(cmpIngdtFileReader, em);
        cmpIngdtFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_CMP_INGDT_MOD loaded");
      }
      
   // If the response_process file exists, load it into the database
      exists = (new File(filePath + "NCPDP_RESPONSE_PROCESS.DAT")).exists();
      if (exists) {
        BufferedReader responseProcessFileReader = FileUtils.readFile(filePath
            + "NCPDP_RESPONSE_PROCESS.DAT");
        loadNcpdpResponseProcess(responseProcessFileReader, em);
        responseProcessFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_RESPONSE_PROCESS loaded");
      }
      
   // If the b1_claim_status file exists, load it into the database
      exists = (new File(filePath + "NCPDP_RB1_CLAIM_STATUS.DAT")).exists();
      if (exists) {
        BufferedReader rb1ClaimStatusFileReader = FileUtils.readFile(filePath
            + "NCPDP_RB1_CLAIM_STATUS.DAT");
        loadNcpdpRb1ClaimStatus(rb1ClaimStatusFileReader, em);
        rb1ClaimStatusFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_RB1_CLAIM_STATUS loaded");
      }
      
   // If the b2_reversal_status file exists, load it into the database
      exists = (new File(filePath + "NCPDP_RB2_REVERSAL_STATUS.DAT")).exists();
      if (exists) {
        BufferedReader rb2ReversalStatusFileReader = FileUtils.readFile(filePath
            + "NCPDP_RB2_REVERSAL_STATUS.DAT");
        loadNcpdpRb2ReversalStatus(rb2ReversalStatusFileReader, em);
        rb2ReversalStatusFileReader.close();
        em.flush();
        LOGGER.debug("NCPDP_RB2_REVERSAL_STATUS loaded");
      }
      
      // Now that everything is in hibernate's cache commit the transaction to the db
      em.getTransaction().commit();
      
      // Delete the source files
      deleteNcpdpSourceFiles(filePath);
    } catch (Exception e) {
      // If an exception occurs roll back the data -- note this does not roll back data if the
      // exception occurs when we are logging the number of claims loaded
      LOGGER.error("Rolling back the database transaction");
      if (em != null) {
        if (em.getTransaction().isActive()) {
          em.getTransaction().rollback();
        }
      }
      LOGGER.info("Rollback successful");
      throw new DbTransactionEnderException("Rolled Back the Transaction", e);
    }

    try {
      em.getTransaction().begin();

      // Log to file the number of claims that were loaded
      // TODO: Why not count these on the way, rather than doing a query afterwards?
      Query countQuery = em.createQuery("SELECT COUNT (c.claimKey)"
                                      + "  FROM gov.va.med.domain.erepos.NcpdpClaim c"
                                      + " WHERE c.fileNumber = ? and c.status = 'NEW' ");
      countQuery.setParameter(1, vFileNumber);
      Long claimsLoaded = (Long) countQuery.getSingleResult();
      LOGGER.info("NCPDP Claims loaded for File Number " + vFileNumber + " : " + claimsLoaded);

      // Get a timestamp to associate with the log entry
      String vTransDate = DateUtil.getCurrentDateFormatted("yyyyMMdd");

      // Log to a table the number of claims loaded
      insertProcessLog(claimsLoaded, vTransDate, em);
      
      // resetModifiedBy(em);
      // Commit this logging transaction
      em.getTransaction().commit();

    } catch (Exception e) {
      // Don't roll anything back in this case
      throw new LoggingException("Exception after data was committed", e);
    }
  }

  //This could be an expensive option check performance - it is not necessary to delete DAO Loaded
  //So this can be shut off if it takes too long
  
  private void resetModifiedBy(EntityManager dbEM)
  {
	     String [] tables = {"NCPDP_BATCH","NCPDP_HEADER","NCPDP_PATIENT","NCPDP_INSURANCE",
	     "NCPDP_CLAIM","NCPDP_CLAIM_PROC_MOD","NCPDP_PROVIDER","NCPDP_PRESCRIBER","NCPDP_COB",
	     "NCPDP_COB_OT_PAYER","NCPDP_COB_OT_PAYER_AMOUNT","NCPDP_COB_OT_PAYER_REJECT",
	     "NCPDP_WORK_COMP","NCPDP_DUR_PPS","NCPDP_PRICING","NCPDP_PRICING_OT_AMOUNT",
	     "NCPDP_COUPON","NCPDP_COMPOUND","NCPDP_COMPOUND_INGR","NCPDP_PRIOR_AUTH",
	     "NCPDP_CLINICAL","NCPDP_CLINICAL_DIAG","NCPDP_CLINICAL_MEAS","NCPDP_ADTL_DOCUMENT",
		 "NCPDP_ADTL_DOC_QSTN_NBR","NCPDP_CLAIM_SUB_CLARIF","NCPDP_CMP_INGDT_MOD",
		 "NCPDP_COB_BENE_STG_AMOUNT","NCPDP_COB_OP_PAT_RESP_AMOUNT","NCPDP_FACILITY",
		 "NCPDP_NARRATIVE","NCPDP_RB1_CLAIM_STATUS","NCPDP_RB2_REVERSAL_STATUS",
		 "NCPDP_RESPONSE_PROCESS"};
	    	      	     
	     for(int i=0; i<tables.length; i++)
	     {
	    	 //TODO TWP:add index on modified by if this is how you want to handle it
	    	 String updateSQL = "UPDATE "+tables[i]+" SET MODIFIED_BY = '' where MODIFIED_BY= 'DAO Loaded'";
	    	 final Query updateQuery = dbEM.createNativeQuery(updateSQL);
	    	 updateQuery.executeUpdate();		     	 
	     }
	  
  }
  /**
   * Deletes the .DAT files expected to be in the directory identified by fPath.
   * 
   * @param fPath
   *          - the name of the directory containing the .DAT files.
   */
  private void deleteNcpdpSourceFiles(String fPath) {
    FileUtils.deleteFile(fPath + "NCPDP_BATCH.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_HEADER.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_PATIENT.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_INSURANCE.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_CLAIM.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_CLAIM_PROC_MOD.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_PROVIDER.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_PRESCRIBER.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COB.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COB_OT_PAYER.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COB_OT_PAYER_AMOUNT.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COB_OT_PAYER_REJECT.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_WORK_COMP.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_DUR_PPS.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_PRICING.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_PRICING_OT_AMOUNT.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COUPON.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COMPOUND.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_COMPOUND_INGR.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_PRIOR_AUTH.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_CLINICAL.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_CLINICAL_DIAG.DAT");
    FileUtils.deleteFile(fPath + "NCPDP_CLINICAL_MEAS.DAT");    
	FileUtils.deleteFile(fPath + "NCPDP_ADTL_DOCUMENT.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_ADTL_DOC_QSTN_NBR.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_CLAIM_SUB_CLARIF.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_COMP_INGDT_MOD.DAT");            //MTN021539 corrected from NCPDP_CMP to NCPDP_COMP
	FileUtils.deleteFile(fPath + "NCPDP_COB_BENE_STG_AMOUNT.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_COB_OP_PAT_RESP_AMOUNT.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_FACILITY.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_NARRATIVE.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_RB1_CLAIM_STATUS.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_RB2_REVERSAL_STATUS.DAT");
	FileUtils.deleteFile(fPath + "NCPDP_RESPONSE_PROCESS.DAT");
    LOGGER.info("NCPDP Load Files deleted from load directory");
  }

  /**
   * Returns true if the batch identified by fileNumber is found in the database. TODO: Write a unit
   * test for this. Change the SQL to SELECT COUNT (*)?
   * 
   * @param dbEM
   *          - the database connection / entity manager
   * @param fileNumber
   *          - the fileNumber that identifies a particular batch
   */
  private boolean isBatchAlreadyLoaded(EntityManager dbEM, String fileNumber) {

    // query the database to see if this batch was already loaded
    Query isBatchAlreadyLoadeQuery = dbEM.createQuery("SELECT b.batchKey"
                        + "  FROM gov.va.med.domain.erepos.NcpdpBatch b"
                        + " WHERE b.fileNumber=?1");
    isBatchAlreadyLoadeQuery.setParameter(1, fileNumber);
    try {
      if (isBatchAlreadyLoadeQuery.getSingleResult() != null) {
        return true;
      }
    } catch (NoResultException e) {
      // Just eat this -- it means the batch has not been loaded before.
    }
    return false;
  }

  /**
   * Loads data, read from the BufferedReader passed in, into the NCPDP_BATCH table
   * 
   * @param inReader
   *          - a BufferedFile reader connected to NCPDP_BATCH.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpBatch(BufferedReader inReader, EntityManager em)
      throws IOException {
    NcpdpBatchDAO crNcpdpBatch = null;
    //NcpdpBatch crNcpdpBatch = null;

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = inReader.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpBatch:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens, in this case always the null transformation
      crNcpdpBatch = new NcpdpBatchDAO();
      //crNcpdpBatch = new NcpdpBatch();
      crNcpdpBatch.setFileNumber(tokens[0]);
      crNcpdpBatch.setBatchKey(tokens[1]);
      crNcpdpBatch.setSegmentId(tokens[2]);
      crNcpdpBatch.setTransmType(tokens[3]);
      crNcpdpBatch.setSenderId(tokens[4]);
      crNcpdpBatch.setBatchNumber(tokens[5]);
      crNcpdpBatch.setCreationDate(tokens[6]);
      crNcpdpBatch.setCreationTime(tokens[7]);
      crNcpdpBatch.setFileType(tokens[8]);
      crNcpdpBatch.setVerRelNumber(tokens[9]);
      crNcpdpBatch.setReceiverId(tokens[10]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("ncpdpdbatch fieldvalues {}", crNcpdpBatch);

      // Save the transformed values to the database
      em.persist(crNcpdpBatch);
    }
  }

  /**
   * Loads data, read from the BufferedReader passed in, into the NCPDP_HEADER table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_HEADER.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpHeader(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpHeader file values:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpHeaderDAO crNcpdpHeader = new NcpdpHeaderDAO();
      crNcpdpHeader.setFileNumber(tokens[0]);
      crNcpdpHeader.setBatchKey(tokens[1]);
      crNcpdpHeader.setRealKey(tokens[2]);
      crNcpdpHeader.setTransactionRefNum(tokens[3]);
      crNcpdpHeader.setBinNumber(tokens[4]);
      crNcpdpHeader.setVerRelNum(tokens[5]);
      crNcpdpHeader.setTransactionCode(tokens[6]);
      crNcpdpHeader.setPrserCtlNum(tokens[7]);
      crNcpdpHeader.setTransactionCount(tokens[8]);
      crNcpdpHeader.setSvcPrvQual(tokens[9]);
      crNcpdpHeader.setSvcPrv(tokens[10]);
      crNcpdpHeader.setDateOfService(tokens[11]);
      crNcpdpHeader.setSwVdrCertId(tokens[12]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpHeader insert values:{}", crNcpdpHeader);

      // Save the transformed values to the database
      em.persist(crNcpdpHeader);
    }

  }

  /**
   * Load the data contained in the NCPDP_PATIENT.DAT file into the NCPDP_PATIENT table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_PATIENT.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpPatient(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpPatient:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpPatientDAO crNcpdpPatient = new NcpdpPatientDAO();
      crNcpdpPatient.setFileNumber(tokens[0]);
      crNcpdpPatient.setRealKey(tokens[1]);
      crNcpdpPatient.setPatientKey(tokens[2]);
      crNcpdpPatient.setSegmentId(tokens[3]);
      crNcpdpPatient.setPatIdQual(tokens[4]);
      crNcpdpPatient.setPatId(tokens[5]);
      crNcpdpPatient.setDateOfBirth(tokens[6]);
      crNcpdpPatient.setPatGenderCode(tokens[7]);
      crNcpdpPatient.setPatFirstName(tokens[8]);
      crNcpdpPatient.setPatLastName(tokens[9]);
      crNcpdpPatient.setPatStreetAddr(tokens[10]);
      crNcpdpPatient.setPatCityAddr(tokens[11]);
      crNcpdpPatient.setPatStateProvAddr(tokens[12]);
      crNcpdpPatient.setPatZipPstZone(tokens[13]);
      crNcpdpPatient.setPatPhoneNum(tokens[14]);
      crNcpdpPatient.setPatLocation(tokens[15]);
      crNcpdpPatient.setEmployerId(tokens[16]);
      crNcpdpPatient.setSmkrNonsmkrCd(tokens[17]);
      crNcpdpPatient.setPregnancyInd(tokens[18]);
      if (tokens.length > 19) {
    	  if(ConversionUtil.checkForNumericType(tokens[19]))
    	  {
    		  crNcpdpPatient.setPlaceOfService(ConversionUtil.convertTypeByte(tokens[19]));
    	  }
    	  crNcpdpPatient.setPatEmail(tokens[20]);

    	  //TWP:Check what is coming in from Shem
    	  if(ConversionUtil.checkForNumericType(tokens[21]))
    	  {
    		  crNcpdpPatient.setPatResidence(ConversionUtil.convertTypeByte(tokens[21]));
    	  }

      }
      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpPatient Fieldvalues {}", crNcpdpPatient);

      // Save the transformed values to the database
      em.persist(crNcpdpPatient);
    }
      
  }

  /**
   * Load the data contained in the NCPDP_INSURANCE.DAT file into the NCPDP_INSURANCE table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_INSURANCE.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpInsurance(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpInsurance:" + loadRec);
      NcpdpInsuranceDAO crNcpdpInsurance = new NcpdpInsuranceDAO();

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      crNcpdpInsurance.setFileNumber(tokens[0]);
      crNcpdpInsurance.setRealKey(tokens[1]);
      crNcpdpInsurance.setInsuranceKey(tokens[2]);
      crNcpdpInsurance.setSegmentId(tokens[3]);
      crNcpdpInsurance.setCrdhldrId(tokens[4]);
      crNcpdpInsurance.setCrdhldrFirstName(tokens[5]);
      crNcpdpInsurance.setCrdhldrLastName(tokens[6]);
      crNcpdpInsurance.setHomePlan(tokens[7]);
      crNcpdpInsurance.setPlanId(tokens[8]);
      crNcpdpInsurance.setEligClarifyCd(tokens[9]);
      crNcpdpInsurance.setFacilityId(tokens[10]);
      crNcpdpInsurance.setGroupId(tokens[11]);
      crNcpdpInsurance.setPersonCode(tokens[12]);
      crNcpdpInsurance.setPatRelshpCd(tokens[13]);
      if (tokens.length > 14) {
    	  crNcpdpInsurance.setMedigapId(tokens[14]);
    	  crNcpdpInsurance.setMedicaidInd(tokens[15]);
    	  crNcpdpInsurance.setPrvdrAccptAssgnInd(tokens[16]);
    	  crNcpdpInsurance.setCmsPartdDefQualFac(tokens[17]);
    	  crNcpdpInsurance.setMedicaidIdNbr(tokens[18]);
    	  crNcpdpInsurance.setMedicaidAgencyNbr(tokens[19]);
      }

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpInsurance Fieldvalues {}", crNcpdpInsurance);

      // Save the transformed values to the database
      em.persist(crNcpdpInsurance);
    }
  }

  /**
   * Load the data contained in the NCPDP_CLAIM.DAT file into the NCPDP_CLAIM table
   * 
   * Note:Several values are mapped and translated in this method, unlike in most of the others. We
   * also initialize the status to NEW for each claim.
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_CLAIM.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
	private void loadNcpdpClaim(BufferedReader in, EntityManager em)
			throws IOException {
		int claimCount = 0;

		// Continue loading lines for the entire file
		String loadRec = "";
		while ((loadRec = in.readLine()) != null) {

			// Log into the trace file the values as found in the file being
			// loaded
			traceLOGGER.trace("NcpdpClaim as read:" + loadRec);

			// Split the line into tokens based on a delimiter
			String[] tokens = loadRec.split(DELIMITER, -1);

			// Transform each of the tokens
			NcpdpClaimDAO crNcpdpClaim = new NcpdpClaimDAO();
			String vFileNumber = tokens[0];
			crNcpdpClaim.setFileNumber(vFileNumber);
			crNcpdpClaim.setBatchKey(tokens[1]);
			crNcpdpClaim.setRealKey(tokens[2]);
			crNcpdpClaim.setPatientKey(tokens[3]);
			crNcpdpClaim.setClaimKey(tokens[4]);
			crNcpdpClaim.setSegmentId(tokens[5]);
			crNcpdpClaim.setRxSvcRefNumQual(tokens[6]);
			crNcpdpClaim.setRxSvcRefNum(tokens[7]);
			crNcpdpClaim.setProductSvcIdQual(tokens[8]);
			crNcpdpClaim.setProductSvcId(tokens[9]);
			crNcpdpClaim.setAsoRxSvcRefNum(tokens[10]);
			crNcpdpClaim.setAsoRxSvcDate(tokens[11]);
			crNcpdpClaim.setProcModCdCount(tokens[12]);

			// The Quantity Dispense must be a Long			
			if(ConversionUtil.checkForNumericType(tokens[13]))
			{
				crNcpdpClaim.setQuantityDispensed(Long.parseLong(tokens[13]));
			}

			// Continue with other tokens
			crNcpdpClaim.setFillNumber(tokens[14]);
			crNcpdpClaim.setDaysSupply(tokens[15]);
			crNcpdpClaim.setCompoundCode(tokens[16]);
			crNcpdpClaim.setDawPrdtSelCd(tokens[17]);
			crNcpdpClaim.setDateRxWritten(tokens[18]);
			crNcpdpClaim.setNoOfRefilsAuth(tokens[19]);
			crNcpdpClaim.setRxOriginCd(tokens[20]);

			crNcpdpClaim.setSubClftnCd(tokens[21]);

			// The Quantity Prescribed must be a BigDecimal
			if(ConversionUtil.checkForNumericType(tokens[22]))
			{
				crNcpdpClaim.setQtyPrescribed(new BigDecimal(tokens[22]));
			}

			// Continue with other tokens
			crNcpdpClaim.setOtCvgCd(tokens[23]);
			crNcpdpClaim.setUnitDoseInd(tokens[24]);
			crNcpdpClaim.setOrgPrsPrdSvcQl(tokens[25]);
			crNcpdpClaim.setOrgPrsPrdSvcCd(tokens[26]);

			
			// Another BigDecimal
			if(ConversionUtil.checkForNumericType(tokens[27]))
			{
				crNcpdpClaim.setOrgPrsQty(new BigDecimal(tokens[27]));
			}


			// Continue with other tokens
			crNcpdpClaim.setAlternateId(tokens[28]);
			crNcpdpClaim.setSchRxIdNum(tokens[29]);
			crNcpdpClaim.setUnitOfMeasure(tokens[30]);
			crNcpdpClaim.setLevelOfService(tokens[31]);
			crNcpdpClaim.setPriorAuthTypeCd(tokens[32]);
			crNcpdpClaim.setPriorAuthNumSub(tokens[33]);
			crNcpdpClaim.setIntmdyAuthTypeI(tokens[34]);
			crNcpdpClaim.setIntmdyAuthId(tokens[35]);
			crNcpdpClaim.setDispensingStatus(tokens[36]);

			// Another BigDecimal
			if(ConversionUtil.checkForNumericType(tokens[37]))
			{
				crNcpdpClaim.setQtyIntdTbDisp(new BigDecimal(tokens[37]));
			}

			// Continue with other tokens
			crNcpdpClaim.setDaysSupIntdDisp(tokens[38]);
			crNcpdpClaim.setClaimType(tokens[39]);

			// 

			// The Network Indicator is based on the File Number [40]
			if (vFileNumber.substring(0, 2).equalsIgnoreCase("NN")) {
				crNcpdpClaim.setNetworkIndicator("N");
			} else {
				crNcpdpClaim.setNetworkIndicator("Y");
			}

			// Initialize the status of this claim [41]
			crNcpdpClaim.setStatus("NEW");

			// The Sender ID is also in one of the tokens.
			crNcpdpClaim.setSenderId(tokens[42]);

			//4010 files have 43 elements - 5010 files have more
			if (tokens.length > 43) {
				if (ConversionUtil.checkForNumericType(tokens[43])) {
					crNcpdpClaim.setDelayRsnCd(ConversionUtil
							.convertTypeByte(tokens[43]));
				}

				crNcpdpClaim.setPatAssgnInd(tokens[44]);
				crNcpdpClaim.setRteOfAdmin(tokens[45]);
				crNcpdpClaim.setCompoundType(tokens[46]);

				if (ConversionUtil.checkForNumericType(tokens[47])) {
					crNcpdpClaim.setRxSvcType(ConversionUtil
							.convertTypeByte(tokens[47]));
				}

				if (ConversionUtil.checkForNumericType(tokens[48])) {
					crNcpdpClaim.setSubClarifCdCnt(ConversionUtil
							.convertTypeByte(tokens[48]));
				}

				// TWP:Check this when running
				if (ConversionUtil.checkForNumericType(tokens[49])) {
					crNcpdpClaim.setSpclPckgInd(ConversionUtil
							.convertTypeByte(tokens[49]));

				}
				crNcpdpClaim.setMedicaidSbrgtnIcntcn(tokens[50]);
				// Log into the trace file the transformed values being saved to
				// the database
				// TWP: THESE CHANGED FIX TO STRING
			}
      traceLOGGER.trace("NcpdpClaim Fieldvalues {}", crNcpdpClaim);

      // Save the transformed values to the database
      em.persist(crNcpdpClaim);

      // Flush the Hibernate cache every 300 claims.
      // TODO: Why do this for claims, but for no other objects?
      if ((claimCount++ % 300) == 0) {
        em.flush();
      }
    }
  }

  
  /**
   * Load the data in the NCPDP_CLAIM_SUB_CLARIF.DAT file into the NCPDP_CLAIM_SUB_CLARIF table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_CLAIM_SUB_CLARIF.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpClaimSubClarif(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpClaimSubClarif:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpClaimSubClarifDAO crNcpdpClaimSubClarif = new NcpdpClaimSubClarifDAO();
      crNcpdpClaimSubClarif.setFileNumber(tokens[0]);
      crNcpdpClaimSubClarif.setClaimKey(tokens[1]);
      crNcpdpClaimSubClarif.setSubClarifKey(tokens[2]);
      crNcpdpClaimSubClarif.setSubClarifCd(tokens[3]);
      crNcpdpClaimSubClarif.setSequenceNum(tokens[4]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpClaimProcMod Fieldvalues {}", crNcpdpClaimSubClarif);

      // Save the transformed values to the database
      em.persist(crNcpdpClaimSubClarif);
    }
  }
  
  /**
   * Load the data in the NCPDP_CLAIM_PROC_MOD.DAT file into the NCPDP_CLAIM_PROC_MOD table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_CLAIM_PROC_MOD.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpClaimProcMod(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpClaimProcMod:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpClaimProcModDAO crNcpdpClaimProcMod = new NcpdpClaimProcModDAO();
      crNcpdpClaimProcMod.setFileNumber(tokens[0]);
      crNcpdpClaimProcMod.setClaimKey(tokens[1]);
      crNcpdpClaimProcMod.setProcModKey(tokens[2]);
      crNcpdpClaimProcMod.setProcModCd(tokens[3]);
      crNcpdpClaimProcMod.setSequenceNum(tokens[4]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpClaimProcMod Fieldvalues {}", crNcpdpClaimProcMod);

      // Save the transformed values to the database
      em.persist(crNcpdpClaimProcMod);
    }
  }

  /**
   * Load the data contained in the NCPDP_PROVIDER.DAT file
   * 
   * @param input
   *          - a BufferedFile reader connected to NCPDP_PROVIDER.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpProvider(BufferedReader input, EntityManager em)
      throws IOException {
    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = input.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpProvider:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpProviderDAO newProvider = new NcpdpProviderDAO();
      newProvider.setFileNumber(tokens[0]);
      newProvider.setClaimKey(tokens[1]);
      newProvider.setProviderKey(tokens[2]);
      newProvider.setSegmentId(tokens[3]);
      newProvider.setProviderIdQual(tokens[4]);
      newProvider.setProviderId(tokens[5]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpProvider Fieldvalues {}", newProvider);

      // Save the transformed values to the database
      em.persist(newProvider);
    }
  }

  /**
   * Load the data contained in NCPDP_PRESCRIBER.DAT file into the NCPDP_PRESCRIBER table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_PRESCRIBER.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */

  private void loadNcpdpPrescriber(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpPrescriber:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpPrescriberDAO crNcpdpPrescriber = new NcpdpPrescriberDAO();
      crNcpdpPrescriber.setFileNumber(tokens[0]);
      crNcpdpPrescriber.setClaimKey(tokens[1]);
      crNcpdpPrescriber.setPrescriberKey(tokens[2]);
      crNcpdpPrescriber.setSegmentId(tokens[3]);
      crNcpdpPrescriber.setPrescriberIdQual(tokens[4]);
      crNcpdpPrescriber.setPrescriberId(tokens[5]);
      crNcpdpPrescriber.setPrescriberLocCd(tokens[6]);
      crNcpdpPrescriber.setPrescriberLastNm(tokens[7]);
      crNcpdpPrescriber.setPrescriberPhNum(tokens[8]);
      crNcpdpPrescriber.setPriCarePrvIdQl(tokens[9]);
      crNcpdpPrescriber.setPriCarePrvId(tokens[10]);
      crNcpdpPrescriber.setPriCarePrvLcCd(tokens[11]);
      crNcpdpPrescriber.setPriCarePrvLsNm(tokens[12]);
      if (tokens.length > 13) {
    	  crNcpdpPrescriber.setPrescriberFname(tokens[13]);
    	  crNcpdpPrescriber.setPrescriberAddress(tokens[14]);
    	  crNcpdpPrescriber.setPrescriberCity(tokens[15]);
    	  crNcpdpPrescriber.setPrescriberState(tokens[16]);
    	  crNcpdpPrescriber.setPrescriberZip(tokens[17]);
      }


      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpPrescriber Fieldvalues {}", crNcpdpPrescriber);

      // Save the transformed values to the database
      em.persist(crNcpdpPrescriber);
    }
  }

  /**
   * Load the data contained in NCPDP_COB.DAT file into the NCPDP_COB table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COB.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpCob(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCob:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCobDAO crNcpdpCob = new NcpdpCobDAO();
      crNcpdpCob.setFileNumber(tokens[0]);
      crNcpdpCob.setClaimKey(tokens[1]);
      crNcpdpCob.setCobKey(tokens[2]);
      crNcpdpCob.setSegmentId(tokens[3]);
      crNcpdpCob.setCobOtPymtCnt(tokens[4]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCob Fieldvalues {}", crNcpdpCob);

      // Save the transformed values to the database
      em.persist(crNcpdpCob);
    }
  }

  /**
   * Load the data contained in the NCPDP_COB_OT_PAYER.DAT file into the NCPDP_COB_OT_PAYER table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COB_OT_PAYER.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpCobOtPayer(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCobOtPayer:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCobOtPayerDAO crNcpdpCobOtPayer = new NcpdpCobOtPayerDAO();
      crNcpdpCobOtPayer.setFileNumber(tokens[0]);
      crNcpdpCobOtPayer.setCobKey(tokens[1]);
      crNcpdpCobOtPayer.setOtPayerKey(tokens[2]);
      crNcpdpCobOtPayer.setOtPayerCvgType(tokens[3]);
      crNcpdpCobOtPayer.setOtPayerIdQual(tokens[4]);
      crNcpdpCobOtPayer.setOtherPayerId(tokens[5]);
      crNcpdpCobOtPayer.setOtherPayerDate(tokens[6]);
      crNcpdpCobOtPayer.setOtPyrAmtPdCnt(tokens[7]);
      crNcpdpCobOtPayer.setOtPyrRejCnt(tokens[8]);
      crNcpdpCobOtPayer.setSequenceNum(tokens[9]);
      if (tokens.length > 10) {
    	  crNcpdpCobOtPayer.setInternalCntrlNbr(tokens[10]);

    	  if(ConversionUtil.checkForNumericType(tokens[11]))
    	  {
    		  crNcpdpCobOtPayer.setOpPatRespAmntPdCnt(ConversionUtil.convertNumericType(tokens[11]));
    	  }

    	  if(ConversionUtil.checkForNumericType(tokens[12]))
    	  {
    		  crNcpdpCobOtPayer.setBeneStgCnt(ConversionUtil.convertTypeByte(tokens[12]));
    	  }            
      }
      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCobOtPayer Fieldvalues {}", crNcpdpCobOtPayer);

      // Save the transformed values to the database
      em.persist(crNcpdpCobOtPayer);
    }
  }

  
  /**
   * Load the data contained in the NCPDP_COB_OP_PAT_RESP_AMOUNT.DAT file into the NCPDP_COB_OP_PAT_RESP_AMOUNT table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COB_OP_PAT_RESP_AMOUNT.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpCobOpPatRespAmount(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCobOpPatRespAmount:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCobOpPatRespAmountDAO crNcpdpCobOpPatRespAmount= new NcpdpCobOpPatRespAmountDAO();
      crNcpdpCobOpPatRespAmount.setFileNumber(tokens[0]);	
      crNcpdpCobOpPatRespAmount.setNcpdpCobOtPayerKey(tokens[1]);
      crNcpdpCobOpPatRespAmount.setOpPayerRespAmountKey(tokens[2]);
      crNcpdpCobOpPatRespAmount.setOpPatRespAmntPdQual(tokens[3]);
      if(ConversionUtil.checkForNumericType(tokens[4]))
      {
    	  crNcpdpCobOpPatRespAmount.setOpPatRespAmnt(ConversionUtil.convertNumericType(tokens[4]));
      }
      crNcpdpCobOpPatRespAmount.setSequenceNum(tokens[5]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCobOpPatRespAmount Fieldvalues {}", crNcpdpCobOpPatRespAmount);

      // Save the transformed values to the database
      em.persist(crNcpdpCobOpPatRespAmount);
    }
  }

  
  /**
   * Load the data contained in the NCPDP_COB_BENE_STG_AMOUNT.DAT file into the NCPDP_COB_BENE_STG_AMOUNT table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COB_BENE_STG_AMOUNT.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpBeneStgAmount(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpBeneStgAmount:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCobBeneStgAmountDAO crNcpdpBeneStgAmount = new NcpdpCobBeneStgAmountDAO();
      crNcpdpBeneStgAmount.setFileNumber(tokens[0]);	
      crNcpdpBeneStgAmount.setNcpdpCobOtPayerKey(tokens[1]);
      crNcpdpBeneStgAmount.setBeneStgAmountKey(tokens[2]);
      crNcpdpBeneStgAmount.setBeneStgQual(tokens[3]);
      if(ConversionUtil.checkForNumericType(tokens[4]))
      {
    	  crNcpdpBeneStgAmount.setBeneStgAmnt(ConversionUtil.convertNumericType(tokens[4]));
      }
      crNcpdpBeneStgAmount.setSequenceNum(tokens[5]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpBeneStgAmount Fieldvalues {}", crNcpdpBeneStgAmount);

      // Save the transformed values to the database
      em.persist(crNcpdpBeneStgAmount);
    }
  }
  
  
  
  /**
   * Load data in the NCPDP_COB_OT_PAYER_AMOUNT.DAT file into the NCPDP_COB_OT_PAYER_AMOUNT table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COB_OT_PAYER_AMOUNT.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */

  private void loadNcpdpCobOtPayerAmount(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCobOtPayerAmount:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCobOtPayerAmountDAO crNcpdpCobOtPayerAmount = new NcpdpCobOtPayerAmountDAO();
      crNcpdpCobOtPayerAmount.setFileNumber(tokens[0]);
      crNcpdpCobOtPayerAmount.setOtPayerKey(tokens[1]);
      crNcpdpCobOtPayerAmount.setOtPayerAmountKey(tokens[2]);
      crNcpdpCobOtPayerAmount.setOtPyrAmtPdQual(tokens[3]);
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crNcpdpCobOtPayerAmount.setOtPyrAmtPaid(new BigDecimal(tokens[4]));
      }
      crNcpdpCobOtPayerAmount.setSequenceNum(tokens[5]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCobOtPayerAmount {}", crNcpdpCobOtPayerAmount);

      // Save the transformed values to the database
      em.persist(crNcpdpCobOtPayerAmount);
    }
  }

  /**
   * Load data from the NCPDP_COB_OT_PAYER_REJECT.DAT file into the NCPDP_COB_OT_PAYER_REJECT table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COB_OT_PAYER_REJECT.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpCobOtPayerReject(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCobOtPayerReject:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCobOtPayerRejectDAO crNcpdpCobOtPayerReject = new NcpdpCobOtPayerRejectDAO();
      crNcpdpCobOtPayerReject.setFileNumber(tokens[0]);
      crNcpdpCobOtPayerReject.setOtPayerKey(tokens[1]);
      crNcpdpCobOtPayerReject.setOtPayerRejectKey(tokens[2]);
      crNcpdpCobOtPayerReject.setOtPyrRejCd(tokens[3]);
      crNcpdpCobOtPayerReject.setSequenceNum(tokens[4]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCobOtPayerReject Fieldvalues {}", crNcpdpCobOtPayerReject);

      // Save the transformed values to the database
      em.persist(crNcpdpCobOtPayerReject);
    }
  }

  /**
   * Load the data contained in the NCPDP_WORK_COMP.DAT file into the NCPDP_WORK_COMP table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_WORK_COMP.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpWorkComp(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpWorkComp:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpWorkCompDAO crNcpdpWorkComp = new NcpdpWorkCompDAO();
      crNcpdpWorkComp.setFileNumber(tokens[0]);
      crNcpdpWorkComp.setClaimKey(tokens[1]);
      crNcpdpWorkComp.setWorkCompKey(tokens[2]);
      crNcpdpWorkComp.setSegmentId(tokens[3]);
      crNcpdpWorkComp.setDateOfInjury(tokens[4]);
      crNcpdpWorkComp.setEmpName(tokens[5]);
      crNcpdpWorkComp.setEmpStreetAddr(tokens[6]);
      crNcpdpWorkComp.setEmpCityAddr(tokens[7]);
      crNcpdpWorkComp.setEmpStateProAddr(tokens[8]);
      crNcpdpWorkComp.setEmpZipPostZone(tokens[9]);

      //TWP: Not sure about treating this as a numeric - but I doubt we will ever have leading 0's so it sould be okay
      if(ConversionUtil.checkForNumericType(tokens[10]))
      {
    	  crNcpdpWorkComp.setEmpPhoneNum(ConversionUtil.convertNumericTypeLong(tokens[11]));
      }
      
      crNcpdpWorkComp.setEmpContactName(tokens[11]);
      crNcpdpWorkComp.setCarrierId(tokens[12]);
      crNcpdpWorkComp.setClaimReferenceId(tokens[13]);
      if (tokens.length > 14) {
    	  if(ConversionUtil.checkForNumericType(tokens[14]))
    	  {
    		  crNcpdpWorkComp.setBillEntityTypeInd(ConversionUtil.convertTypeByte(tokens[14]));
    	  }

    	  crNcpdpWorkComp.setPayToQual(tokens[15]);
    	  crNcpdpWorkComp.setPayToId(tokens[16]);
    	  crNcpdpWorkComp.setPayToName(tokens[17]);
    	  crNcpdpWorkComp.setPayToAddress(tokens[18]);
    	  crNcpdpWorkComp.setPayToCity(tokens[19]);
    	  crNcpdpWorkComp.setPayToState(tokens[20]);
    	  crNcpdpWorkComp.setPayToZip(tokens[21]);
    	  crNcpdpWorkComp.setGenericEqProdIdQual(tokens[22]);
    	  crNcpdpWorkComp.setGenericEqProdId(tokens[23]);
      }
      // Log into the trace file the transformed values being saved to the database
      //TWP: THESE CHANGED FIX TO STRING
      traceLOGGER.trace("NcpdpWorkComp Fieldvalues {}", crNcpdpWorkComp);

      // Save the transformed values to the database
      em.persist(crNcpdpWorkComp);
    }
  }

  /**
   * Load the data contained in NCPDP_DUR_PPS.DAT file into the NCPDP_DUR_PPS table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_DUR_PPS.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpDurPps(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpDurPps:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpDurPpsDAO crNcpdpDurPps = new NcpdpDurPpsDAO();
      crNcpdpDurPps.setFileNumber(tokens[0]);
      crNcpdpDurPps.setClaimKey(tokens[1]);
      crNcpdpDurPps.setDurPpsKey(tokens[2]);
      crNcpdpDurPps.setSegmentId(tokens[3]);
      crNcpdpDurPps.setDurPpsCdCtr(tokens[4]);
      crNcpdpDurPps.setReasonForSvcCd(tokens[5]);
      crNcpdpDurPps.setPrfsnSvcCd(tokens[6]);
      crNcpdpDurPps.setResultOfSvcCd(tokens[7]);
      crNcpdpDurPps.setDurPpsLvlOfEft(tokens[8]);
      crNcpdpDurPps.setDurCoagentIdQual(tokens[9]);
      crNcpdpDurPps.setDurCoagentId(tokens[10]);
      crNcpdpDurPps.setSequenceNum(tokens[11]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpDurPps Fieldvalues {}", crNcpdpDurPps);

      // Save the transformed values to the database
      em.persist(crNcpdpDurPps);
    }
  }

  /**
   * Load the data contained in the NCPDP_PRICING.DAT file into the NCPDP_PRICING table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_PRICING.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpPricing(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpPricing:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpPricingDAO crNcpdpPricing = new NcpdpPricingDAO();
      crNcpdpPricing.setFileNumber(tokens[0]);
      crNcpdpPricing.setClaimKey(tokens[1]);
      crNcpdpPricing.setPricingKey(tokens[2]);
      crNcpdpPricing.setSegmentId(tokens[3]);

      // Several BigDecimal types in this one
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crNcpdpPricing.setIngdtCost(new BigDecimal(tokens[4]));
      }
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crNcpdpPricing.setDispensingFeeSub(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crNcpdpPricing.setPrfsnSvcFeeSub(new BigDecimal(tokens[6]));
      }
      if (!((tokens[7].equals("")) || (tokens[7] == null))) {
        crNcpdpPricing.setPatPaidAmt(new BigDecimal(tokens[7]));
      }
      if (!((tokens[8].equals("")) || (tokens[8] == null))) {
        crNcpdpPricing.setIncentiveAmtSub(new BigDecimal(tokens[8]));
      }
      crNcpdpPricing.setOtAmtClaimSubCnt(tokens[9]);
      
      if (!((tokens[10].equals("")) || (tokens[10] == null))) {
        crNcpdpPricing.setFlatSalesTaxAmt(new BigDecimal(tokens[10]));
      }
      if (!((tokens[11].equals("")) || (tokens[11] == null))) {
        crNcpdpPricing.setPctSleTaxAmtSub(new BigDecimal(tokens[11]));
      }
      if (!((tokens[12].equals("")) || (tokens[12] == null))) {
        crNcpdpPricing.setPctSleTaxRtSub(new BigDecimal(tokens[12]));
      }
      
      crNcpdpPricing.setPctSleTaxBsSub(tokens[13]);
      
      if (!((tokens[14].equals("")) || (tokens[14] == null))) {
        crNcpdpPricing.setUsualAndCstChg(new BigDecimal(tokens[14]));
      }
      if (!((tokens[15].equals("")) || (tokens[15] == null))) {
        crNcpdpPricing.setGrossAmtDue(new BigDecimal(tokens[15]));
      }
      crNcpdpPricing.setBasisOfCostDetm(tokens[16]);
      if (tokens.length > 17) {
    	  //TWP:Question    	  
    	  if (!((tokens[17].equals("")) || (tokens[17] == null))) {
    		  crNcpdpPricing.setMedicaidPdAmnt(new BigDecimal(tokens[17]));
    	  }
      }
      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpPricing Fieldnames {}", crNcpdpPricing);

      // Save the transformed values to the database
      em.persist(crNcpdpPricing);
    }
  }

  /**
   * Load data from the NCPDP_PRICING_OT_AMOUNT.DAT file into the NCPDP_PRICING_OT_AMOUNT table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_PRICING_OT_AMOUNT.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpPricingOtAmount(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpPricingOtAmount:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpPricingOtAmountDAO crNcpdpPricingOtAmount = new NcpdpPricingOtAmountDAO();
      crNcpdpPricingOtAmount.setFileNumber(tokens[0]);
      crNcpdpPricingOtAmount.setPricingKey(tokens[1]);
      crNcpdpPricingOtAmount.setOtAmountKey(tokens[2]);
      crNcpdpPricingOtAmount.setOtAmtClaimSubQual(tokens[3]);
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crNcpdpPricingOtAmount.setOtAmtClaimSub(new BigDecimal(tokens[4]));
      }
      crNcpdpPricingOtAmount.setSequenceNum(tokens[5]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpPricingOtAmount Fieldvalues {}", crNcpdpPricingOtAmount);

      // Save the transformed values to the database
      em.persist(crNcpdpPricingOtAmount);
    }
  }

  /**
   * Load the data contained in the NCPDP_COUPON.DAT file into the NCPDP_COUPON table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COUPON.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadoNcpdpCoupon(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCoupon:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCouponDAO crNcpdpCoupon = new NcpdpCouponDAO();
      crNcpdpCoupon.setFileNumber(tokens[0]);
      crNcpdpCoupon.setClaimKey(tokens[1]);
      crNcpdpCoupon.setCouponKey(tokens[2]);
      crNcpdpCoupon.setSegmentId(tokens[3]);
      crNcpdpCoupon.setCouponType(tokens[4]);
      crNcpdpCoupon.setCouponNumber(tokens[5]);
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crNcpdpCoupon.setCouponValueAmt(new BigDecimal(tokens[6]));
      }

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCoupon Fieldvalues {}", crNcpdpCoupon);

      // Save the transformed values to the database
      em.persist(crNcpdpCoupon);
    }
  }

  /**
   * Load the data contained in the NCPDP_COMPOUND.DAT file, into the NCPDP_COMPOUND table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COMPOUND.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpCompound(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCompound:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCompoundDAO crNcpdpCompound = new NcpdpCompoundDAO();
      crNcpdpCompound.setFileNumber(tokens[0]);
      crNcpdpCompound.setClaimKey(tokens[1]);
      crNcpdpCompound.setCompoundKey(tokens[2]);
      crNcpdpCompound.setSegmentId(tokens[3]);
      crNcpdpCompound.setCmpDsgFrmDesCd(tokens[4]);
      crNcpdpCompound.setCmpDispUntFrIn(tokens[5]);
      crNcpdpCompound.setCmpRtOfAdmin(tokens[6]);
      crNcpdpCompound.setCmpIngdtCompCnt(tokens[7]);      

      
      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCompound Fieldvalues {}", crNcpdpCompound);

      // Save the transformed values to the database
      em.persist(crNcpdpCompound);
    }
  }

  /**
   * Load the data contained in the NCPDP_COMPOUND_INGR.DAT file into the NCPDP_COMPOUND_INGR table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_COMPOUND_INGR.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpCompoundIngr(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpCompoundIngr:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpCompoundIngrDAO crNcpdpCompoundIngr = new NcpdpCompoundIngrDAO();
      crNcpdpCompoundIngr.setFileNumber(tokens[0]);
      crNcpdpCompoundIngr.setCompoundKey(tokens[1]);
      crNcpdpCompoundIngr.setCompoundIngredKey(tokens[2]);
      crNcpdpCompoundIngr.setCmpPrdtIdQual(tokens[3]);
      crNcpdpCompoundIngr.setCmpPrdtId(tokens[4]);

      // A couple of the fields are BigDecimals
      if(ConversionUtil.checkForNumericType(tokens[5]))
      {
    		crNcpdpCompoundIngr.setCmpIngdtQty(new BigDecimal(tokens[5]));
      }
              
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crNcpdpCompoundIngr
            .setCmpIngdtDrgCost(new BigDecimal(tokens[6]));
      }

      crNcpdpCompoundIngr.setCmpIngdtCostDet(tokens[7]);
      crNcpdpCompoundIngr.setSequenceNum(tokens[8]);

      if (tokens.length > 14) {
    	  if(ConversionUtil.checkForNumericType(tokens[8]))
    	  {
    		  crNcpdpCompoundIngr.setCmpIngdtModCdCnt(ConversionUtil.convertTypeByte(tokens[8]));
    	  }
      }
      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpCompoundIngr Fieldvalues {}", crNcpdpCompoundIngr);

      // Save the transformed values to the database
      em.persist(crNcpdpCompoundIngr);
    }
  }

  /**
   * Load the data contained in the NCPDP_PRIOR_AUTH.DAT file into the NCPDP_PRIOR_AUTH table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_PRIOR_AUTH.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpPriorAuth(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpPriorAuth:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpPriorAuthDAO crNcpdpPriorAuth = new NcpdpPriorAuthDAO();
      crNcpdpPriorAuth.setFileNumber(tokens[0]);
      crNcpdpPriorAuth.setClaimKey(tokens[1]);
      crNcpdpPriorAuth.setPriorAuthKey(tokens[2]);
      crNcpdpPriorAuth.setSegmentId(tokens[3]);
      crNcpdpPriorAuth.setRequestType(tokens[4]);
      crNcpdpPriorAuth.setReqPerdDateBegn(tokens[5]);
      crNcpdpPriorAuth.setReqPerdDateEnd(tokens[6]);
      crNcpdpPriorAuth.setBasisOfRequest(tokens[7]);
      crNcpdpPriorAuth.setAuthRepFirstName(tokens[8]);
      crNcpdpPriorAuth.setAuthRepLastName(tokens[9]);
      crNcpdpPriorAuth.setAuthRepStrtAddr(tokens[10]);
      crNcpdpPriorAuth.setAuthRepCityAddr(tokens[11]);
      crNcpdpPriorAuth.setAuthRepStProAddr(tokens[12]);
      crNcpdpPriorAuth.setAuthRepZipPtZn(tokens[13]);
      crNcpdpPriorAuth.setPriorAuthNumAsg(tokens[14]);
      crNcpdpPriorAuth.setAuthNum(tokens[15]);
      crNcpdpPriorAuth.setPriorAuthSupDoc(tokens[16]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpPriorAuth Fieldvalues {}", crNcpdpPriorAuth);

      // Save the transformed values to the database
      em.persist(crNcpdpPriorAuth);
    }
  }

  /**
   * Load the data contained in NCPDP_CLINICAL.DAT file into the NCPDP_CLINICAL table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_CLINICAL.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpClinical(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpClinical:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpClinicalDAO crNcpdpClinical = new NcpdpClinicalDAO();
      crNcpdpClinical.setFileNumber(tokens[0]);
      crNcpdpClinical.setClaimKey(tokens[1]);
      crNcpdpClinical.setClinicalKey(tokens[2]);
      crNcpdpClinical.setSegmentId(tokens[3]);
      crNcpdpClinical.setDiagCodeCnt(tokens[4]);
      if (tokens.length > 5) {
    	  if(ConversionUtil.checkForNumericType(tokens[5]))
    	  {
    		  crNcpdpClinical.setClinicalInfoCnt(tokens[5]);
    	  }
      }

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpClinical Fieldvalues {}", crNcpdpClinical);

      // Save the transformed values to the database
      em.persist(crNcpdpClinical);
    }
  }

  /**
   * Load the data contained in NCPDP_CLINICAL_DIAG.DAT file into the NCPDP_CLINICAL_DIAG table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_CLINICAL_DIAG.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpClinicalDiag(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpClinicalDiag:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpClinicalDiagDAO crNcpdpClinicalDiag = new NcpdpClinicalDiagDAO();
      crNcpdpClinicalDiag.setFileNumber(tokens[0]);
      crNcpdpClinicalDiag.setClaimKey(tokens[1]);
      crNcpdpClinicalDiag.setClinicalKey(tokens[2]);
      crNcpdpClinicalDiag.setDiagKey(tokens[3]);
      crNcpdpClinicalDiag.setDiagCodeQual(tokens[4]);
      crNcpdpClinicalDiag.setDiagnosisCode(tokens[5]);
      crNcpdpClinicalDiag.setSequenceNum(tokens[6]);

      // Log into the trace file the transformed values being saved to the database
      traceLOGGER.trace("NcpdpClinicalDiag Fieldvalues {}", crNcpdpClinicalDiag);

      // Save the transformed values to the database
      em.persist(crNcpdpClinicalDiag);
    }
  }

  /**
   * Load the data contained in the NCPDP_CLINICAL_MEAS.DAT file into the NCPDP_CLINICAL_MEAS table
   * 
   * @param in
   *          - a BufferedFile reader connected to NCPDP_CLINICAL_MEAS.DAT
   * @param em
   *          - the entity manager that is connected to the database
   * @return
   * @throws IOException
   *           - if the .dat file can not be read
   */
  private void loadNcpdpClinicalMeas(BufferedReader in, EntityManager em)
      throws IOException {

    // Continue loading lines for the entire file
    String loadRec = "";
    while ((loadRec = in.readLine()) != null) {

      // Log into the trace file the values as found in the file being loaded
      traceLOGGER.trace("NcpdpClinicalMeas:" + loadRec);

      // Split the line into tokens based on a delimiter
      String[] tokens = loadRec.split(DELIMITER, -1);

      // Transform each of the tokens
      NcpdpClinicalMeasDAO crNcpdpClinicalMeas = new NcpdpClinicalMeasDAO();
      crNcpdpClinicalMeas.setFileNumber(tokens[0]);
      crNcpdpClinicalMeas.setClaimKey(tokens[1]);
      crNcpdpClinicalMeas.setClinicalKey(tokens[2]);
      crNcpdpClinicalMeas.setMeasKey(tokens[3]);      ;
      crNcpdpClinicalMeas.setClinicalInfoCnt(tokens[4]);
      crNcpdpClinicalMeas.setMeasDate(tokens[5]);
      crNcpdpClinicalMeas.setMeasTime(tokens[6]);
      crNcpdpClinicalMeas.setMeasDimension(tokens[7]);
      crNcpdpClinicalMeas.setMeasUnit(tokens[8]);
      crNcpdpClinicalMeas.setMeasValue(tokens[9]);
      crNcpdpClinicalMeas.setSequenceNum(tokens[10]);

      // Log into the trace file the transformed values being saved to the database
      //TWP:Modified - fix logging
      traceLOGGER.trace("NcpdpClinicalMeas Fieldvalues {}", crNcpdpClinicalMeas);

      // Save the transformed values to the database
      em.persist(crNcpdpClinicalMeas);
    }
  }
  
  /**
  * Load the data contained in the NCPDP_ADTL_DOCUMENT.DAT file into the 
  * NCPDP_ADTL_DOCUMENT table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_ADTL_DOCUMENT.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpAdtlDocument(BufferedReader in, EntityManager em)
     throws IOException {

	 
	  String loadRec = "";
      while ((loadRec = in.readLine()) != null) {

        // Log into the trace file the values as found in the file being loaded
        traceLOGGER.trace("NcpdpAdtlDocument:" + loadRec);

        // Split the line into tokens based on a delimiter
        String[] tokens = loadRec.split(DELIMITER, -1);
        
        NcpdpAdtlDocumentDAO crNcpdpAdtlDocument = new NcpdpAdtlDocumentDAO();
        crNcpdpAdtlDocument.setAdtlDocKey(tokens[0]);
        crNcpdpAdtlDocument.setFileNumber(tokens[1]);
        crNcpdpAdtlDocument.setClaimKey(tokens[2]);
        crNcpdpAdtlDocument.setSegmentId(tokens[3]);
        crNcpdpAdtlDocument.setAdtlDocTypeId(tokens[4]);
        
        if(ConversionUtil.checkForNumericType(tokens[5]))
        {
           crNcpdpAdtlDocument.setReqPrdBeginDt(ConversionUtil.convertNumericTypeInteger(tokens[5]));
        }
        
        if(ConversionUtil.checkForNumericType(tokens[6]))
        {
           crNcpdpAdtlDocument.setReqPrdRecertReviseDt(ConversionUtil.convertNumericTypeInteger(tokens[6]));
        }
        
        crNcpdpAdtlDocument.setReqStatus(tokens[7]);
        
        if(ConversionUtil.checkForNumericType(tokens[8]))
        {
           crNcpdpAdtlDocument.setLnghtOfNeedQual(ConversionUtil.convertTypeByte(tokens[8]));
        }
        
        if(ConversionUtil.checkForNumericType(tokens[9]))
        {
           crNcpdpAdtlDocument.setLngthOfNeed(ConversionUtil.convertNumericTypeShortInteger(tokens[9]));
        }
        
        if(ConversionUtil.checkForNumericType(tokens[10]))
        {
           crNcpdpAdtlDocument.setPresSupplierDtSigned(ConversionUtil.convertNumericTypeInteger(tokens[10]));
        }
        
        crNcpdpAdtlDocument.setSupportDoc(tokens[11]);
        
        if(ConversionUtil.checkForNumericType(tokens[12]))
        {
           crNcpdpAdtlDocument.setQstnNbrLetterCnt(ConversionUtil.convertTypeByte(tokens[12]));
        }
        
        if(ConversionUtil.checkForNumericType(tokens[13]))
        {
           crNcpdpAdtlDocument.setSequenceNum(tokens[13]);
        }
        
        
        // Log into the trace file the transformed values being saved to the database
         traceLOGGER.trace("NcpdpAdtlDocument Fieldvalues {}", crNcpdpAdtlDocument);

     // Save the transformed values to the database
     em.persist(crNcpdpAdtlDocument);
   }
 }
 

 /**
  * Load the data contained in the NCPDP_ADTL_DOC_QSTN_NBR.DAT file into the 
  * NCPDP_ADTL_DOC_QSTN_NBR table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_ADTL_DOC_QSTN_NBR.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpAdtlDocQstnNbr(BufferedReader in, EntityManager em)
 throws IOException {


	 String loadRec = "";
	 while ((loadRec = in.readLine()) != null) {

		 // Log into the trace file the values as found in the file being loaded
		 traceLOGGER.trace("NcpdpAdtlDocQstnNbr:" + loadRec);

		 // Split the line into tokens based on a delimiter
		 String[] tokens = loadRec.split(DELIMITER, -1);

		 NcpdpAdtlDocQstnNbrDAO crNcpdpAdtlDocument = new NcpdpAdtlDocQstnNbrDAO();
		 crNcpdpAdtlDocument.setAdtlDocQstnNbrKey(tokens[0]);
		 crNcpdpAdtlDocument.setFileNumber(tokens[1]);
		 crNcpdpAdtlDocument.setNcpdpAdtlDocKey(tokens[2]);
		 crNcpdpAdtlDocument.setQstnNbrLetter(tokens[3]);
		 if(ConversionUtil.checkForNumericType(tokens[4]))
		 {
			 crNcpdpAdtlDocument.setQstnPctResponse(ConversionUtil.convertNumericType(tokens[4]));
		 }

		 if(ConversionUtil.checkForNumericType(tokens[5]))
		 {
			 crNcpdpAdtlDocument.setQstnDtResponse(ConversionUtil.convertNumericTypeInteger(tokens[5]));
		 }

		 if(ConversionUtil.checkForNumericType(tokens[6]))
		 {
			 crNcpdpAdtlDocument.setQstnAmtResponse(ConversionUtil.convertNumericType(tokens[6]));
		 }

		 if(ConversionUtil.checkForNumericType(tokens[7]))
		 {
			 crNcpdpAdtlDocument.setQstnNumResponse(ConversionUtil.convertNumericTypeLong(tokens[7]));
		 }

		 crNcpdpAdtlDocument.setQstnAlpanumResponse(tokens[8]);
		 
		 crNcpdpAdtlDocument.setSequenceNum(tokens[9]);
		 
		 traceLOGGER.trace("NcpdpAdtlDocQstnNbr Fieldvalues {}", crNcpdpAdtlDocument);
		 
		 em.persist(crNcpdpAdtlDocument);
	 }
 }
 
 /**
  * Load the data contained in the NCPDP_FACILITY.DAT file into the 
  * NCPDP_FACILITY table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_FACILITY.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
 * 
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpFacility(BufferedReader in, EntityManager em) throws IOException
 {
	 String loadRec = "";
     while ((loadRec = in.readLine()) != null) {

       // Log into the trace file the values as found in the file being loaded
       traceLOGGER.trace("NcpdpFacility:" + loadRec);

       // Split the line into tokens based on a delimiter
       String[] tokens = loadRec.split(DELIMITER, -1);

       // Transform each of the tokens, in this case always the null transformation
       

       NcpdpFacilityDAO crNcpdpFacility = new NcpdpFacilityDAO();
       crNcpdpFacility.setFacilityKey(tokens[0]);
       crNcpdpFacility.setFileNumber(tokens[1]);
       crNcpdpFacility.setClaimKey(tokens[2]);
       crNcpdpFacility.setSegmentId(tokens[3]);
       crNcpdpFacility.setFacilityId(tokens[4]);
       crNcpdpFacility.setFacilityName(tokens[5]);
       crNcpdpFacility.setFacilityAddress(tokens[6]);
       crNcpdpFacility.setFacilityCity(tokens[7]);
       crNcpdpFacility.setFacilityState(tokens[8]);
       crNcpdpFacility.setFaciltyZip(tokens[9]);
       
       traceLOGGER.trace("NcpdpFacility Fieldvalues {}", crNcpdpFacility);

       // Save the transformed values to the database
       em.persist(crNcpdpFacility);
     }
 }
 
 /**
  * Load the data contained in the NCPDP_NARRATIVE file into the 
  * NCPDP_NARRATIVE table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_NARRATIVE.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpNarrative(BufferedReader in, EntityManager em) throws IOException
 {
	 String loadRec = "";
     while ((loadRec = in.readLine()) != null) {

       // Log into the trace file the values as found in the file being loaded
       traceLOGGER.trace("loadNarrative:" + loadRec);

       // Split the line into tokens based on a delimiter
       String[] tokens = loadRec.split(DELIMITER, -1);

       // Transform each of the tokens, in this case always the null transformation
       NcpdpNarrativeDAO crNcpdpNarrative = new NcpdpNarrativeDAO();

       crNcpdpNarrative.setNarrativeKey(tokens[0]);
       crNcpdpNarrative.setFileNumber(tokens[1]);
       crNcpdpNarrative.setClaimKey(tokens[2]);
       crNcpdpNarrative.setSegmentId(tokens[3]);
       crNcpdpNarrative.setNarrativeMsg(tokens[4]);
       
       traceLOGGER.trace("NcpdpNarrative Fieldvalues {}", crNcpdpNarrative);
       // Save the transformed values to the database
       em.persist(crNcpdpNarrative);
     } 
 }
 
 
 
 /**
  * Load the data contained in the NCPDP_CMP_INGDT_MOD.DAT file into the 
  * NCPDP_CMP_INGDT_MOD table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_CMP_INGDT_MOD.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpCmpIngdtMod(BufferedReader in, EntityManager em) throws IOException
 {
	 String loadRec = "";
     while ((loadRec = in.readLine()) != null) {

       // Log into the trace file the values as found in the file being loaded
       traceLOGGER.trace("NcpdpCmpIngdtMod:" + loadRec);

       // Split the line into tokens based on a delimiter
       String[] tokens = loadRec.split(DELIMITER, -1);

       // Transform each of the tokens, in this case always the null transformation
       NcpdpCmpIngdtModDAO crNcpdpCmpIngdtMode = new NcpdpCmpIngdtModDAO();

       crNcpdpCmpIngdtMode.setFileNumber(tokens[0]);       
       crNcpdpCmpIngdtMode.setCmpIngdtModKey(tokens[1]);
       crNcpdpCmpIngdtMode.setCmpIngdtModCd(tokens[2]);
       crNcpdpCmpIngdtMode.setSequenceNum(tokens[3]);
       crNcpdpCmpIngdtMode.setNcpdpCompoundIngredKey(tokens[4]);
       
       traceLOGGER.trace("NcpdpCmpIngdtMode Fieldvalues {}", crNcpdpCmpIngdtMode);
       // Save the transformed values to the database
       em.persist(crNcpdpCmpIngdtMode);
     } 
 }
 
 /**
  * Load the data contained in the NCPDP_RESPONSE_PROCESS.DAT file into the 
  * NCPDP_RESPONSE_PROCESS table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_RESPONSE_PROCESS.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpResponseProcess(BufferedReader in, EntityManager em) throws IOException
 {
	 String loadRec = "";
     while ((loadRec = in.readLine()) != null) {

       // Log into the trace file the values as found in the file being loaded
       traceLOGGER.trace("NcpdpResponseProcess:" + loadRec);

       // Split the line into tokens based on a delimiter
       String[] tokens = loadRec.split(DELIMITER, -1);

       // Transform each of the tokens, in this case always the null transformation
       NcpdpResponseProcess crNcpdpResponseProcess = new NcpdpResponseProcess();
       NcpdpResponseProcessId id = new NcpdpResponseProcessId();
       id.setFileNumber(tokens[0]);
       id.setType(tokens[1]);
       id.setStatus(tokens[2]);
       id.setClearinghouse(tokens[3]);
       crNcpdpResponseProcess.setId(id);

       // Save the transformed values to the database
       em.persist(crNcpdpResponseProcess);
     } 
 }
 
 
 /**
  * Load the data contained in the NCPDP_RB1_CLAIM_STATUS.DAT file into the 
  * NCPDP_RB1_CLAIM_STATUS table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_RB1_CLAIM_STATUS.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpRb1ClaimStatus(BufferedReader in, EntityManager em) throws IOException
 {
	 String loadRec = "";
     while ((loadRec = in.readLine()) != null) {

       // Log into the trace file the values as found in the file being loaded
       traceLOGGER.trace("NcpdpRb1ClaimStatus:" + loadRec);

       // Split the line into tokens based on a delimiter
       String[] tokens = loadRec.split(DELIMITER, -1);

       // Transform each of the tokens, in this case always the null transformation
       NcpdpRb1ClaimStatus crNcpdpRb1ClaimStatus = new NcpdpRb1ClaimStatus();
       crNcpdpRb1ClaimStatus.setB1ClaimStatusKey(tokens[0]);
       crNcpdpRb1ClaimStatus.setFileNumber(tokens[1]);
       crNcpdpRb1ClaimStatus.setBatchNumber(tokens[2]);
       crNcpdpRb1ClaimStatus.setPatientName(tokens[3]);
       crNcpdpRb1ClaimStatus.setPatientSsn(tokens[4]);
       crNcpdpRb1ClaimStatus.setPharmacyEin(tokens[5]);
       crNcpdpRb1ClaimStatus.setRxNumber(tokens[6]);
       crNcpdpRb1ClaimStatus.setDateOfService(tokens[7]);
       crNcpdpRb1ClaimStatus.setTotalBilledAmt(tokens[8]);
       crNcpdpRb1ClaimStatus.setHacClaimNum(tokens[9]);
       crNcpdpRb1ClaimStatus.setClaimStatus(tokens[10]);
       crNcpdpRb1ClaimStatus.setCheckNum(tokens[11]);
       crNcpdpRb1ClaimStatus.setTotalPaidAmt(tokens[12]);
       crNcpdpRb1ClaimStatus.setAmtPaidVendor(tokens[13]);
       crNcpdpRb1ClaimStatus.setAmtPaidPatient(tokens[14]);
       crNcpdpRb1ClaimStatus.setDedAppliedAmt(tokens[15]);
       crNcpdpRb1ClaimStatus.setRejectedAmt(tokens[16]);
       crNcpdpRb1ClaimStatus.setPatientCostshare(tokens[17]);
       crNcpdpRb1ClaimStatus.setErrorRejCode(tokens[18]);
       crNcpdpRb1ClaimStatus.setRejCode(tokens[19]);
       crNcpdpRb1ClaimStatus.setNcpdpRejCd(tokens[20]);
       crNcpdpRb1ClaimStatus.setHicNum(tokens[21]);
       crNcpdpRb1ClaimStatus.setFmsDocId(tokens[22]);
       crNcpdpRb1ClaimStatus.setClearinghouse(tokens[23]);
       
       // Save the transformed values to the database
       em.persist(crNcpdpRb1ClaimStatus);
     } 
 }
 
 
 /**
  * Load the data contained in the NCPDP_RB2_REVERSAL_STATUS.DAT file into the 
  * NCPDP_RB2_REVERSAL_STATUS table
  * @param in
  *          - a BufferedFile reader connected to NCPDP_RB2_REVERSAL_STATUS.DAT
  * @param em
  *          - the entity manager that is connected to the database
  * @return
  * @throws IOException
  *           - if the .dat file can not be read
  */
 private void loadNcpdpRb2ReversalStatus(BufferedReader in, EntityManager em) throws IOException
 {
	 String loadRec = "";
     while ((loadRec = in.readLine()) != null) {

       // Log into the trace file the values as found in the file being loaded
       traceLOGGER.trace("NcpdpRb2ReversalStatus:" + loadRec);

       // Split the line into tokens based on a delimiter
       String[] tokens = loadRec.split(DELIMITER, -1);

       // Transform each of the tokens, in this case always the null transformation
       NcpdpRb2ReversalStatus crNcpdpRb2ReversalStatus = new NcpdpRb2ReversalStatus();
       crNcpdpRb2ReversalStatus.setB2RevStatusKey(tokens[0]);
       crNcpdpRb2ReversalStatus.setFileNumber(tokens[1]);
       crNcpdpRb2ReversalStatus.setDateSubmitted(tokens[2]);
       crNcpdpRb2ReversalStatus.setDateOfService(tokens[3]);
       crNcpdpRb2ReversalStatus.setRxNumber(tokens[4]);
       crNcpdpRb2ReversalStatus.setNdcCode(tokens[5]);
       crNcpdpRb2ReversalStatus.setQuantity(tokens[6]);
       crNcpdpRb2ReversalStatus.setDaysSupply(tokens[7]);
       crNcpdpRb2ReversalStatus.setAmtPaidVendor(tokens[8]);
       crNcpdpRb2ReversalStatus.setBenePayment(tokens[9]);
       crNcpdpRb2ReversalStatus.setPharmacyNumber(tokens[10]);
       crNcpdpRb2ReversalStatus.setBeneSsn(tokens[11]);
       crNcpdpRb2ReversalStatus.setBeneLastName(tokens[12]);
       crNcpdpRb2ReversalStatus.setBeneFirstName(tokens[13]);
       crNcpdpRb2ReversalStatus.setBeneDob(tokens[14]);
       crNcpdpRb2ReversalStatus.setHacClaimNum(tokens[15]);
       crNcpdpRb2ReversalStatus.setHacAmtPaidVendor(tokens[16]);
       crNcpdpRb2ReversalStatus.setMmiRefNum(tokens[17]);
       crNcpdpRb2ReversalStatus.setHacRetClaimNum(tokens[18]);
       crNcpdpRb2ReversalStatus.setHacRetClmDatePaid(tokens[19]);
       crNcpdpRb2ReversalStatus.setHacRetClmPaidAmt(tokens[20]);
       crNcpdpRb2ReversalStatus.setHacRetClmPaidCheckNum(tokens[21]);
       crNcpdpRb2ReversalStatus.setClearinghouse(tokens[22]);
       crNcpdpRb2ReversalStatus.setBatchNumber(tokens[23]);

       // Save the transformed values to the database
       em.persist(crNcpdpRb2ReversalStatus);
     } 
 }
 
  /**
   * This method stores the number of claims that were inserted from the batch into the edi process
   * log.
   * 
   * @param vClmCnt
   *          - number of claims read
   * @param vFileDt
   *          - the file number
   * @param dbEM
   *          - the entity manager connected to the database
   */
  private void insertProcessLog(Long vClmCnt, String vFileDt, EntityManager em)
      throws SQLException {

    EdiProcessLog crEdiProcessLog = new EdiProcessLog();

    crEdiProcessLog.setEdiProcessId(UUID.randomUUID().toString().toUpperCase());
    crEdiProcessLog.setFileDate(DateUtil.getSqlDate(vFileDt));
    crEdiProcessLog.setProcessName("NCPDP LOAD TO EDI REPOSITORY");
    crEdiProcessLog.setMessage("NUM. OF NCPDP CLAIMS WERE LOADED INTO EDI REPOSITORY");
    crEdiProcessLog.setCounts(vClmCnt);
    crEdiProcessLog.setClaimType("NCPDP");
    crEdiProcessLog.setStatus("NEW");

    em.persist(crEdiProcessLog);
  }
}
