package gov.va.med.hac.edi.utils.jpa;

import gov.va.med.hac.edi.extract.x12.jpa.ExtractDntlClaims;
import gov.va.med.hac.edi.reposload.ncpdp.jpa.ProcessNCPDPLoad;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConversionUtil {

	static final Logger LOGGER = LoggerFactory.getLogger("trace.gov.va.med.hac.edi.utils.jpa.ConversionUtil");
    /**
     * will determine if the string is a valid numeric type
     * 
     * @param token - string from file
     * @return boolean (is valid numeric type?)
     */
    public static boolean checkForNumericType(String token)
    {
        boolean ret = isNumeric(token);
        
        if(!ret)
        {
           if(token != null && !token.isEmpty())
           {
        	   LOGGER.debug("data: "+token+" which is supposed to be numeric, is not...");
           }
        }
        
        return ret;
    }
    
    /**
     * will convert a string to a BigDecimal, provided that it is a valid numeric type
     * 
     * @param token - string from file
     * @return a BigDecimal
     */
    public  static BigDecimal convertNumericType(String token)
    {
        BigDecimal ret = null;
        
        // we repeat this test, because we should never assume that the user
        // has called checkForNumericType...
        
        if (isNumeric(token)) 
        {
            ret = new BigDecimal(token);
        }
        
        return ret;
    }
    
    /**
     * will convert a string to a BigDecimal, provided that it is a valid numeric type
     * 
     * @param token - string from file
     * @return a BigDecimal
     */
    public  static Integer convertNumericTypeInteger(String token)
    {
        BigDecimal ret = convertNumericType(token);
        
        return ((ret == null) ? null:ret.intValue() );
    }
    
    public  static Byte convertTypeByte(String string)
    {
        return Byte.parseByte(string == null ? null:string);
    }
    
    public  static Short convertNumericTypeShortInteger(String string)
    {
        Integer ret = convertNumericTypeInteger(string);
        
        return ((ret == null) ? null: ret.shortValue());
    }
    
    public  static Long convertNumericTypeLong(String string)
    {
        Integer ret = convertNumericTypeInteger(string);
        
        return ((ret == null) ? null : ret.longValue());
    }

    
    
    /**
     * will determine if the string is a valid numeric type
     * 
     * @param token - string from file
     * @return boolean (is valid numeric type?)
     */
    private  static boolean isNumeric(String input)
    {
        boolean ret = true;
        
        if((input != null) && (input != "") && !input.isEmpty())
        {
            for(int x=0;x<=(input.length()-1);x++)
            {
                if(!Character.isDigit(input.charAt(x)))
                {
                    ret = false;
                    break;
                }
            }
        }
        else if(input == null)
        {
            ret = false;
        }
        else if(input == "")
        {
            ret = false;
        }else if(input.isEmpty())
        {
            ret = false;
        }
        
        return ret;
    }
    
    public  static Character convertToCharacterType(String string)
    {
        Character c = null;
        
        if((string != null) && (string.length() >= 1))
        {
            c = string.charAt(0);    
        }        
        
        return c;
    }
    
    /**
     * Booleans will come in as 1=true and 0=false
     * so, we can apply a numeric edit
     * 
     * @param bool
     * @return
     */
    public  static boolean checkForBooleanType(String bool)
    {
        boolean ret = false;
        
        if(checkIsLike(bool, "0","1"))
        {
           ret = true;
        }
        
        if(!ret)
        {
            if(bool != null && !bool.isEmpty())
            {
            	LOGGER.debug("data: "+bool+" which is supposed to be boolean, is not...");
            }
        }
        
        return ret;
    }
    
    public  static int convertBooleanType(String string)
    { 
        Character c = null;
        
        if(checkIsLike(string, "0","1"))
        {
            if((string != null) && (string.length() >= 1))
            {
                c = string.charAt(0);    
            }
        }
        
        return ((c == '0') ? 0:1);
    }
    
    
    
    
    /**
     * The following validators can either operate in a 
     * "pass through" mode, meaning that the data is examined
     * and errors are logged without preventing the original data
     * from being written to db.
     * 
     * You can also use these validators as a way to prevent invalid
     * data from reaching the db. If this is your goal, then use
     * the methods that return boolean.
     * 
     */
    
    public  static String checkDataLength(int maxLen, String codeStr)
    {
        validateDataLength(maxLen, codeStr);
        
        return codeStr;
    }
    
    public  static boolean validateDataLength(int maxLen, String codeStr)
    {
        boolean ret = false;
        
        if(codeStr != null && codeStr.length() <= maxLen)
        {
            ret = true;
        }
        else
        {
            LOGGER.debug("data: "+codeStr+" which is supposed to have a max length of: "+maxLen+", is of length: "+codeStr.length());     
        }
        
        return ret;
    }
    
    public  static char isLike(char data, String ... strings)
    {
        String strData = new String(new char[]{data});
        
        return isLike(strData, strings).charAt(0);
    }
    
    /**
     * will make note via logging of data items that do not 
     * conform to expected values. The data is not altered.
     * 
     * @param data
     * @param strings
     * @return
     */
    public  static String isLike(String data, String ... strings)
    {
        checkIsLike(data, strings);
        
        return data;
    }
    
    public  static boolean checkIsLike(String data, String ... strings)
    {
        boolean ret = false;
        
        if(data != null)
        {
            data = data.trim();
            
            // should match any ONE of the data examples
            
            for(String example : strings)
            {
                if(example.equals(data))
                {
                    ret = true;
                    break;
                }
            }
        }
        
        if(!ret)
        {
        	if(data != null && !data.isEmpty())
        	{
        		LOGGER.debug("data: "+data+" which is supposed to match one of: "+strings+", does NOT");
        	}
        }
        
        return ret;
    }
    
    /**
     * to be valid, the numeric string must be:
     *     1. the same number of digits as the format mask
     *     2. a parseable date
     *     3. not null
     * 
     * 
     * @param dateString
     * @param formatFlags
     * @return
     */
    
    public  static String checkValidDateType(String dateString, String formatFlags)
    {
        isValidDateType(dateString, formatFlags);
        
        return dateString;
    }
    
    public  static boolean isValidDateType(String dateString, String formatFlags)
    {
        boolean ret = false;
        
        SimpleDateFormat format = new SimpleDateFormat(formatFlags);

        try 
        {
            if((dateString != null) && (dateString.length() == formatFlags.length()))
            {
                Date d = format.parse(dateString);
                
                if(d != null)
                {
                    if (format.format(d).equals(dateString)) 
                    {
                       ret = true;
                    }
                }
            }
        }
        catch(ParseException pe) 
        {
            LOGGER.debug("ERROR: Cannot parse date \"" + dateString + "\"");
        }
        catch(Exception pe) 
        {
            LOGGER.debug("ERROR: Cannot parse date \"" + dateString + "\"");
        }
        
        if(!ret)
        {
            LOGGER.debug("date string: "+dateString+" which is supposed to format like: "+formatFlags+", does NOT");     
        }
        
        return ret;
    }
    
    /**
     * use this to evaluate HHMM types, which is a little more difficult
     * by using date parse techniques...
     * 
     * @param timeString
     * @return
     */
    
    public  static String checkValidTimeType(String timeString)
    {
        isValidTimeType(timeString);
        
        return timeString;
    }
    
    public  static boolean isValidTimeType(String timeString)
    {
        boolean ret = false;
        
        if((timeString != null) && (timeString.length() == 4))
        {
            if(isNumeric(timeString))
            {
                String hours = timeString.substring(0,2);
                String minutes = timeString.substring(2,timeString.length());
                
                int hour = Integer.parseInt(hours);
                int min = Integer.parseInt(minutes);
                
                
                // 24 hour clock is assumed...
                
                if(((min >= 0) && (min <= 59)) && 
                   ((hour >= 0) && (hour <= 23)))
                {
                    ret = true;
                }
            }
        }
        
        if(!ret)
        {
            LOGGER.debug("time string: "+timeString+", is NOT valid");     
        }
        
        return ret;
    }

	// Returns the first character in the given string
  public static char stringToSingleChar(String s)
	{
		if(s==null || s.length()<1) return 0;
		return s.charAt(0);				
	}
	
  // Returns the first character in the given string
  // Use with old versions of Java that don't convert automatically between char and Character
	public static Character stringToSingleCharacter(String s)
	{
		if(s==null || s.length()<1) return null;
		return new Character(s.charAt(0));				
	}

	 /**
   * Returns the first token from a string
   * 
   * @param inString - String containing tokens
   */
  public static String getFirstToken(String inString, String delimiter) throws IOException {
    // Return the first token in the string
    if (inString != null) {
      String[] tokens = inString.split(delimiter, -1);
      if (tokens.length > 0) {
        return tokens[0];
      } else {
        return null;
      }
    } else {
      return null;
    }
  }

}
