package gov.va.med.hac.edi.utils.jpa;

import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import java.io.File;
import java.math.BigDecimal;
import java.net.URL;


/**
 * <code>HibernateUtil</code> provides convienience methods for 
 * obtaining the instance of the Hibernate <code>SessionFactory</code>.
 */
public class HibernateUtil {
  final public static BigDecimal zeroBD = new BigDecimal("0");
  final public static BigDecimal hundredBD = new BigDecimal("100");

  /** the single instance of the <code>SessionFactory</code>. */
  static SessionFactory sessionFactory;

  /** private constructor to prevent instantiation of this utility Class. */
  private HibernateUtil() {
  }


  /**
   * Configures the <code>SessionFactory</code> using a default
   * hibernate configuration file.  This method expects that the configuration
   * file is named hibernate.cfg.xml and that this file is somewhere
   * in the classpath.
   *
   * @throws HibernateException 
   */
  public static synchronized void configure() {
    sessionFactory = new Configuration().configure().buildSessionFactory();
  }


  /**
   * Use the mappings and properties specified in the given application
   * file.  The format of the file is defined in 
   * <code>hibernate-configuration-3.0.dtd</code>.
   *
   * @param configfile - the configuration file from which you wish to load
   *               the configuration.
   * @throws HibernateException
   */
  public static synchronized void configure(File configfile) {
    if (configfile == null) {
      throw new HibernateException("Configuration file cannot be null.");
    }
    sessionFactory = new Configuration().configure(configfile).buildSessionFactory();
  }


  /**
   * Use the mappings and properties specified in the given document.  The
   * format of the document is defined in
   * <code>hibernate-configuration-3.0.dtd</code>.
   *
   * @param url the URL from which you wish to load the configuration.
   * @throws HibernateException
   */
  public static synchronized void configure(URL url) {
    if (url == null) {
      throw new HibernateException("Configuration url cannot be null.");
    }
    sessionFactory = new Configuration().configure(url).buildSessionFactory();
  }


  /**
   * Returns the instance of the Hibernate <code>SessionFactory</code>.
   *
   * @return the instance of the Hibernate <code>SessionFactory</code>.
   * @throws HibernateException if the <code>SessionFactory</code> is 
   *         closed.
   */
  public static synchronized SessionFactory getSessionFactory() {
    if (sessionFactory == null) {
      throw new HibernateException("The SessionFactory has not been configured.");
    }
    if (sessionFactory.isClosed()) {
      throw new HibernateException("SessionFactory is closed.");
    }
    return sessionFactory;
  }


  /**
   * Destroy this <code>SessionFactory</code> and release all resources.  It
   * is the responsiblity of the application to ensure that here are no
   * open <code>Sessions</code> before calling <code>shtdown()</code>.
   *
   * @throws HibernateException
   */
  public static synchronized void shutdown() {
    if (getSessionFactory().isClosed()) {
      return;
    }
    getSessionFactory().close();
  }
}
