package gov.va.med.hac.edi.regression;

import java.io.*;
import java.sql.*;

import java.util.Calendar;
import java.util.Properties;

import javax.naming.NamingException;

import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.FileUtils;
import gov.va.med.hac.edi.utils.jpa.StreamGobbler;

/*
 * 
 * @author HIPAA-Ready
 */
public abstract class RegressionBase {
  // These two are passed in
  public static Connection connection;
  public Properties properties;
  public boolean verbose = true;

  // These are pulled out of properties because they are used repeatedly
  String legacySchema;
  String legacySuffix;
  String newSchema;
  String newSuffix;
  String[][] tableDefs;

  // constructor for use with dependency injection
  // This pulls several things out of properties, since they are used repeatedly
  public RegressionBase(Connection inConnection, Properties inProperties, Boolean inVerbose) {
    connection = inConnection;
    properties = inProperties;
    if (inVerbose != null) {
      verbose = inVerbose;
    }

    // Pull out the schemas and suffixes now, since they are used repeatedly
    legacySchema = (String) properties.get("legacy.schema.name");
    newSchema = (String) properties.get("new.schema.name");
    legacySuffix = (String) properties.get("legacy.table.suffix");
    newSuffix = (String) properties.get("new.table.suffix");

    // Pull out the table names now, since they are used over and over
    String tableDefsProperty = (String) properties.get("table.names");
    if (tableDefsProperty != null && tableDefsProperty.length() > 0) {
      String[] tableDefStrings = tableDefsProperty.split(";");
      tableDefs = new String[tableDefStrings.length][6];
      for (int i = 0; i < tableDefStrings.length; i++) {
        String[] thisTableDefs = tableDefStrings[i].split("#");

        // Set the table name
        tableDefs[i][0] = thisTableDefs[0];

        // Set the "exclude columns like" parameter
        if (thisTableDefs.length >= 2 && thisTableDefs[1] != null
            && thisTableDefs[1].length() > 0) {
          tableDefs[i][1] = thisTableDefs[1];
        } else {
          tableDefs[i][1] = "NULL";
        }

        // Set the "exclude columns in" parameter
        if (thisTableDefs.length >= 3 && thisTableDefs[2] != null
            && thisTableDefs[2].length() > 0) {
          tableDefs[i][2] = thisTableDefs[2];
        } else {
          tableDefs[i][2] = "NULL";
        }

        // Set the "exclude audit columns" parameter
        if (thisTableDefs.length >= 4 && thisTableDefs[3] != null
            && thisTableDefs[3].length() > 0) {
          tableDefs[i][3] = thisTableDefs[3];
        } else {
          tableDefs[i][3] = "'Y'";
        }

        // Set the "exclude key columns" parameter
        if (thisTableDefs.length >= 5 && thisTableDefs[4] != null
            && thisTableDefs[4].length() > 0) {
          tableDefs[i][4] = thisTableDefs[4];
        } else {
          tableDefs[i][4] = "NULL";
        }

        // Set the "verbose" parameter
        if (thisTableDefs.length >= 6 && thisTableDefs[5] != null
            && thisTableDefs[5].length() > 0) {
          tableDefs[i][5] = thisTableDefs[5];
        } else {
          tableDefs[i][5] = "'FALSE'";
        }
      }

      if (verbose) {
        System.out.println("Table Defs:");
        for (int i = 0; i < tableDefs.length; i++) {
          for (int j = 0; j < tableDefs[i].length; j++)
            System.out.print(tableDefs[i][j] + ",");
          System.out.println("");
        }
      }
    }
    System.out.println("Leaving constructor");
  }

  public void clearTables(String inSchema, String inSuffix)
      throws IOException, SQLException {
    System.out.println("Executing Deletes from schema: " + inSchema + " with suffix: " + inSuffix);
    
    // Put together script to delete from all tables
    StringBuffer deletes = new StringBuffer();
    for (int i = 0; i < tableDefs.length; i++) {
      deletes.append("DELETE FROM ");
      if (inSchema != null && inSchema.length() > 0) {
        deletes.append(inSchema + ".");
      }
      deletes.append(tableDefs[i][0]);
      if (inSuffix != null && inSuffix.length() > 0) {
        deletes.append(inSuffix);
      }
      deletes.append(";\n");
    }

    if (verbose) {
      System.out.println(deletes.toString());
    }
    
    // Execute the script
    Reader deletesReader = new StringReader(deletes.toString());
    RegressionTestingUtils.runSqlScript(connection, deletesReader, true);
  }

  /**
   * 
   */
  public void dropTables(String inSchema, String inSuffix)
      throws IOException, SQLException {
    System.out.println("Executing Drops from schema: " + inSchema + " with suffix: " + inSuffix);
    
    // Put together script to drop all tables
    StringBuffer drops = new StringBuffer();
    for (int i = 0; i < tableDefs.length; i++) {
      drops.append("DROP TABLE ");
      if (inSchema != null && inSchema.length() > 0) {
        drops.append(inSchema + ".");
      }
      drops.append(tableDefs[i][0]);
      if (inSuffix != null && inSuffix.length() > 0) {
        drops.append(inSuffix);
      }
      drops.append(";\n");
    }

    // Execute the script
    Reader dropsReader = new StringReader(drops.toString());
    RegressionTestingUtils.runSqlScript(connection, dropsReader, false);

  }

  public void clearDirectory(String inDirectoryString) throws IOException {
    File directoryToDelete = new File(inDirectoryString);
    FileUtils.deleteFilesInDirectory(directoryToDelete);
  }

  /**
   * This method gets run before the run of the legacy code By default, it does nothing. But, it
   * might be overridden
   */
  public void setupBeforeLegacyRun() throws SQLException, IOException {
    boolean singleSchema = isSingleSchema();
    if (singleSchema) {
      setupBeforeLegacyRunSingleSchema();
    } else {
      setupBeforeLegacyRunTwoSchemas();
    }
  }

  /**
   * This method gets run between the run of the legacy code and the run of the new code By default,
   * it does nothing. But, it might be overridden
   */
  public void setupBetweenRuns() throws SQLException, IOException {
    boolean singleSchema = isSingleSchema();
    if (singleSchema) {
      setupBetweenRunsSingleSchema();
    } else {
      setupBetweenRunsTwoSchemas();
    }
  }

  protected boolean isSingleSchema() {
	String singleSchemaString = (String) properties.get("same.schema");
    boolean singleSchema = Boolean.parseBoolean(singleSchemaString);
	return singleSchema;
  }

  /**
   */
  public void setupBetweenRunsSingleSchema() throws SQLException, IOException {
  }

  /**
   */
  public void setupBetweenRunsTwoSchemas() throws SQLException, IOException {
  }

  /**
   */
  public void setupBeforeLegacyRunSingleSchema() throws SQLException,
      IOException {
  }

  /**
   */
  public void setupBeforeLegacyRunTwoSchemas() throws SQLException,
      IOException {
  }

  /**
   * Caution: be vary careful when running this.
   * It reads db connection properties from file DBConnections.properties
   * 
   * This method uses the java executable set by the "java.exec" property to run the legacy process
   * set by "legacy.process" . It uses the property "legacy.javaargs" to pass arguments, such as the
   * classpath, to java.exe. Finally, it passes the arguments set by "data.directory" to the
   * "legacy.process." Since NCPDPLoad loads data from files the "data.directory" argument is simply
   * a pointer to the directory that the legacy data lives in.
   * 
   * @throws IOException
   */
  public void runLegacyCode() throws IOException, InterruptedException {
    runLegacyCode("java.exec", "legacy.classpath",
        "legacy.process", "legacy.javaargs",
        "legacy.appargs");
  }

  /**
   * **Caution be vary careful when running this it reads the db connection properties from JDev
   * 
   * This method uses the java executable set by the property name passed in javaExecPropertyName to
   * run the legacy process set by the property set by legacyProcessPropertyName . It uses the
   * property set by legacyJavaArgsPropertyName to pass arguments, and property set by
   * legacyClasspathPropertyName to pass the class path. Finally, it passes the arguments set by the
   * property set by legacyAppArgsPropertyName to the legacyProcessPropertyName.
   * 
   * @throws IOException
   */
  protected void runLegacyCode(String javaExecPropertyName,
      String legacyClasspathPropertyName,
      String legacyProcessPropertyName,
      String legacyJavaArgsPropertyName,
      String legacyAppArgsPropertyName)
      throws IOException, InterruptedException {
    String javaExec = (String) properties.get(javaExecPropertyName);
    String legacyClasspath = (String) properties.get(legacyClasspathPropertyName);
    String legacyProcess = (String) properties.get(legacyProcessPropertyName);
    String legacyJavaArgs = (String) properties.get(legacyJavaArgsPropertyName);
    String legacyAppArgs = (String) properties.get(legacyAppArgsPropertyName);
    String command = javaExec + " " + legacyJavaArgs + " "
        + legacyClasspath + " " + legacyProcess + " " + legacyAppArgs;

    System.out.println(Calendar.getInstance().getTime() + " Executing command:" + command);

    // When this process kicks off, if there is an error, the process might
    // hang forever.
    // We set up threads to read and print the error stream and output
    // stream of the process
    // This keeps everything running smoothly, no hangs
    Process proc = Runtime.getRuntime().exec(command);

    // any error message?
    StreamGobbler errorGobbler = new StreamGobbler(proc.getErrorStream(), "ERROR");

    // any output?
    StreamGobbler outputGobbler = new StreamGobbler(proc.getInputStream(), "OUTPUT");

    // kick off the stream gobblers
    errorGobbler.start();
    outputGobbler.start();

    // Wait for the process to finish, and report its exit value
    int exitVal = proc.waitFor();
    System.out.println(Calendar.getInstance().getTime() + " ExitValue: " + exitVal);
  }

  /**
   */
  public void cloneTables(String inSourceSchema, String inSourceSuffix,
      String inTargetSchema, String inTargetSuffix)
      throws FileNotFoundException, SQLException, IOException {
    // Put together script to clone all tables
    StringBuffer clones = new StringBuffer();
    for (int i = 0; i < tableDefs.length; i++) {
      clones.append("CREATE TABLE ");
      if (inTargetSchema != null && inTargetSchema.length() > 0) {
        clones.append(inTargetSchema + ".");
      }
      clones.append(tableDefs[i][0]);
      if (inTargetSuffix != null && inTargetSuffix.length() > 0) {
        clones.append(inTargetSuffix);
      }

      clones.append(" AS SELECT * FROM ");
      if (inSourceSchema != null && inSourceSchema.length() > 0) {
        clones.append(inSourceSchema + ".");
      }
      clones.append(tableDefs[i][0]);
      if (inSourceSuffix != null && inSourceSuffix.length() > 0) {
        clones.append(inSourceSuffix);
      }
      clones.append(";\n");
    }

    // Execute the script
    Reader clonesReader = new StringReader(clones.toString());
    RegressionTestingUtils.runSqlScript(connection, clonesReader, false);

  }

  /**
   * This method executes the refactored code in this JVM. If the refactored process does not run
   * correctly you may want to convert this method to use a ProcessBuilder that sets up an isolated
   * environment.
   * 
   * @throws Exception
   */
  public abstract void runNewCode() throws IOException, NamingException,
      DbTransactionEnderException;

  // Run the PL/SQL-based regression procedure on the two sets of tables.
  public int compareTableLegacyToNew(String[] inTableDefs)
      throws SQLException {

    // Put together a SQL command to call a procedure to report on
    // differences
    StringBuffer diffQuery = new StringBuffer();
    diffQuery.append("SELECT ");
    diffQuery.append("REGRESSION_TESTING.COMPARE_TABLE_ROWS ('");

    // Put in name of legacy table
    if (legacySchema != null && legacySchema.length() > 0) {
      diffQuery.append(legacySchema + ".");
    }
    diffQuery.append(inTableDefs[0]);
    if (legacySuffix != null && legacySuffix.length() > 0) {
      diffQuery.append(legacySuffix);
    }

    diffQuery.append("','");

    // Put in name of new table
    if (newSchema != null && newSchema.length() > 0) {
      diffQuery.append(newSchema + ".");
    }
    diffQuery.append(inTableDefs[0]);
    if (newSuffix != null && newSuffix.length() > 0) {
      diffQuery.append(newSuffix);
    }

    // Exclude columns like
    diffQuery.append("', " + inTableDefs[1]);

    // Exclude columns in
    diffQuery.append(", " + inTableDefs[2]);

    // Exclude audit columns, yes or no
    diffQuery.append(", " + inTableDefs[3]);

    // Exclude columns ending with KEY
    diffQuery.append(", " + inTableDefs[4]);

    // Verbose output?
    diffQuery.append(", " + inTableDefs[5]);

    // Finish off the query
    diffQuery.append(" ) AS DIFF_COUNT FROM DUAL");

    // execute the query
    String diffQueryString = diffQuery.toString();
    System.out.println(diffQueryString);
    Statement stmt = connection.createStatement();
    ResultSet rs;
    rs = stmt.executeQuery(diffQueryString);

    // Get the result, the number of differences between the legacy and new
    // code base runs
    int numDiffs = -1;
    if (rs.next()) {
      numDiffs = rs.getInt("DIFF_COUNT");
    }

    // Report on any differences
    if (numDiffs != 0) {
      System.err.println(numDiffs + " differences for " + inTableDefs[0]);
    }

    return numDiffs;
  }

  public int compareAllTablesLegacyToNew() throws SQLException {
    int diffs_one_table = 0;
    int diffs_total = 0;
    for (int i = 0; i < tableDefs.length; i++) {
      diffs_one_table = compareTableLegacyToNew(tableDefs[i]);
      diffs_total += diffs_one_table;
    }

    return diffs_total;
  }

  /**
   * This method copies any files in the directory set by the "new.stagedatadir" property to the
   * directory set by the data.directory property. The intent of the method is to copy data files
   * required by the running process (the refactored process) into the input directory.
   * 
   * @throws Exception
   */
  public void copyFiles(String inSourceDirectoryString,
      String inTargetDirectoryString) throws IOException {
    File moveFrom = new File(inSourceDirectoryString);
    File moveTo = new File(inTargetDirectoryString);
    FileUtils.copyDirectory(moveFrom, moveTo);
  }

}
