package gov.va.med.hac.edi.regression;

import gov.va.med.hac.edi.utils.jpa.ScriptRunner;

import java.io.*;
import java.sql.*;
import java.util.*;

public class RegressionTestingUtils {
  
  /**
   * This method runs a sql script given a database connection.
   * The caller may specify whether a sql error should stop the processing of the script.
   * Note: This method uses code that was taken from the ibatis.apache.org project.
   * 
   * @param connection is the database connection to use
   * @param scriptStream is an input stream that will stream the sql script to execute
   * @param stopOnError indicates if an SQL error should stop the processing of the sql.
   * 
   * @throws Exception if it can't run the script or if an error occurs when stopOnError=true.
   */
  public static void runSqlScript(Connection connection, InputStream scriptStream,
      boolean stopOnError) {
    ScriptRunner scriptRunner = new ScriptRunner(connection);
    scriptRunner.setAutoCommit(false);
    scriptRunner.setStopOnError(stopOnError);
    scriptRunner.runScript(new InputStreamReader(scriptStream));
  }

  /**
   * This method runs a sql script given a database connection.
   * The caller may specify whether a sql error should stop the processing of the script.
   * Note: This method uses code that was taken from the ibatis.apache.org project.
   * 
   * @param connection is the database connection to use
   * @param scriptReader is a Reader that will read the sql script to execute
   * @param stopOnError indicates if an SQL error should stop the processing of the sql.
   * 
   * @throws Exception if it can't run the script or if an error occurs when stopOnError=true.
   */
  public static void runSqlScript(Connection connection, Reader scriptReader,
      boolean stopOnError) {
    ScriptRunner scriptRunner = new ScriptRunner(connection);
    scriptRunner.setAutoCommit(false);
    scriptRunner.setStopOnError(stopOnError);
    scriptRunner.runScript(scriptReader);
  }

  /**
   * This is a generic method that is used to find a resource (generally a
   * file) on the classpath.
   * 
   * @param resource
   *          the name of the file or resource to read.
   * @throws Exception
   *           if it can not read the resource for any reason.
   */
  public static InputStream getResourceAsStream(String resource, Class inClass) {
    InputStream scriptStream = inClass.getClassLoader().getResourceAsStream(resource);
    return scriptStream;
  }

  /**
   * 
   * @throws IOException if the properties file can not be read for any reason
   */
  public static Properties getPropertiesFromFile(String propertiesFileName, Class inClass) throws IOException {
    InputStream propertiesFile = RegressionTestingUtils.getResourceAsStream(propertiesFileName,
        inClass);
    Properties prop = new Properties();
    prop.load(propertiesFile);
    return prop;
  }

  /**
   * Reads the dbhibernate.properties file that is present on the classpath to
   * set up a JDBC database connection. It expects that the following
   * properties are set: "hibernate.connection.url"
   * "hibernate.connection.username" "hibernate.connection.password"
   * 
   * Note: The intention of this method is to read the dbhibernate.properties
   * file that is used by the refactored code to set up the connection. This
   * way, any scripts that delete files, drop or create _ORIG tables, etc.
   * will be run against the database that the refactored code is using.
   * 
   * @return a connection to the database.
   * @throws SQLException
   * @throws IOException
   * @throws FileNotFoundException
   */
  public static Connection getDatabaseConnection(Class inClass) throws SQLException,
      IOException, FileNotFoundException {
    DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
    Properties dbProps = getPropertiesFromFile("dbhibernate.properties", inClass);
    String url = (String) dbProps.get("hibernate.connection.url");
    String user = (String) dbProps.get("hibernate.connection.username");
    String password = (String) dbProps.get("hibernate.connection.password");
    Connection connection = DriverManager.getConnection(url, user, password);
    return connection;
  }

  // Parse a properties string
  public static String[] parsePropertiesString(String inDefinitions) {
    // The table names are separated by commas
    String[] tableNames = inDefinitions.split(",");

    return tableNames;
  }

}
