package gov.va.med.fee.tool;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.UserRoleUsages;
import gov.va.med.domain.fee.VaFacility;
import gov.va.med.domain.fee.VaFacilityType;
import gov.va.med.domain.fee.Visn;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.faces.model.SelectItem;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.core.SeamResourceBundle;
import org.jboss.seam.framework.EntityQuery;
import org.jboss.seam.security.Identity;

/**
 * The Class VaFacilityList.
 */
@Name("vaFacilityList")
@Restrict("#{s:hasRole('FEE_SUPERVISOR') || s:hasRole('FEE_VISN_PGM_MANAGER') || s:hasRole('ADMIN')}")
public class VaFacilityList extends EntityQuery<VaFacility> {
    /** The app user. */
    @In(value = "appUser", required = true, scope = ScopeType.SESSION)
    private AppUser appUser;

    /** The identity. */
    @In
    private Identity identity;

    /** sets the maximum number of results to be returned. */
    private static final int MAX_RESULTS = 15;

    /** The Constant EJBQL. */
    private static final String EJBQL = "select vaFacility from fVaFacility vaFacility";

    private static final Pattern WHERE_PATTERN = Pattern.compile("\\s(where)\\s", Pattern.CASE_INSENSITIVE);
    private static final Pattern ORDER_PATTERN = Pattern.compile("\\s(order)(\\s)+by\\s", Pattern.CASE_INSENSITIVE);

    /** The Constant RESTRICTIONS. */
    private static final String[] RESTRICTIONS = {
            "lower(vaFacility.vaFacilityCd) like concat(lower(#{vaFacilityList.vaFacility.vaFacilityCd}),'%')",
            "lower(vaFacility.shortName) like concat(lower(#{vaFacilityList.vaFacility.shortName}),'%')",
            "lower(vaFacility.vaFacilityName) like concat(lower(#{vaFacilityList.vaFacility.vaFacilityName}),'%')",
    };

    /** The va facility. */
    private final VaFacility vaFacility;

    /**
     * Instantiates a new va facility list.
     */
    public VaFacilityList() {
        setEjbql(EJBQL);
        setRestrictionExpressionStrings(Arrays.asList(RESTRICTIONS));
        setMaxResults(MAX_RESULTS);
        vaFacility = new VaFacility();
        vaFacility.setVaFacilityType(new VaFacilityType());
    }

    /**
     * Gets the va facility.
     *
     * @return the va facility
     */
    public VaFacility getVaFacility() {
        return vaFacility;
    }

    @Override
    protected String getRenderedEjbql() {
        String query = super.getRenderedEjbql();
        Matcher orderMatcher = ORDER_PATTERN.matcher(query);
        int orderLoc = orderMatcher.find() ? orderMatcher.start(1) : query.length();
        String orderClause = query.substring(orderLoc, query.length());
        query = restrictSearch(query.substring(0, orderLoc));
        query = new StringBuilder(query).append(" ").append(orderClause).toString();
        return query;
    }

    @Override
    protected String getCountEjbql() {
        String query = super.getCountEjbql();
        return query;
    }

    @Override
    protected final String getPersistenceContextName() {
       return "FPPSEntityManager";
    }

    /**
     * Admin is not restricted to any stations.  VISN Manger are limited to their
     * VISN while supervisors have access to assigned stations.
     */
    private String restrictSearch(String query) {
        // admin is not restricted to anything
        if (identity.hasRole("ADMIN")) {
            return query;
        }

        if (identity.hasRole("FEE_VISN_PGM_MANAGER")) {
            return restrictSearchForVisnMgr(query);
        }

        return restrictSearchForSupervisor(query);
    }

    @SuppressWarnings("unchecked")
    private String restrictSearchForVisnMgr(String query) {
        StringBuilder builder = new StringBuilder().append(query);
        if (WHERE_PATTERN.matcher(builder).find()) {
           builder.append(" and ");
        } else {
           builder.append(" where ");
        }

        String q = "select distinct (v.visn_id_cd) from Va_Facility v where v.va_facility_cd in "
            + getFacilities();

        List<Object> list = getEntityManager().createNativeQuery(q).getResultList();

        String visnList = list.toString();
        visnList = visnList.replace('[', '(');
        visnList = visnList.replace(']', ')');

        builder.append("vaFacility.visn.visnIdCd in ")
            .append(visnList);

        return builder.toString();
    }

    private String restrictSearchForSupervisor(String query) {
        StringBuilder builder = new StringBuilder().append(query);
        if (WHERE_PATTERN.matcher(builder).find()) {
           builder.append(" and ");
        } else {
           builder.append(" where ");
        }

        // supervisors are limited to stations
        builder.append("vaFacility.vaFacilityCd in ").append(getFacilities());
        return builder.toString();
    }

    private String getFacilities() {
        StringBuilder facs = new StringBuilder("(");
        Set<UserRoleUsages> userRoleUsages = appUser.getUserRoleUsages();
        boolean first = true;
        for (UserRoleUsages urs : userRoleUsages) {
            if (!first) {
                facs.append(",");
            }
            facs.append("'")
                .append(urs.getVaFacility().getVaFacilityCd())
                .append("'");
            first = false;
        }
        facs.append(")");

        return facs.toString();
    }

    /**
     * Gets the facility type list for selection.
     *
     * @return the facility type list
     */
    @SuppressWarnings("unchecked")
    @Factory(value = "vaFacilityFacilityTypeList", scope = ScopeType.PAGE)
    public List<SelectItem> getFacilityTypeList() {
        List<VaFacilityType> results = getEntityManager().createQuery("from fVaFacilityType")
            .getResultList();

        List<SelectItem> outList = new ArrayList<SelectItem>();
        String selectText = SeamResourceBundle.getBundle().getString("gov.va.med.fee.SELECT_FACILITY_TYPE");
        outList.add(new SelectItem(null, selectText));

        for (VaFacilityType v : results) {
            outList.add(new SelectItem(v.getVaTypeCd(),
                    v.getVaTypeCd() +
                    (v.getDescription() != null ?  " -  " + v.getDescription() : "")));
        }
        return outList;
    }

    /**
     * Gets the visn list.
     *
     * @return the visn list
     */
    @SuppressWarnings("unchecked")
    @Factory(value = "vaFacilityVisnList", scope = ScopeType.PAGE)
    public List<SelectItem> getVisnList() {
        StringBuilder builder = new StringBuilder().append("select * from visn v1");
        Query query = null;
        if (identity.hasRole("FEE_VISN_PGM_MANAGER")) {
            builder.append(" where v1.visn_id_cd in (");
            builder.append("select distinct (v.visn_id_cd) from Va_Facility v where v.va_facility_cd in ");
            builder.append(getFacilities());
            builder.append(")");
            query = getEntityManager().createNativeQuery(builder.toString(), Visn.class);
        } else {
            query = getEntityManager().createNativeQuery(builder.toString(), Visn.class);
        }

        List<Visn> results = query.getResultList();

        List<SelectItem> outList = new ArrayList<SelectItem>();
        String selectText = SeamResourceBundle.getBundle().getString("gov.va.med.fee.SELECT_VISN");
        outList.add(new SelectItem("-1", selectText));

        for (Visn v : results) {
            outList.add(new SelectItem(v.getVisnIdCd(),
                    v.getVisnIdCd() +
                    (v.getDescription() != null ? " - " + v.getDescription() : "")));
        }
        return outList;
    }
}
